/*
 * Copyright (c) 2016, 2017 Oracle and/or its affiliates. All rights reserved. This
 * code is released under a tri EPL/GPL/LGPL license. You can use it,
 * redistribute it and/or modify it under the terms of the:
 *
 * Eclipse Public License version 1.0
 * GNU General Public License version 2
 * GNU Lesser General Public License version 2.1
 *
 * This file contains configuration values translated from Rubinius.
 *
 * Copyright (c) 2007-2014, Evan Phoenix and contributors
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 * * Redistributions of source code must retain the above copyright notice, this
 *   list of conditions and the following disclaimer.
 * * Redistributions in binary form must reproduce the above copyright notice
 *   this list of conditions and the following disclaimer in the documentation
 *   and/or other materials provided with the distribution.
 * * Neither the name of Rubinius nor the names of its contributors
 *   may be used to endorse or promote products derived from this software
 *   without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
 * CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
package org.truffleruby.platform.solaris;

import org.truffleruby.RubyContext;
import org.truffleruby.platform.DefaultNativeConfiguration;
import org.truffleruby.platform.NativeConfiguration;

public class SolarisSparcV9NativeConfiguration extends DefaultNativeConfiguration {

    public static void load(NativeConfiguration configuration, RubyContext context) {
        // Generated from tool/generate-native-config.rb on sparcv9-solaris2.11
        configuration.config("platform.sigaction.sizeof", 32);
        configuration.config("platform.sigaction.sa_handler.offset", 8);
        configuration.config("platform.sigaction.sa_handler.size", 8);
        configuration.config("platform.sigaction.sa_handler.type", string(context, "pointer"));
        configuration.config("platform.addrinfo.sizeof", 48);
        configuration.config("platform.addrinfo.ai_flags.offset", 0);
        configuration.config("platform.addrinfo.ai_flags.size", 4);
        configuration.config("platform.addrinfo.ai_flags.type", string(context, "int"));
        configuration.config("platform.addrinfo.ai_family.offset", 4);
        configuration.config("platform.addrinfo.ai_family.size", 4);
        configuration.config("platform.addrinfo.ai_family.type", string(context, "int"));
        configuration.config("platform.addrinfo.ai_socktype.offset", 8);
        configuration.config("platform.addrinfo.ai_socktype.size", 4);
        configuration.config("platform.addrinfo.ai_socktype.type", string(context, "int"));
        configuration.config("platform.addrinfo.ai_protocol.offset", 12);
        configuration.config("platform.addrinfo.ai_protocol.size", 4);
        configuration.config("platform.addrinfo.ai_protocol.type", string(context, "int"));
        configuration.config("platform.addrinfo.ai_addrlen.offset", 20);
        configuration.config("platform.addrinfo.ai_addrlen.size", 4);
        configuration.config("platform.addrinfo.ai_addrlen.type", string(context, "int"));
        configuration.config("platform.addrinfo.ai_addr.offset", 32);
        configuration.config("platform.addrinfo.ai_addr.size", 8);
        configuration.config("platform.addrinfo.ai_addr.type", string(context, "pointer"));
        configuration.config("platform.addrinfo.ai_canonname.offset", 24);
        configuration.config("platform.addrinfo.ai_canonname.size", 8);
        configuration.config("platform.addrinfo.ai_canonname.type", string(context, "string"));
        configuration.config("platform.addrinfo.ai_next.offset", 40);
        configuration.config("platform.addrinfo.ai_next.size", 8);
        configuration.config("platform.addrinfo.ai_next.type", string(context, "pointer"));
        configuration.config("platform.ifaddrs.sizeof", 56);
        configuration.config("platform.ifaddrs.ifa_next.offset", 0);
        configuration.config("platform.ifaddrs.ifa_next.size", 8);
        configuration.config("platform.ifaddrs.ifa_next.type", string(context, "pointer"));
        configuration.config("platform.ifaddrs.ifa_name.offset", 8);
        configuration.config("platform.ifaddrs.ifa_name.size", 8);
        configuration.config("platform.ifaddrs.ifa_name.type", string(context, "string"));
        configuration.config("platform.ifaddrs.ifa_flags.offset", 16);
        configuration.config("platform.ifaddrs.ifa_flags.size", 8);
        configuration.config("platform.ifaddrs.ifa_flags.type", string(context, "int"));
        configuration.config("platform.ifaddrs.ifa_addr.offset", 24);
        configuration.config("platform.ifaddrs.ifa_addr.size", 8);
        configuration.config("platform.ifaddrs.ifa_addr.type", string(context, "pointer"));
        configuration.config("platform.ifaddrs.ifa_netmask.offset", 32);
        configuration.config("platform.ifaddrs.ifa_netmask.size", 8);
        configuration.config("platform.ifaddrs.ifa_netmask.type", string(context, "pointer"));
        configuration.config("platform.ifaddrs.ifa_broadaddr.offset", 40);
        configuration.config("platform.ifaddrs.ifa_broadaddr.size", 8);
        configuration.config("platform.ifaddrs.ifa_broadaddr.type", string(context, "pointer"));
        configuration.config("platform.ifaddrs.ifa_dstaddr.offset", 40);
        configuration.config("platform.ifaddrs.ifa_dstaddr.size", 8);
        configuration.config("platform.ifaddrs.ifa_dstaddr.type", string(context, "pointer"));
        configuration.config("platform.sockaddr.sizeof", 16);
        configuration.config("platform.sockaddr.sa_data.offset", 2);
        configuration.config("platform.sockaddr.sa_data.size", 14);
        configuration.config("platform.sockaddr.sa_data.type", string(context, "char_array"));
        configuration.config("platform.sockaddr.sa_family.offset", 0);
        configuration.config("platform.sockaddr.sa_family.size", 2);
        configuration.config("platform.sockaddr.sa_family.type", string(context, "sa_family_t"));
        configuration.config("platform.sockaddr_in.sizeof", 16);
        configuration.config("platform.sockaddr_in.sin_family.offset", 0);
        configuration.config("platform.sockaddr_in.sin_family.size", 2);
        configuration.config("platform.sockaddr_in.sin_family.type", string(context, "sa_family_t"));
        configuration.config("platform.sockaddr_in.sin_port.offset", 2);
        configuration.config("platform.sockaddr_in.sin_port.size", 2);
        configuration.config("platform.sockaddr_in.sin_port.type", string(context, "ushort"));
        configuration.config("platform.sockaddr_in.sin_addr.offset", 4);
        configuration.config("platform.sockaddr_in.sin_addr.size", 4);
        configuration.config("platform.sockaddr_in.sin_zero.offset", 8);
        configuration.config("platform.sockaddr_in.sin_zero.size", 8);
        configuration.config("platform.sockaddr_in.sin_zero.type", string(context, "char_array"));
        configuration.config("platform.sockaddr_in6.sizeof", 32);
        configuration.config("platform.sockaddr_in6.sin6_family.offset", 0);
        configuration.config("platform.sockaddr_in6.sin6_family.size", 2);
        configuration.config("platform.sockaddr_in6.sin6_family.type", string(context, "sa_family_t"));
        configuration.config("platform.sockaddr_in6.sin6_port.offset", 2);
        configuration.config("platform.sockaddr_in6.sin6_port.size", 2);
        configuration.config("platform.sockaddr_in6.sin6_port.type", string(context, "ushort"));
        configuration.config("platform.sockaddr_in6.sin6_flowinfo.offset", 4);
        configuration.config("platform.sockaddr_in6.sin6_flowinfo.size", 4);
        configuration.config("platform.sockaddr_in6.sin6_addr.offset", 8);
        configuration.config("platform.sockaddr_in6.sin6_addr.size", 16);
        configuration.config("platform.sockaddr_in6.sin6_addr.type", string(context, "char_array"));
        configuration.config("platform.sockaddr_in6.sin6_scope_id.offset", 24);
        configuration.config("platform.sockaddr_in6.sin6_scope_id.size", 4);
        configuration.config("platform.sockaddr_un.sizeof", 110);
        configuration.config("platform.sockaddr_un.sun_family.offset", 0);
        configuration.config("platform.sockaddr_un.sun_family.size", 2);
        configuration.config("platform.sockaddr_un.sun_family.type", string(context, "sa_family_t"));
        configuration.config("platform.sockaddr_un.sun_path.offset", 2);
        configuration.config("platform.sockaddr_un.sun_path.size", 108);
        configuration.config("platform.sockaddr_un.sun_path.type", string(context, "char_array"));
        configuration.config("platform.hostent.sizeof", 32);
        configuration.config("platform.hostent.h_name.offset", 0);
        configuration.config("platform.hostent.h_name.size", 8);
        configuration.config("platform.hostent.h_name.type", string(context, "string"));
        configuration.config("platform.hostent.h_aliases.offset", 8);
        configuration.config("platform.hostent.h_aliases.size", 8);
        configuration.config("platform.hostent.h_aliases.type", string(context, "pointer"));
        configuration.config("platform.hostent.h_addrtype.offset", 16);
        configuration.config("platform.hostent.h_addrtype.size", 4);
        configuration.config("platform.hostent.h_addrtype.type", string(context, "int"));
        configuration.config("platform.hostent.h_length.offset", 20);
        configuration.config("platform.hostent.h_length.size", 4);
        configuration.config("platform.hostent.h_length.type", string(context, "int"));
        configuration.config("platform.hostent.h_addr_list.offset", 24);
        configuration.config("platform.hostent.h_addr_list.size", 8);
        configuration.config("platform.hostent.h_addr_list.type", string(context, "pointer"));
        configuration.config("platform.linger.sizeof", 8);
        configuration.config("platform.linger.l_onoff.offset", 0);
        configuration.config("platform.linger.l_onoff.size", 4);
        configuration.config("platform.linger.l_onoff.type", string(context, "int"));
        configuration.config("platform.linger.l_linger.offset", 4);
        configuration.config("platform.linger.l_linger.size", 4);
        configuration.config("platform.linger.l_linger.type", string(context, "int"));
        configuration.config("platform.iovec.sizeof", 16);
        configuration.config("platform.iovec.iov_base.offset", 0);
        configuration.config("platform.iovec.iov_base.size", 8);
        configuration.config("platform.iovec.iov_base.type", string(context, "pointer"));
        configuration.config("platform.iovec.iov_len.offset", 8);
        configuration.config("platform.iovec.iov_len.size", 8);
        configuration.config("platform.iovec.iov_len.type", string(context, "size_t"));
        configuration.config("platform.msghdr.sizeof", 48);
        configuration.config("platform.msghdr.msg_name.offset", 0);
        configuration.config("platform.msghdr.msg_name.size", 8);
        configuration.config("platform.msghdr.msg_name.type", string(context, "pointer"));
        configuration.config("platform.msghdr.msg_namelen.offset", 8);
        configuration.config("platform.msghdr.msg_namelen.size", 4);
        configuration.config("platform.msghdr.msg_namelen.type", string(context, "int"));
        configuration.config("platform.msghdr.msg_iov.offset", 16);
        configuration.config("platform.msghdr.msg_iov.size", 8);
        configuration.config("platform.msghdr.msg_iov.type", string(context, "pointer"));
        configuration.config("platform.msghdr.msg_iovlen.offset", 24);
        configuration.config("platform.msghdr.msg_iovlen.size", 4);
        configuration.config("platform.msghdr.msg_iovlen.type", string(context, "size_t"));
        configuration.config("platform.msghdr.msg_control.offset", 32);
        configuration.config("platform.msghdr.msg_control.size", 8);
        configuration.config("platform.msghdr.msg_control.type", string(context, "pointer"));
        configuration.config("platform.msghdr.msg_controllen.offset", 40);
        configuration.config("platform.msghdr.msg_controllen.size", 4);
        configuration.config("platform.msghdr.msg_controllen.type", string(context, "size_t"));
        configuration.config("platform.msghdr.msg_flags.offset", 44);
        configuration.config("platform.msghdr.msg_flags.size", 4);
        configuration.config("platform.msghdr.msg_flags.type", string(context, "int"));
        configuration.config("platform.servent.sizeof", 32);
        configuration.config("platform.servent.s_name.offset", 0);
        configuration.config("platform.servent.s_name.size", 8);
        configuration.config("platform.servent.s_name.type", string(context, "pointer"));
        configuration.config("platform.servent.s_aliases.offset", 8);
        configuration.config("platform.servent.s_aliases.size", 8);
        configuration.config("platform.servent.s_aliases.type", string(context, "pointer"));
        configuration.config("platform.servent.s_port.offset", 16);
        configuration.config("platform.servent.s_port.size", 4);
        configuration.config("platform.servent.s_port.type", string(context, "int"));
        configuration.config("platform.servent.s_proto.offset", 24);
        configuration.config("platform.servent.s_proto.size", 8);
        configuration.config("platform.servent.s_proto.type", string(context, "pointer"));
        configuration.config("platform.errno.EPERM", 1);
        configuration.config("platform.errno.ENOENT", 2);
        configuration.config("platform.errno.ESRCH", 3);
        configuration.config("platform.errno.EINTR", 4);
        configuration.config("platform.errno.EIO", 5);
        configuration.config("platform.errno.ENXIO", 6);
        configuration.config("platform.errno.E2BIG", 7);
        configuration.config("platform.errno.ENOEXEC", 8);
        configuration.config("platform.errno.EBADF", 9);
        configuration.config("platform.errno.ECHILD", 10);
        configuration.config("platform.errno.EDEADLK", 45);
        configuration.config("platform.errno.ENOMEM", 12);
        configuration.config("platform.errno.EACCES", 13);
        configuration.config("platform.errno.EFAULT", 14);
        configuration.config("platform.errno.ENOTBLK", 15);
        configuration.config("platform.errno.EBUSY", 16);
        configuration.config("platform.errno.EEXIST", 17);
        configuration.config("platform.errno.EXDEV", 18);
        configuration.config("platform.errno.ENODEV", 19);
        configuration.config("platform.errno.ENOTDIR", 20);
        configuration.config("platform.errno.EISDIR", 21);
        configuration.config("platform.errno.EINVAL", 22);
        configuration.config("platform.errno.ENFILE", 23);
        configuration.config("platform.errno.EMFILE", 24);
        configuration.config("platform.errno.ENOTTY", 25);
        configuration.config("platform.errno.ETXTBSY", 26);
        configuration.config("platform.errno.EFBIG", 27);
        configuration.config("platform.errno.ENOSPC", 28);
        configuration.config("platform.errno.ESPIPE", 29);
        configuration.config("platform.errno.EROFS", 30);
        configuration.config("platform.errno.EMLINK", 31);
        configuration.config("platform.errno.EPIPE", 32);
        configuration.config("platform.errno.EDOM", 33);
        configuration.config("platform.errno.ERANGE", 34);
        configuration.config("platform.errno.EWOULDBLOCK", 11);
        configuration.config("platform.errno.EAGAIN", 11);
        configuration.config("platform.errno.EINPROGRESS", 150);
        configuration.config("platform.errno.EALREADY", 149);
        configuration.config("platform.errno.ENOTSOCK", 95);
        configuration.config("platform.errno.EDESTADDRREQ", 96);
        configuration.config("platform.errno.EMSGSIZE", 97);
        configuration.config("platform.errno.EPROTOTYPE", 98);
        configuration.config("platform.errno.ENOPROTOOPT", 99);
        configuration.config("platform.errno.EPROTONOSUPPORT", 120);
        configuration.config("platform.errno.ESOCKTNOSUPPORT", 121);
        configuration.config("platform.errno.EOPNOTSUPP", 122);
        configuration.config("platform.errno.EPFNOSUPPORT", 123);
        configuration.config("platform.errno.EAFNOSUPPORT", 124);
        configuration.config("platform.errno.EADDRINUSE", 125);
        configuration.config("platform.errno.EADDRNOTAVAIL", 126);
        configuration.config("platform.errno.ENETDOWN", 127);
        configuration.config("platform.errno.ENETUNREACH", 128);
        configuration.config("platform.errno.ENETRESET", 129);
        configuration.config("platform.errno.ECONNABORTED", 130);
        configuration.config("platform.errno.ECONNRESET", 131);
        configuration.config("platform.errno.ENOBUFS", 132);
        configuration.config("platform.errno.EISCONN", 133);
        configuration.config("platform.errno.ENOTCONN", 134);
        configuration.config("platform.errno.ESHUTDOWN", 143);
        configuration.config("platform.errno.ETOOMANYREFS", 144);
        configuration.config("platform.errno.ETIMEDOUT", 145);
        configuration.config("platform.errno.ECONNREFUSED", 146);
        configuration.config("platform.errno.ELOOP", 90);
        configuration.config("platform.errno.ENAMETOOLONG", 78);
        configuration.config("platform.errno.EHOSTDOWN", 147);
        configuration.config("platform.errno.EHOSTUNREACH", 148);
        configuration.config("platform.errno.ENOTEMPTY", 93);
        configuration.config("platform.errno.EUSERS", 94);
        configuration.config("platform.errno.EDQUOT", 49);
        configuration.config("platform.errno.ESTALE", 151);
        configuration.config("platform.errno.EREMOTE", 66);
        configuration.config("platform.errno.ENOLCK", 46);
        configuration.config("platform.errno.ENOSYS", 89);
        configuration.config("platform.errno.EOVERFLOW", 79);
        configuration.config("platform.errno.EIDRM", 36);
        configuration.config("platform.errno.ENOMSG", 35);
        configuration.config("platform.errno.EILSEQ", 88);
        configuration.config("platform.errno.EBADMSG", 77);
        configuration.config("platform.errno.EMULTIHOP", 74);
        configuration.config("platform.errno.ENODATA", 61);
        configuration.config("platform.errno.ENOLINK", 67);
        configuration.config("platform.errno.ENOSR", 63);
        configuration.config("platform.errno.ENOSTR", 60);
        configuration.config("platform.errno.EPROTO", 71);
        configuration.config("platform.errno.ETIME", 62);
        configuration.config("platform.limits.NSIG", 73);
        configuration.config("platform.langinfo.CODESET", 49);
        configuration.config("platform.file.FNM_CASEFOLD", 8);
        configuration.config("platform.file.FNM_NOESCAPE", 2);
        configuration.config("platform.file.FNM_PATHNAME", 1);
        configuration.config("platform.file.O_RDONLY", 0);
        configuration.config("platform.file.O_WRONLY", 1);
        configuration.config("platform.file.O_RDWR", 2);
        configuration.config("platform.file.O_ACCMODE", 6291459);
        configuration.config("platform.file.O_CREAT", 256);
        configuration.config("platform.file.O_EXCL", 1024);
        configuration.config("platform.file.O_NOCTTY", 2048);
        configuration.config("platform.file.O_TRUNC", 512);
        configuration.config("platform.file.O_APPEND", 8);
        configuration.config("platform.file.O_NONBLOCK", 128);
        configuration.config("platform.file.O_NDELAY", 4);
        configuration.config("platform.file.O_SYNC", 16);
        configuration.config("platform.file.S_IRUSR", 256);
        configuration.config("platform.file.S_IWUSR", 128);
        configuration.config("platform.file.S_IXUSR", 64);
        configuration.config("platform.file.S_IRGRP", 32);
        configuration.config("platform.file.S_IWGRP", 16);
        configuration.config("platform.file.S_IXGRP", 8);
        configuration.config("platform.file.S_IROTH", 4);
        configuration.config("platform.file.S_IWOTH", 2);
        configuration.config("platform.file.S_IXOTH", 1);
        configuration.config("platform.file.S_IFMT", 61440);
        configuration.config("platform.file.S_IFIFO", 4096);
        configuration.config("platform.file.S_IFCHR", 8192);
        configuration.config("platform.file.S_IFDIR", 16384);
        configuration.config("platform.file.S_IFBLK", 24576);
        configuration.config("platform.file.S_IFREG", 32768);
        configuration.config("platform.file.S_IFLNK", 40960);
        configuration.config("platform.file.S_IFSOCK", 49152);
        configuration.config("platform.file.S_ISUID", 2048);
        configuration.config("platform.file.S_ISGID", 1024);
        configuration.config("platform.file.S_ISVTX", 512);
        configuration.config("platform.io.SEEK_SET", 0);
        configuration.config("platform.io.SEEK_CUR", 1);
        configuration.config("platform.io.SEEK_END", 2);
        configuration.config("platform.fcntl.F_GETFL", 3);
        configuration.config("platform.fcntl.F_SETFL", 4);
        configuration.config("platform.fcntl.F_DUPFD", 0);
        configuration.config("platform.fcntl.F_GETFD", 1);
        configuration.config("platform.fcntl.F_SETFD", 2);
        configuration.config("platform.fcntl.FD_CLOEXEC", 1);
        configuration.config("platform.fcntl.F_GETOWN", 23);
        configuration.config("platform.fcntl.F_SETOWN", 24);
        configuration.config("platform.fcntl.F_GETLK", 14);
        configuration.config("platform.fcntl.F_SETLK", 6);
        configuration.config("platform.fcntl.F_SETLKW", 7);
        configuration.config("platform.fcntl.F_RDLCK", 1);
        configuration.config("platform.fcntl.F_UNLCK", 3);
        configuration.config("platform.fcntl.F_WRLCK", 2);
        configuration.config("platform.socket.AF_APPLETALK", 16);
        configuration.config("platform.socket.PF_APPLETALK", 16);
        configuration.config("platform.socket.AF_INET", 2);
        configuration.config("platform.socket.PF_INET", 2);
        configuration.config("platform.socket.AF_INET6", 26);
        configuration.config("platform.socket.PF_INET6", 26);
        configuration.config("platform.socket.AF_IPX", 23);
        configuration.config("platform.socket.PF_IPX", 23);
        configuration.config("platform.socket.AF_LOCAL", 1);
        configuration.config("platform.socket.PF_LOCAL", 1);
        configuration.config("platform.socket.AF_MAX", 33);
        configuration.config("platform.socket.PF_MAX", 33);
        configuration.config("platform.socket.AF_PACKET", 32);
        configuration.config("platform.socket.PF_PACKET", 32);
        configuration.config("platform.socket.AF_ROUTE", 24);
        configuration.config("platform.socket.PF_ROUTE", 24);
        configuration.config("platform.socket.AF_SNA", 11);
        configuration.config("platform.socket.PF_SNA", 11);
        configuration.config("platform.socket.AF_UNIX", 1);
        configuration.config("platform.socket.PF_UNIX", 1);
        configuration.config("platform.socket.AF_UNSPEC", 0);
        configuration.config("platform.socket.PF_UNSPEC", 0);
        configuration.config("platform.socket.PF_KEY", 27);
        configuration.config("platform.socket.AI_ADDRCONFIG", 4);
        configuration.config("platform.socket.AI_ALL", 2);
        configuration.config("platform.socket.AI_CANONNAME", 16);
        configuration.config("platform.socket.AI_NUMERICHOST", 32);
        configuration.config("platform.socket.AI_NUMERICSERV", 64);
        configuration.config("platform.socket.AI_PASSIVE", 8);
        configuration.config("platform.socket.AI_V4MAPPED", 1);
        configuration.config("platform.socket.EAI_ADDRFAMILY", 1);
        configuration.config("platform.socket.EAI_AGAIN", 2);
        configuration.config("platform.socket.EAI_BADFLAGS", 3);
        configuration.config("platform.socket.EAI_FAIL", 4);
        configuration.config("platform.socket.EAI_FAMILY", 5);
        configuration.config("platform.socket.EAI_MEMORY", 6);
        configuration.config("platform.socket.EAI_NODATA", 7);
        configuration.config("platform.socket.EAI_NONAME", 8);
        configuration.config("platform.socket.EAI_OVERFLOW", 12);
        configuration.config("platform.socket.EAI_SERVICE", 9);
        configuration.config("platform.socket.EAI_SOCKTYPE", 10);
        configuration.config("platform.socket.EAI_SYSTEM", 11);
        configuration.config("platform.socket.IFF_ALLMULTI", 512);
        configuration.config("platform.socket.IFF_BROADCAST", 2);
        configuration.config("platform.socket.IFF_DEBUG", 4);
        configuration.config("platform.socket.IFF_LOOPBACK", 8);
        configuration.config("platform.socket.IFF_MULTICAST", 2048);
        configuration.config("platform.socket.IFF_NOARP", 128);
        configuration.config("platform.socket.IFF_NOTRAILERS", 32);
        configuration.config("platform.socket.IFF_POINTOPOINT", 16);
        configuration.config("platform.socket.IFF_PROMISC", 256);
        configuration.config("platform.socket.IFF_RUNNING", 64);
        configuration.config("platform.socket.IFF_UP", 1);
        configuration.config("platform.socket.IF_NAMESIZE", 32);
        configuration.config("platform.socket.INADDR_ALLHOSTS_GROUP", 3758096385L);
        configuration.config("platform.socket.INADDR_ANY", 0);
        configuration.config("platform.socket.INADDR_BROADCAST", 4294967295L);
        configuration.config("platform.socket.INADDR_LOOPBACK", 2130706433);
        configuration.config("platform.socket.INADDR_MAX_LOCAL_GROUP", 3758096639L);
        configuration.config("platform.socket.INADDR_NONE", 4294967295L);
        configuration.config("platform.socket.INADDR_UNSPEC_GROUP", 3758096384L);
        configuration.config("platform.socket.INET6_ADDRSTRLEN", 46);
        configuration.config("platform.socket.INET_ADDRSTRLEN", 16);
        configuration.config("platform.socket.IPPORT_RESERVED", 1024);
        configuration.config("platform.socket.IPPORT_USERRESERVED", 5000);
        configuration.config("platform.socket.IPPROTO_AH", 51);
        configuration.config("platform.socket.IPPROTO_DSTOPTS", 60);
        configuration.config("platform.socket.IPPROTO_EGP", 8);
        configuration.config("platform.socket.IPPROTO_ESP", 50);
        configuration.config("platform.socket.IPPROTO_FRAGMENT", 44);
        configuration.config("platform.socket.IPPROTO_HOPOPTS", 0);
        configuration.config("platform.socket.IPPROTO_ICMP", 1);
        configuration.config("platform.socket.IPPROTO_ICMPV6", 58);
        configuration.config("platform.socket.IPPROTO_IDP", 22);
        configuration.config("platform.socket.IPPROTO_IGMP", 2);
        configuration.config("platform.socket.IPPROTO_IP", 0);
        configuration.config("platform.socket.IPPROTO_IPV6", 41);
        configuration.config("platform.socket.IPPROTO_NONE", 59);
        configuration.config("platform.socket.IPPROTO_PUP", 12);
        configuration.config("platform.socket.IPPROTO_RAW", 255);
        configuration.config("platform.socket.IPPROTO_ROUTING", 43);
        configuration.config("platform.socket.IPPROTO_TCP", 6);
        configuration.config("platform.socket.IPPROTO_UDP", 17);
        configuration.config("platform.socket.IPV6_CHECKSUM", 24);
        configuration.config("platform.socket.IPV6_DONTFRAG", 33);
        configuration.config("platform.socket.IPV6_DSTOPTS", 15);
        configuration.config("platform.socket.IPV6_HOPLIMIT", 12);
        configuration.config("platform.socket.IPV6_HOPOPTS", 14);
        configuration.config("platform.socket.IPV6_JOIN_GROUP", 9);
        configuration.config("platform.socket.IPV6_LEAVE_GROUP", 10);
        configuration.config("platform.socket.IPV6_MULTICAST_HOPS", 7);
        configuration.config("platform.socket.IPV6_MULTICAST_IF", 6);
        configuration.config("platform.socket.IPV6_MULTICAST_LOOP", 8);
        configuration.config("platform.socket.IPV6_NEXTHOP", 13);
        configuration.config("platform.socket.IPV6_PATHMTU", 37);
        configuration.config("platform.socket.IPV6_PKTINFO", 11);
        configuration.config("platform.socket.IPV6_RECVDSTOPTS", 40);
        configuration.config("platform.socket.IPV6_RECVHOPLIMIT", 19);
        configuration.config("platform.socket.IPV6_RECVHOPOPTS", 20);
        configuration.config("platform.socket.IPV6_RECVPATHMTU", 36);
        configuration.config("platform.socket.IPV6_RECVPKTINFO", 18);
        configuration.config("platform.socket.IPV6_RECVRTHDR", 22);
        configuration.config("platform.socket.IPV6_RECVTCLASS", 25);
        configuration.config("platform.socket.IPV6_RTHDR", 16);
        configuration.config("platform.socket.IPV6_RTHDRDSTOPTS", 17);
        configuration.config("platform.socket.IPV6_RTHDR_TYPE_0", 0);
        configuration.config("platform.socket.IPV6_TCLASS", 38);
        configuration.config("platform.socket.IPV6_UNICAST_HOPS", 5);
        configuration.config("platform.socket.IPV6_V6ONLY", 39);
        configuration.config("platform.socket.IP_ADD_MEMBERSHIP", 19);
        configuration.config("platform.socket.IP_ADD_SOURCE_MEMBERSHIP", 23);
        configuration.config("platform.socket.IP_BLOCK_SOURCE", 21);
        configuration.config("platform.socket.IP_DEFAULT_MULTICAST_LOOP", 1);
        configuration.config("platform.socket.IP_DEFAULT_MULTICAST_TTL", 1);
        configuration.config("platform.socket.IP_DROP_MEMBERSHIP", 20);
        configuration.config("platform.socket.IP_DROP_SOURCE_MEMBERSHIP", 24);
        configuration.config("platform.socket.IP_HDRINCL", 2);
        configuration.config("platform.socket.IP_MULTICAST_IF", 16);
        configuration.config("platform.socket.IP_MULTICAST_LOOP", 18);
        configuration.config("platform.socket.IP_MULTICAST_TTL", 17);
        configuration.config("platform.socket.IP_OPTIONS", 1);
        configuration.config("platform.socket.IP_PKTINFO", 26);
        configuration.config("platform.socket.IP_RECVOPTS", 5);
        configuration.config("platform.socket.IP_RECVRETOPTS", 6);
        configuration.config("platform.socket.IP_RECVTTL", 11);
        configuration.config("platform.socket.IP_RETOPTS", 8);
        configuration.config("platform.socket.IP_TOS", 3);
        configuration.config("platform.socket.IP_TTL", 4);
        configuration.config("platform.socket.IP_UNBLOCK_SOURCE", 22);
        configuration.config("platform.socket.MCAST_BLOCK_SOURCE", 43);
        configuration.config("platform.socket.MCAST_EXCLUDE", 2);
        configuration.config("platform.socket.MCAST_INCLUDE", 1);
        configuration.config("platform.socket.MCAST_JOIN_GROUP", 41);
        configuration.config("platform.socket.MCAST_JOIN_SOURCE_GROUP", 45);
        configuration.config("platform.socket.MCAST_LEAVE_GROUP", 42);
        configuration.config("platform.socket.MCAST_LEAVE_SOURCE_GROUP", 46);
        configuration.config("platform.socket.MCAST_UNBLOCK_SOURCE", 44);
        configuration.config("platform.socket.MSG_CTRUNC", 16);
        configuration.config("platform.socket.MSG_DONTROUTE", 4);
        configuration.config("platform.socket.MSG_DONTWAIT", 128);
        configuration.config("platform.socket.MSG_EOR", 8);
        configuration.config("platform.socket.MSG_OOB", 1);
        configuration.config("platform.socket.MSG_PEEK", 2);
        configuration.config("platform.socket.MSG_TRUNC", 32);
        configuration.config("platform.socket.MSG_WAITALL", 64);
        configuration.config("platform.socket.NI_DGRAM", 16);
        configuration.config("platform.socket.NI_MAXHOST", 1025);
        configuration.config("platform.socket.NI_MAXSERV", 32);
        configuration.config("platform.socket.NI_NAMEREQD", 4);
        configuration.config("platform.socket.NI_NOFQDN", 1);
        configuration.config("platform.socket.NI_NUMERICHOST", 2);
        configuration.config("platform.socket.NI_NUMERICSERV", 8);
        configuration.config("platform.socket.SCM_RIGHTS", 4112);
        configuration.config("platform.socket.SCM_TIMESTAMP", 4115);
        configuration.config("platform.socket.SEEK_CUR", 1);
        configuration.config("platform.socket.SEEK_END", 2);
        configuration.config("platform.socket.SEEK_SET", 0);
        configuration.config("platform.socket.SHUT_RD", 0);
        configuration.config("platform.socket.SHUT_RDWR", 2);
        configuration.config("platform.socket.SHUT_WR", 1);
        configuration.config("platform.socket.SOCK_DGRAM", 1);
        configuration.config("platform.socket.SOCK_RAW", 4);
        configuration.config("platform.socket.SOCK_RDM", 5);
        configuration.config("platform.socket.SOCK_SEQPACKET", 6);
        configuration.config("platform.socket.SOCK_STREAM", 2);
        configuration.config("platform.socket.SOL_SOCKET", 65535);
        configuration.config("platform.socket.SO_ACCEPTCONN", 2);
        configuration.config("platform.socket.SO_ATTACH_FILTER", 1073741825);
        configuration.config("platform.socket.SO_BROADCAST", 32);
        configuration.config("platform.socket.SO_DEBUG", 1);
        configuration.config("platform.socket.SO_DETACH_FILTER", 1073741826);
        configuration.config("platform.socket.SO_DOMAIN", 4108);
        configuration.config("platform.socket.SO_DONTROUTE", 16);
        configuration.config("platform.socket.SO_ERROR", 4103);
        configuration.config("platform.socket.SO_KEEPALIVE", 8);
        configuration.config("platform.socket.SO_LINGER", 128);
        configuration.config("platform.socket.SO_OOBINLINE", 256);
        configuration.config("platform.socket.SO_RCVBUF", 4098);
        configuration.config("platform.socket.SO_RCVLOWAT", 4100);
        configuration.config("platform.socket.SO_RCVTIMEO", 4102);
        configuration.config("platform.socket.SO_REUSEADDR", 4);
        configuration.config("platform.socket.SO_REUSEPORT", 4110);
        configuration.config("platform.socket.SO_SNDBUF", 4097);
        configuration.config("platform.socket.SO_SNDLOWAT", 4099);
        configuration.config("platform.socket.SO_SNDTIMEO", 4101);
        configuration.config("platform.socket.SO_TIMESTAMP", 4115);
        configuration.config("platform.socket.SO_TYPE", 4104);
        configuration.config("platform.socket.TCP_CONGESTION", 35);
        configuration.config("platform.socket.TCP_CORK", 24);
        configuration.config("platform.socket.TCP_INFO", 34);
        configuration.config("platform.socket.TCP_LINGER2", 28);
        configuration.config("platform.socket.TCP_MAXSEG", 2);
        configuration.config("platform.socket.TCP_MD5SIG", 36);
        configuration.config("platform.socket.TCP_NODELAY", 1);
        configuration.config("platform.socket.SOMAXCONN", 128);
        configuration.config("platform.process.EXIT_SUCCESS", 0);
        configuration.config("platform.process.EXIT_FAILURE", 1);
        configuration.config("platform.process.WNOHANG", 64);
        configuration.config("platform.process.WUNTRACED", 4);
        configuration.config("platform.process.PRIO_PROCESS", 0);
        configuration.config("platform.process.PRIO_PGRP", 1);
        configuration.config("platform.process.PRIO_USER", 2);
        configuration.config("platform.process.RLIMIT_CPU", 0);
        configuration.config("platform.process.RLIMIT_FSIZE", 1);
        configuration.config("platform.process.RLIMIT_DATA", 2);
        configuration.config("platform.process.RLIMIT_STACK", 3);
        configuration.config("platform.process.RLIMIT_CORE", 4);
        configuration.config("platform.process.RLIMIT_NOFILE", 5);
        configuration.config("platform.process.RLIMIT_AS", 6);
        configuration.config("platform.process.RLIM_INFINITY", newBignum(context, "18446744073709551613"));
        configuration.config("platform.process.RLIM_SAVED_MAX", newBignum(context, "18446744073709551614"));
        configuration.config("platform.process.RLIM_SAVED_CUR", newBignum(context, "18446744073709551615"));
        configuration.config("platform.signal.SIGHUP", 1);
        configuration.config("platform.signal.SIGINT", 2);
        configuration.config("platform.signal.SIGQUIT", 3);
        configuration.config("platform.signal.SIGILL", 4);
        configuration.config("platform.signal.SIGTRAP", 5);
        configuration.config("platform.signal.SIGIOT", 6);
        configuration.config("platform.signal.SIGABRT", 6);
        configuration.config("platform.signal.SIGEMT", 7);
        configuration.config("platform.signal.SIGFPE", 8);
        configuration.config("platform.signal.SIGKILL", 9);
        configuration.config("platform.signal.SIGBUS", 10);
        configuration.config("platform.signal.SIGSEGV", 11);
        configuration.config("platform.signal.SIGSYS", 12);
        configuration.config("platform.signal.SIGPIPE", 13);
        configuration.config("platform.signal.SIGALRM", 14);
        configuration.config("platform.signal.SIGTERM", 15);
        configuration.config("platform.signal.SIGURG", 21);
        configuration.config("platform.signal.SIGSTOP", 23);
        configuration.config("platform.signal.SIGTSTP", 24);
        configuration.config("platform.signal.SIGCONT", 25);
        configuration.config("platform.signal.SIGCHLD", 18);
        configuration.config("platform.signal.SIGCLD", 18);
        configuration.config("platform.signal.SIGTTIN", 26);
        configuration.config("platform.signal.SIGTTOU", 27);
        configuration.config("platform.signal.SIGIO", 22);
        configuration.config("platform.signal.SIGXCPU", 30);
        configuration.config("platform.signal.SIGXFSZ", 31);
        configuration.config("platform.signal.SIGVTALRM", 28);
        configuration.config("platform.signal.SIGPROF", 29);
        configuration.config("platform.signal.SIGWINCH", 20);
        configuration.config("platform.signal.SIGUSR1", 16);
        configuration.config("platform.signal.SIGUSR2", 17);
        configuration.config("platform.signal.SIGLOST", 37);
        configuration.config("platform.signal.SIGPWR", 19);
        configuration.config("platform.signal.SIGPOLL", 22);
        configuration.config("platform.dlopen.RTLD_LAZY", 1);
        configuration.config("platform.dlopen.RTLD_NOW", 2);
        configuration.config("platform.dlopen.RTLD_LOCAL", 0);
        configuration.config("platform.dlopen.RTLD_GLOBAL", 256);
        configuration.config("platform.typedef.int8_t", string(context, "char"));
        configuration.config("platform.typedef.int16_t", string(context, "short"));
        configuration.config("platform.typedef.int32_t", string(context, "int"));
        configuration.config("platform.typedef.int64_t", string(context, "long"));
        configuration.config("platform.typedef.uint8_t", string(context, "uchar"));
        configuration.config("platform.typedef.uint16_t", string(context, "ushort"));
        configuration.config("platform.typedef.uint32_t", string(context, "uint"));
        configuration.config("platform.typedef.uint64_t", string(context, "ulong"));
        configuration.config("platform.typedef.intmax_t", string(context, "long"));
        configuration.config("platform.typedef.uintmax_t", string(context, "ulong"));
        configuration.config("platform.typedef.intptr_t", string(context, "long"));
        configuration.config("platform.typedef.uintptr_t", string(context, "ulong"));
        configuration.config("platform.typedef.int_fast8_t", string(context, "char"));
        configuration.config("platform.typedef.int_fast16_t", string(context, "int"));
        configuration.config("platform.typedef.int_fast32_t", string(context, "int"));
        configuration.config("platform.typedef.int_fast64_t", string(context, "long"));
        configuration.config("platform.typedef.uint_fast8_t", string(context, "uchar"));
        configuration.config("platform.typedef.uint_fast16_t", string(context, "uint"));
        configuration.config("platform.typedef.uint_fast32_t", string(context, "uint"));
        configuration.config("platform.typedef.uint_fast64_t", string(context, "ulong"));
        configuration.config("platform.typedef.int_least8_t", string(context, "char"));
        configuration.config("platform.typedef.int_least16_t", string(context, "short"));
        configuration.config("platform.typedef.int_least32_t", string(context, "int"));
        configuration.config("platform.typedef.int_least64_t", string(context, "long"));
        configuration.config("platform.typedef.uint_least8_t", string(context, "uchar"));
        configuration.config("platform.typedef.uint_least16_t", string(context, "ushort"));
        configuration.config("platform.typedef.uint_least32_t", string(context, "uint"));
        configuration.config("platform.typedef.uint_least64_t", string(context, "ulong"));
        configuration.config("platform.typedef.lock_t", string(context, "uint"));
        configuration.config("platform.typedef.longlong_t", string(context, "long_long"));
        configuration.config("platform.typedef.u_longlong_t", string(context, "ulong_long"));
        configuration.config("platform.typedef.t_scalar_t", string(context, "int"));
        configuration.config("platform.typedef.t_uscalar_t", string(context, "uint"));
        configuration.config("platform.typedef.uchar_t", string(context, "uchar"));
        configuration.config("platform.typedef.ushort_t", string(context, "ushort"));
        configuration.config("platform.typedef.uint_t", string(context, "uint"));
        configuration.config("platform.typedef.ulong_t", string(context, "ulong"));
        configuration.config("platform.typedef.caddr_t", string(context, "string"));
        configuration.config("platform.typedef.daddr_t", string(context, "long"));
        configuration.config("platform.typedef.cnt_t", string(context, "short"));
        configuration.config("platform.typedef.ptrdiff_t", string(context, "long"));
        configuration.config("platform.typedef.pfn_t", string(context, "ulong"));
        configuration.config("platform.typedef.pgcnt_t", string(context, "ulong"));
        configuration.config("platform.typedef.spgcnt_t", string(context, "long"));
        configuration.config("platform.typedef.use_t", string(context, "uchar"));
        configuration.config("platform.typedef.sysid_t", string(context, "short"));
        configuration.config("platform.typedef.index_t", string(context, "short"));
        configuration.config("platform.typedef.timeout_id_t", string(context, "pointer"));
        configuration.config("platform.typedef.bufcall_id_t", string(context, "pointer"));
        configuration.config("platform.typedef.off_t", string(context, "long"));
        configuration.config("platform.typedef.off64_t", string(context, "long"));
        configuration.config("platform.typedef.ino_t", string(context, "ulong"));
        configuration.config("platform.typedef.blkcnt_t", string(context, "long"));
        configuration.config("platform.typedef.fsblkcnt_t", string(context, "ulong"));
        configuration.config("platform.typedef.fsfilcnt_t", string(context, "ulong"));
        configuration.config("platform.typedef.ino64_t", string(context, "ulong"));
        configuration.config("platform.typedef.blkcnt64_t", string(context, "long"));
        configuration.config("platform.typedef.fsblkcnt64_t", string(context, "ulong"));
        configuration.config("platform.typedef.fsfilcnt64_t", string(context, "ulong"));
        configuration.config("platform.typedef.blksize_t", string(context, "int"));
        configuration.config("platform.typedef.pad64_t", string(context, "long"));
        configuration.config("platform.typedef.upad64_t", string(context, "ulong"));
        configuration.config("platform.typedef.offset_t", string(context, "long_long"));
        configuration.config("platform.typedef.u_offset_t", string(context, "ulong_long"));
        configuration.config("platform.typedef.len_t", string(context, "ulong_long"));
        configuration.config("platform.typedef.diskaddr_t", string(context, "ulong_long"));
        configuration.config("platform.typedef.k_fltset_t", string(context, "uint"));
        configuration.config("platform.typedef.id_t", string(context, "int"));
        configuration.config("platform.typedef.lgrp_id_t", string(context, "int"));
        configuration.config("platform.typedef.useconds_t", string(context, "uint"));
        configuration.config("platform.typedef.suseconds_t", string(context, "long"));
        configuration.config("platform.typedef.major_t", string(context, "uint"));
        configuration.config("platform.typedef.minor_t", string(context, "uint"));
        configuration.config("platform.typedef.pri_t", string(context, "short"));
        configuration.config("platform.typedef.cpu_flag_t", string(context, "ushort"));
        configuration.config("platform.typedef.o_mode_t", string(context, "ushort"));
        configuration.config("platform.typedef.o_dev_t", string(context, "short"));
        configuration.config("platform.typedef.o_uid_t", string(context, "ushort"));
        configuration.config("platform.typedef.o_gid_t", string(context, "ushort"));
        configuration.config("platform.typedef.o_nlink_t", string(context, "short"));
        configuration.config("platform.typedef.o_pid_t", string(context, "short"));
        configuration.config("platform.typedef.o_ino_t", string(context, "ushort"));
        configuration.config("platform.typedef.key_t", string(context, "int"));
        configuration.config("platform.typedef.mode_t", string(context, "uint"));
        configuration.config("platform.typedef.uid_t", string(context, "uint"));
        configuration.config("platform.typedef.gid_t", string(context, "uint"));
        configuration.config("platform.typedef.datalink_id_t", string(context, "uint"));
        configuration.config("platform.typedef.vrid_t", string(context, "uint"));
        configuration.config("platform.typedef.taskid_t", string(context, "int"));
        configuration.config("platform.typedef.projid_t", string(context, "int"));
        configuration.config("platform.typedef.poolid_t", string(context, "int"));
        configuration.config("platform.typedef.zoneid_t", string(context, "int"));
        configuration.config("platform.typedef.ctid_t", string(context, "int"));
        configuration.config("platform.typedef.pthread_t", string(context, "uint"));
        configuration.config("platform.typedef.pthread_key_t", string(context, "uint"));
        configuration.config("platform.typedef.dev_t", string(context, "ulong"));
        configuration.config("platform.typedef.nlink_t", string(context, "uint"));
        configuration.config("platform.typedef.pid_t", string(context, "int"));
        configuration.config("platform.typedef.size_t", string(context, "ulong"));
        configuration.config("platform.typedef.ssize_t", string(context, "long"));
        configuration.config("platform.typedef.time_t", string(context, "long"));
        configuration.config("platform.typedef.clock_t", string(context, "long"));
        configuration.config("platform.typedef.clockid_t", string(context, "int"));
        configuration.config("platform.typedef.timer_t", string(context, "int"));
        configuration.config("platform.typedef.unchar", string(context, "uchar"));
        configuration.config("platform.typedef.ushort", string(context, "ushort"));
        configuration.config("platform.typedef.uint", string(context, "uint"));
        configuration.config("platform.typedef.ulong", string(context, "ulong"));
        configuration.config("platform.typedef.u_char", string(context, "uchar"));
        configuration.config("platform.typedef.u_short", string(context, "ushort"));
        configuration.config("platform.typedef.u_int", string(context, "uint"));
        configuration.config("platform.typedef.u_long", string(context, "ulong"));
        configuration.config("platform.typedef.hrtime_t", string(context, "long_long"));
        configuration.config("platform.typedef.fd_mask", string(context, "long"));
        configuration.config("platform.typedef.fds_mask", string(context, "long"));
        configuration.config("platform.typedef.mrpid_t", string(context, "int"));
        configuration.config("platform.typedef.memsize_t", string(context, "ulong_long"));
        configuration.config("platform.typedef.memtime_sec_t", string(context, "long_long"));
        configuration.config("platform.typedef.sa_family_t", string(context, "ushort"));
        configuration.config("platform.typedef.socklen_t", string(context, "uint"));
        configuration.config("platform.typedef.Psocklen_t", string(context, "pointer"));
        configuration.config("platform.typedef.nfds_t", string(context, "ulong"));
        configuration.config("platform.typedef.disp_lock_t", string(context, "uint"));
        configuration.config("platform.typedef.)", string(context, "pointer"));
        configuration.config("platform.typedef.model_t", string(context, "uint"));
        configuration.config("platform.typedef.in_port_t", string(context, "ushort"));
        configuration.config("platform.typedef.in_addr_t", string(context, "uint"));
        configuration.config("platform.typedef.ipaddr_t", string(context, "uint"));
        configuration.config("platform.typedef.rlim_t", string(context, "ulong"));
        configuration.config("platform.typedef.rlim64_t", string(context, "ulong_long"));
    }

}
