/*
 ***** BEGIN LICENSE BLOCK *****
 * Version: EPL 1.0/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Eclipse Public
 * License Version 1.0 (the "License"); you may not use this file
 * except in compliance with the License. You may obtain a copy of
 * the License at http://www.eclipse.org/legal/epl-v10.html
 *
 * Software distributed under the License is distributed on an "AS
 * IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
 * implied. See the License for the specific language governing
 * rights and limitations under the License.
 *
 * Copyright (C) 2013-2017 The JRuby Team (jruby@jruby.org)
 * 
 * Alternatively, the contents of this file may be used under the terms of
 * either of the GNU General Public License Version 2 or later (the "GPL"),
 * or the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the EPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the EPL, the GPL or the LGPL.
 ***** END LICENSE BLOCK *****/
package org.truffleruby.parser.parser;

public class YyTables {
   private static short[] combine(short[] t1, short[] t2, 
                                  short[] t3, short[] t4) {
      short[] t = new short[t1.length + t2.length + t3.length + t4.length];
      int index = 0;
      System.arraycopy(t1, 0, t, index, t1.length);
      index += t1.length;
      System.arraycopy(t2, 0, t, index, t2.length);
      index += t2.length;
      System.arraycopy(t3, 0, t, index, t3.length);
      index += t3.length;
      System.arraycopy(t4, 0, t, index, t4.length);
      return t;
   }

   public static final short[] yyTable() {
      return combine(yyTable1(), yyTable2(), yyTable3(), yyTable4());
   }

   public static final short[] yyCheck() {
      return combine(yyCheck1(), yyCheck2(), yyCheck3(), yyCheck4());
   }
   private static final short[] yyTable1() {
      return new short[] {

          306,  355,  309,  488,  379,   89,   89,  135,  135,  419, 
          303,  421,  251,  251,  251,  229,  231,  277,  251,  251, 
          665,  308,  251,  646,  499,  684,  249,  249,  249,  644, 
          134,  134,  325,  215,  617,  229,  231,  783,   74,  108, 
          134,  353,  631,  632,  216,  216,  519,  520,   89,   84, 
           84,  441,  351,  803,  251,  428,  466,  600,   17,  608, 
          644,  388,  613,  671,  684,  216,  633,  812,  604,  289, 
          289,  683,  644,  289,  683,  134,  293,  293,  644,  361, 
          293,  644,    8,  527,  630,   75,  645,  529,  580,   72, 
          216,   84,    8,  436,  572,  644,  434,  305,  824,  257, 
          600,   87,  608,  351,  261,  693,  578,   17,  332,  336, 
          388,  660,  645,  998,  377,  515,  645,  644,   73,   87, 
         1021,  644,  580,  305,  603,  674,  134,  549,  572,  315, 
          644,  579,  251,  590,   89,  633,   73,  580,  257,  935, 
          578,    8,   84,  572,  598,  394,  438,  524,  256,  644, 
          645,  572,  377,  257,  302,  578,  344,  292,  292,  639, 
         1028,  292,  842,  255,  258,  579,  644,  590,  439,  311, 
          257,  462,  440,  603,  754,  645,  549,  639,  598,  711, 
          579,  523,  590,  925,  705,  572,  524,  256,  564,  331, 
          409,  564,  564,  598,  394,  633,  378,  476,  261,  795, 
          572,  468,  256,  943,  808,  344,  488,  409,  756,  685, 
          749,  995,  633,  301,  437,  782,  298,  689,  690,  256, 
          526,   76,  624,  273, 1021,  787,  684,  305,  588,  603, 
          998,  275,  843,  510,  378,  698,  747,  564,   89,  644, 
          433,  564,  107,  107,  750,  301,  302,  552,  708,  107, 
          107,  107,  553,  564,  381,  107,  107,  501,  374,  107, 
          379,  564,  588, 1028,  382,  251,  251,  311,  274,  377, 
          433,  535,   76,  380,  683,  683,  375,  588,  603,  249, 
          249,  314,  644,  378,  376,  107, 1041,  316,  644,  450, 
          446,  107,  644,  305,  786,  820,  788,  451,  935,  798, 
          795,  251,  366,  251,  251,  531,  325,  251,  791,  251, 
           17,   17,   17,  559,   89,  314,   17,   17,  560,   17, 
           89,  498,   89,  500,  286,  502,  503, 1060,  725,  726, 
          314,  251, 1072,  644,  644,  644,   73,  776,  368,  446, 
          644,  644,   82,  878,  775,  521,  290,  296,  372,  885, 
          297,  261,  289,  351,  289,  289,  457,  374,  431,  293, 
           84,  293,  314, 1086,  513,  643,  633,  785,  837,  107, 
           74,  107,   53,  614,  413,  618,  134,  385,  386,  628, 
          632,  424,  617,  251,  251,  251,  251,   89,  251,  251, 
           17,  893,  816,   82,  423,  644,  425,  249,  249,  249, 
          249,  578,  568,  569,  916,  631,  632,  644,  429,  668, 
          388,   17,  818,  325,  261,  471,  472,   75,  633,  626, 
          821,  251,  549,   84,  304,  644,  251,  457,  251,   89, 
          351,  567,  251,  251,  432,   89,   17,  580,  578,  598, 
          292,  430,  292,  572,  645,    8,  572,  645,  251,  644, 
           89,  251,  822,  134,  304,  684,   80,  609, 1084,  549, 
          609,   89,   96,   96,  251,  647,  989,  809,  634,   89, 
          966,  578,  289,  603,  279,  107,  598,  644,  251,   84, 
          579,  644,  590,  572,  609,   93,   93,  136,  136,  891, 
           76,  531,  549,  598,  394,  442,  644,  278,  691,  289, 
          609,  311,  107,  107,  683,   96,  897,   80,  251,  598, 
           89,  467,  229,  231,  609,  289,  572,  453,  537,  538, 
          564,   89,  692,  588,  603,  540,  644,  816,   93,  289, 
          463,  320,  352,  563,  564,  565,  566,  251,  107,  351, 
          107,  107,  216, 1031,  107,  682,  107,  446,  609,  644, 
          644,  107,  610,   76,   84,  273,  924,  107,  273,  107, 
          588,  603,  134,  275,  670,  670,  275,  623,  107,  755, 
          456,  623,  921,  609,  517,  537,  538,  588,  762,  610, 
          531,  833,  540,  352,  701,  564,  381, 1042,  289,  760, 
          374,   96,  379,  588,  603,  610,  382,  766,  418,  314, 
          274,  377,  769,  274,  644,  380,  314,  314,  375,  610, 
          666,   82,  619,  464,   93,  378,  376,  320,  762,  446, 
          107,  107,  107,  107,  107,  107,  107,  468,  351,  489, 
          314,  492,  445,  445,  762,  796,  465,   97,  445,  470, 
          994,  251,   89,  610,  933,  806,  790,  477,  762,  473, 
          518,  134,   89,  644,  445,  249,  483,  823,  107,  104, 
          229,  231,  477,  107,  484,  107,  107,  477,  610,  107, 
          107,  487,  107,  507,   82,  645,   77,  644,  789,  762, 
          251,  479,  330,  334, 1065,  107,  479,  107,  107,  216, 
          216,  833,  922,  104,  249,   96,  385,  386,  107,  887, 
           89,  107,   85,  984,  333,  351,  107,  874,  876,  986, 
          104,   78,   83,  881,  883,  107,  461,  387,   93,  511, 
          644,  512,  516,  618,  869,   80,  644,   77,  134,  522, 
          780,  644,  531,  759,  528,  530,   98,  104,  536,  547, 
          436,  765,  415,  933,   84,  107,  765,  107,  817,  416, 
          420,  551,  555,   85,  746,   89,  627,   89,  107,   89, 
          251,  251,   78,   83,  856,  558,  251,  251,  635,  644, 
           98,   96,   16,  505,  107,  759,  251,   96,  765,   96, 
          351,  833,  648,  833,  436,  103,  103,   98,   80,  653, 
          655,  781,  686,  667,   93,  623,   61,  508,   89,  933, 
           93,  251,   93,  134,  657,  320,   61,   61,   62,   63, 
           64,  869,   89,   89,   89,  249,  867,  518,  687,  826, 
          827,   16,  828,  688,  694,  697,   81,  713,  103,  962, 
          813,  869,   79,  352,  714,  415,  715,  716,  717,  718, 
          748,  702,  416,  417,   96,   89,   89,  933,  746,  714, 
          746,  715,  716,  717,  718,   61,  833, 1043,  644,  931, 
          757,  934,  758,  644,  773,  644,  784,   93,  475,  763, 
          955,  764,  644,  774,  895,  644,  777,   81,  107,  107, 
          795,  899,  251,   79,  801,  622,   96,   59,   60,  107, 
          807,  415,   96,  134,  987,  802,   89,   89,  416,  444, 
          833, 1035,  833,  804,  981,  805,  670,   96,   89,   93, 
          352,  819,  320,  838,  103,   93,  839,  107,   96,  840, 
          845,  644,  746,  847,  644,  849,   96,  134,  104,  436, 
           93,  852,  387,  833,  644,  851,  494,  107,  478,  479, 
          480,   93,  872,  482,  656,   77,  461,  855,  880,   93, 
          868,  908,  664,  415,  853,  846,  848,  534, 1015, 1024, 
          416,  460, 1019,  896,  402,  917,  879,   96,  404,  405, 
           89,   85,  123,  436,  644,  898,  901,  436,   96,  644, 
           78,   83,  134,  905,  436,  436,  909,   89,  910,   89, 
           93,  104,  107,  911,  107,   89,  107,  107,  107,  918, 
         1061,   93,  923,  107,  107,   98,  644,  867,   77,  445, 
          867,  436,  867,  107,  867,  537,  538,  941,  103,  352, 
         1045,  436,  540,  944,   16,   16,   16,  545,  436,  436, 
           16,   16,  946,   16,   85,  107,  948,  251,  107,  762, 
          950,  956,   89,   78,   83,  746,  957,  746,  958,  107, 
          107,  107,  959,  494,  518,  436,  961,  990,   61,   61, 
           61,  518,  518,   61,   61,   61, 1080,   61,   98, 1034, 
          991,  947,   92,  609,  996,  999,  951,   61,   61,  737, 
          494,  621,  107,  107, 1000,  631,   61,   61,  289,   61, 
           61,   61,   61,   61,  103,   81,  494, 1002, 1003,   96, 
          103,   79,  103, 1004,   16,  415,   92, 1006,  352,   96, 
          494,  111,  416,  474,  867, 1008,  867, 1014,  867,  107, 
         1022,  867,   93,   92,  621,   16,  644, 1029,  644,  644, 
          644,  644,   93,  107,  107, 1030,   97, 1040,   61,  679, 
          681,   61,   97,  333,  868,  107, 1001, 1032,  868, 1011, 
           16, 1005, 1007,  512, 1009,  867, 1010,   96,   81,   61, 
          111, 1048, 1050, 1052,   79, 1055, 1062,  103,  610,  494, 
           97, 1070, 1073,  737,   92,  737,  108, 1075, 1085,  681, 
           93, 1089,  333,  415,   61,  352,  631,   97,  644,  644, 
          416,  509,  512,  830,  415,  633,  753,  753,  537,  538, 
          539,  416,  533,  632,  637,  540,  746,  107,  637,  103, 
          108,  633,   96,  639,   96,  103,   96,  517,  633,  542, 
          633,  512,  376,  644,  107,  481,  107,  108, 1054,  364, 
          103,  753,  107,  622,  557,   93,  114,   93,  870,   93, 
          871,  103,  873,  768,  138,  719,  415,  737,  768,  103, 
          720, 1069,  868,  416, 1023,   96,  127,  314,  699,  218, 
          352,  770,  967,  932, 1074, 1076, 1077, 1078,  542,   96, 
           96,   96,  771,  859,  107,  937, 1017, 1016,   93,  107, 
          768,  904,   99,  725,  726,  114,  886,  280,  794,  797, 
          103,  797,   93,   93,   93,  913,  914,  915, 1068,  797, 
          920,  103,   96,   96,    0,  127,  415, 1093,  244,  792, 
            0, 1083,    0,  416, 1033,    0,   99,  515,  244,    0, 
          727,    0,  305,    0,    0,   93,   93,    0,  929,  930, 
            0,    0,   97,   99,  814,    0,  815,  314,    0,    0, 
          117,   92,  615,  681,  333,    0,    0,    0,  294,    0, 
          136,    0,  244,   96,   96,  537,  538,  542,  294,    0, 
            0,  983,  540,  519,    0,   96,  244,  244,  111,    0, 
            0,  244,    0,  678,    0,    0,   93,   93,    0,  965, 
            0,  615,    0,    0,  982,    0,    0,    0,   93,  117, 
          512,  985,  287,  537,  538,  542,    0,  512,  512,    0, 
          540,  854,    0,  615,   92,   97,    0,  294,    0,  111, 
          615,    0,    0,    0,  753,  714,    0,  715,  716,  717, 
          718,  518,  103,    0,  615,    0,    0,   96,  903,  512, 
          890,  907,  103,    0,    0,  116,  512,  512,   53,  136, 
            0,    0,  615,    0,   96,  108,   96,  125,   53,  831, 
           93,    0,   96,    0,  517,  288,  288,  832,  512,  288, 
            0,  517,  517,  615,  912,  512,  512,   93,   97,   93, 
         1036,  615, 1037,    0,    0,   93,    0,    0, 1038,    0, 
          103,  312,  314,    0,  116,    0,    0,  288,  288,    0, 
          354,  356,  615,  114,  314,    0,  125,   53,    0,   96, 
            0,  314,  314,    0,  387,    0,  616,  619,  108,    0, 
            0,  940,    0,  127,    0,  542,  517,  542,  542,  542, 
          542,  542,   93,    0,   52, 1071,  542,    0,  622,    0, 
            0,  537,  538,  544,   52,  103,  402,  103,  540,  103, 
          404,  405,  406,  407,    0,  964,  512,    0,  797,  542, 
            0,   99,    0,    0,  515,    0,    0,  542,  542,  542, 
          542,  515,  515,    0,    0,    0,    0,  616,    0,    0, 
          244,  244,  244,  333,  314,  244,  244,  244,  103,  244, 
          512,  314,  314,   52,  537,  538,  548,  619,  616,  244, 
          244,  540,  103,  103,  103,  512,  542,  117,  244,  244, 
          519,  244,  244,  244,  244,  244,    0,  519,  519,    0, 
          294,  294,  294,  244,   99,  294,  294,  294,  714,  294, 
          715,  716,  717,  718,  719,  103,  103,  512,    0,  720, 
            0,    0,  244,  244,    0,  118,    0,  244,  244,    0, 
            0,  294,  294,  294,  294,  294,   98,  244,    0,    0, 
          244,    0,  721,  244,    0,  244,    0,    0,  518,    0, 
          723,  724,  725,  726,    0,  518,  518,    0,    0,  494, 
            0,  244,  797,  537,  538,  556,  103,  103,    0,    0, 
          540,    0,    0,  244,  118,    0,    0,  294,  103,    0, 
          294,    0,  116,  518,    0,    0,  244,    0,    0,  727, 
           53,   53,   53,    0,  125,   53,   53,   53,    0,   53, 
            0,  294,    0,  483,  483,  483,  519,  622,  435,   53, 
          483,    0,  288,  288,  288,  356,    0,  288,  435,    0, 
            0,   53,   53,   53,   53,   53,  294,    0,  288,    0, 
          288,  288,  543,    0,  546,    0,  549,    0,    0,  506, 
          103,    0,  549,  517,    0,    0,  465,  465,  465,    0, 
          517,  517,  435,  465,    0,  622,    0,  103,    0,  103, 
            0,    0,  622,  622,    0,  103,    0,  435,  618,    0, 
           53,  435,  471,  471,  471,    0,   52,   52,   52,  471, 
            0,   52,   52,   52,    0,   52,    0,    0,    0,  622, 
            0,   53,    0,    0,    0,   52,    0,    0, 1025,  435, 
          715,  716,  717,  718,   52,   52,  615,   52,   52,   52, 
           52,   52,  103,    0,    0,    0,   53,  570,  571,  572, 
          573,  574,  575,  576,  577,  578,  579,  580,  581,  582, 
          583,  584,  585,  586,  587,  588,  589,  590,  591,  592, 
          593,  594,  595,  380,  381,  382,  383,  384,  288,    0, 
          543,    0,    0,  615,  512,  619,   52,    0,   95,  288, 
          633,  512,  512,    0,    0,    0,    0,  615,    0,  108, 
            0,  117,    0,  119,  230,  288,  714,   52,  715,  716, 
          717,  718,  118,    0,  230,  512,    0,    0,  615,   95, 
            0,  288,   95,  615,  663,  633,    0,    0,  288,    0, 
            0,    0,   52,    0,    0,  288,    0,    0,  387,   95, 
          831,    0,    0,  288,  288,    0,  314,  288,  230,  712, 
          518,    0,  119,    0,  400,  401,  712,  518,  518,    0, 
            0,    0,  230,  230,    0,    0,  515,  230,    0,  515, 
          402,  695,  696,  519,  404,  405,  406,  407,    0,    0, 
          519,  519,    0,  288,  520,    0,  288,  207,    0,    0, 
            0,    0,    0,    0,  288,  281,  281,  281,    0,  281, 
          435,  435,  435,  281,  281,  435,  435,  435,  281,  435, 
          281,  281,  281,  281,  281,  281,  281,    0,  435,  435, 
          435,  281,  281,  281,  281,  281,  281,  281,  435,  435, 
          281,  435,  435,  435,  435,  435,  207,  281,    0,    0, 
          281,  281,  281,    0,  281,  281,  281,  281,  281,  281, 
          281,  281,  281,  281,  281,  435,  435,  435,  435,  435, 
          435,  435,  435,  435,  435,  435,  435,  435,  435,   97, 
            0,  435,  435,  435,  435,  281,  288,  435,  281,    0, 
          435,  281,  281,  435,  281,  435,  281,  435,  281,  435, 
          281,  435,  435,  435,  435,  435,  435,  435,  281,  435, 
          435,  435,    0,  281,  281,  281,  281,  281,  281,    0, 
            0,    0,  281,  288,  281,  281,  517,  281,  281,  281, 
          281,  512,    0,  281,  281,  281,  435,    0,    0,    0, 
            0,  512,    0,    0,  127,    0,    0,    0,  288,    0, 
          288,    0,    0,    0,    0,  945,    0,  288,  288,  949, 
            0,    0,    0,    0,  114,  616,    0,   95,  616,    0, 
          119,    0,    0,    0,    0,  512,  230,  230,  230,    0, 
            0,  230,  230,  230,    0,  230,    0,    0,    0,  512, 
          512,    0,  111,  314,  512,  230,  230,    0,    0,    0, 
          314,  314,    0,    0,  230,  230,  619,  230,  230,  230, 
          230,  230,    0,  515,    0,  288,  515,    0,    0,  230, 
          515,  515,  512,  515,  515,    0,  616,  875,  877,  616, 
           95,  520,    0,  882,  884,    0,    0,    0,  520,  520, 
            0,    0,  888,  663,  288,    0,    0,  616,  892,  387, 
          616,    0,    0,  230,    0,    0,  230,    0,    0,  230, 
            0,  230,    0,  515,    0,  400,  401,    0,    0,    0, 
            0,  875,  877,  515,  882,  884,    0,  230,  288,    0, 
            0,  402,    0,  403,    0,  404,  405,  406,  407,  230, 
            0,    0,    0,  207,    0,  207,  207,  207,  207,  207, 
            0,    0,  230,    0,  207,    0,    0,  515,    0, 1049, 
            0,    0, 1051, 1053,  116,  518, 1056, 1057,    0,    0, 
            0,  515,  515,  518,  114,  288,  515,  207,  207,    0, 
            0,    0,    0,    0,    0,  207,  207,  207,  207,    0, 
            0,    0,  960,    0,    0,    0,    0,    0,    0,  963, 
            0,    0,    0,    0,  515,    0,    0,    0,    0,  288, 
            0,    0,    0,  517,    0,    0,    0,    0,    0,    0, 
          517,  517,    0,    0,  207,    0, 1088, 1090, 1091, 1092, 
          963,    0,    0,  200,    0,    0,    0,  288,  615,  615, 
          615, 1095,  615,  512,  512,  512,  615,  615,  512,  512, 
          512,  615,  512,  615,  615,  615,  615,  615,  615,  615, 
            0,  615,  512,  512,  615,  615,  615,  615,  615,  615, 
          615,  512,  512,  615,  512,  512,  512,  512,  512,    0, 
          615,    0,  200,  615,  615,  615,  512,  615,  615,  615, 
          615,  615,  615,  615,  615,  615,  615,  615,  512,  512, 
          512,  512,  512,  512,  512,  512,  512,  512,  512,  512, 
          512,  512,    0,    0,  512,  512,  512,  512,  615,  524, 
          512,  615,  615,  512,  615,  615,  512,  615,  512,  615, 
          512,  615,  512,  615,  512,  512,  512,  512,  512,  512, 
          512,  615,  512,  615,  512,  866,  615,  615,  615,  615, 
          615,  615,    0,    0,  288,  615,  512,  615,  615,    0, 
          615,  615,  615,  615,  123,  210,  615,  615,  615,  512, 
          616,  616,  616,    0,  616,  515,  515,  515,  616,  616, 
          515,  515,  515,  616,  515,  616,  616,  616,  616,  616, 
          616,  616,    0,  616,  515,  515,  616,  616,  616,  616, 
          616,  616,  616,  515,  515,  616,  515,  515,  515,  515, 
          515,    0,  616,  123,  210,  616,  616,  616,  515,  616, 
          616,  616,  616,  616,  616,  616,  616,  616,  616,  616, 
          515,  515,  515,  515,  515,  515,  515,  515,  515,  515, 
          515,  515,  515,  515,  939,  436,  515,  515,  515,  515, 
          616,  527,  515,  616,  616,  515,  616,  616,  515,  616, 
          515,  616,  515,  616,  515,  616,  515,  515,  515,  515, 
          515,  515,  515,  616,  515,  616,  515,    0,  616,  616, 
          616,  616,  616,  616,    0,    0,  622,  616,  515,  616, 
          616,    0,  616,  616,  616,  616,  622,    0,  616,  616, 
          616,  515,    0,  631,  631,  631,    0,    0,    0,  631, 
          631,    0,  631,  632,  632,  632,    0,    0,    0,  632, 
          632,    0,  632,  714,    0,  715,  716,  717,  718,  200, 
          622,  200,  200,  200,  200,  200,  866,    0,    0,  866, 
          200, 1018,    0,  866,  622,  622,    0,  125,    0,  622, 
          714,  517,  715,  716,  717,  718,  719,  831,    0,  517, 
            0,  720,    0,  200,  200,  997,    0,    0,    0,  387, 
            0,  200,  200,  200,  200,    0,    0,  622,    0,    0, 
            0,    0,    0,  631,  721,  400,  401,    0,    0,    0, 
            0,    0,    0,  632,  725,  726,    0,    0,    0,    0, 
            0,  402,    0,  403,  631,  404,  405,  406,  407,    0, 
          200,  410,    0,  411,  632,    0,    0,    0,  618,    0, 
            0,  123,    0,    0,    0,    0,    0,    0,  618,  631, 
            0,  727,    0,    0,  436,    0,    0,    0,    0,  632, 
            0,    0,    0,  866,    0,  866,    0,  866,    0,    0, 
          866,    0,    0,    0,    0,    0,    0,    0,    0,    0, 
            0,  210,  618,  210,  210,  210,  210,  210,    0,    0, 
            0,    0,  210,    0,    0,    0,  618,  618,    0,  124, 
            0,  618,  436,  519,  866,    0,    0,    0,    0,  436, 
          436,  519,    0,    0,    0,  210,  210,    0,    0,    0, 
            0,    0,    0,  210,  210,  210,  210,    0,    0,  618, 
            0,    0,    0,    0,    0,    0,  436,    0,    0,    0, 
            0,    0,    0,    0,    0,    0,    0,    0,    0,    0, 
            0,    0,    0,    0,    0,    0,    0,    0,  195,    0, 
            0,    0,  210,  618,  618,  618,    0,  618,  622,  622, 
          622,  618,  618,  622,  622,  622,  618,  622,  618,  618, 
          618,  618,  618,  618,  618,    0,  622,  622,  622,  618, 
          618,  618,  618,  618,  618,  618,  622,  622,  618,  622, 
          622,  622,  622,  622,    0,  618,    0,  195,  618,  618, 
          618,  622,  618,  618,  618,  618,  618,  618,  618,  618, 
          618,  618,  618,  622,  622,  622,  622,  622,  622,  622, 
          622,  622,  622,  622,  622,  622,  622,    0,    0,  622, 
          622,  622,  622,  618,    0,  622,  618,  618,  622,  618, 
          618,  622,  618,  622,  618,  622,  618,  622,  618,  622, 
          622,  622,  622,  622,  622,  622,  618,  622,  622,  622, 
          412,  618,  618,  618,  618,  618,  618,    0,    0,    0, 
          618,  622,  618,  618,    0,  618,  618,  618,  618,  121, 
          177,  618,  618,  618,  622,  618,  618,  618,    0,  618, 
          618,  618,  618,  618,  618,  618,  618,  618,  618,  618, 
          618,  618,  618,  618,  618,  618,  618,    0,  618,  618, 
          618,  618,  618,  618,  618,  618,  618,  618,  618,  618, 
          618,  618,  618,  618,  618,  618,    0,  618,  121,  177, 
          618,  618,  618,  618,  618,  618,  618,  618,  618,  618, 
          618,  618,  618,  618,  618,  618,  618,  618,  618,  618, 
          618,  618,  618,  618,  618,  618,  618,  618,  618,   99, 
          522,  618,  618,  618,  618,  618,    0,  618,  618,  618, 
          618,  618,  618,  618,  618,  618,  618,  618,  618,  618, 
          618,  618,  618,  618,  618,  618,  618,  618,  618,  618, 
          618,  618,    0,  618,  618,  618,  618,  618,  618,    0, 
            0,  619,  618,  618,  618,  618,  519,  618,  618,  618, 
          618,  619,    0,  618,  618,  618,  618,    0,    0,    0, 
            0,    0,    0,    0,    0,    0,    0,  714,    0,  715, 
          716,  717,  718,  719,    0,    0,    0,    0,  720,    0, 
            0,    0,    0,    0,  195,  619,  195,  195,  195,  195, 
          195,    0,    0,    0,    0,  195,    0,    0,    0,  619, 
          619,  721,  126,    0,  619,    0,  520,    0,    0,  723, 
          724,  725,  726,    0,  520,    0,    0,    0,  195,  195, 
            0,    0,    0,    0,    0,    0,  195,  195,  195,  195, 
            0,    0,  619,    0,  772,    0,    0,    0,    0,    0, 
            0,    0,    0,    0,    0,    0,    0,    0,  727,    0, 
            0,    0,    0,    0,  387,  388,  389,  390,  391,  392, 
          393,  394,  395,  396,  397,  195,  398,  399,    0,    0, 
          400,  401,    0,  305,    0,    0,  121,    0,    0,    0, 
            0,    0,    0,  305,    0,    0,  402,    0,  403,    0, 
          404,  405,  406,  407,  408,  409,  410,    0,  411,    0, 
            0,    0,    0,    0,    0,    0,    0,    0,    0,    0, 
            0,    0,    0,    0,    0,    0,  177,  305,  177,  177, 
          177,  177,  177,    0,    0,    0,    0,  177,    0,    0, 
            0,  305,  305,    0,  118,    0,  305,  522,  522,    0, 
            0,    0,    0,    0,  522,  522,  522,    0,    0,    0, 
          177,  177,    0,    0,    0,    0,    0,    0,  177,  177, 
          177,  177,    0,    0,  305,    0,    0,    0,    0,    0, 
            0,    0,    0,    0,    0,    0,    0,    0,    0,    0, 
            0,    0,    0,  519,    0,    0,    0,    0,    0,    0, 
          519,  519,    0,  178,    0,    0,    0,  177,  619,  619, 
          619,    0,  619,  619,  619,  619,  619,  619,  619,  619, 
          619,  619,  619,  619,  619,  619,  619,  619,  619,  619, 
            0,  619,  619,  619,  619,  619,  619,  619,  619,  619, 
          619,  619,  619,  619,  619,  619,  619,  619,  619,    0, 
          619,    0,  178,  619,  619,  619,  619,  619,  619,  619, 
          619,  619,  619,  619,  619,  619,  619,  619,  619,  619, 
          619,  619,  619,  619,  619,  619,  619,  619,  619,  619, 
          619,  619,  106,    0,  619,  619,  619,  619,  619,    0, 
          619,  619,  619,  619,  619,  619,  619,  619,  619,  619, 
          619,  619,  619,  619,  619,  619,  619,  619,  619,  619, 
          619,  619,  619,  619,  619,  100,  619,  619,  619,  619, 
          619,  619,    0,    0,    0,  619,  619,  619,  619,  622, 
          619,  619,  619,  619,  120,  179,  619,  619,  619,  619, 
          617,  617,  617,    0,  617,  305,  305,  305,  617,  617, 
          305,  305,  305,  617,  305,  617,  617,  617,  617,  617, 
          617,  617,  520,  617,  305,  305,  617,  617,  617,  617, 
          617,  617,  617,  305,  305,  617,  305,  305,  305,  305, 
          305,    0,  617,  120,  179,  617,  617,  617,  305,  617, 
          617,  617,  617,  617,  617,  617,  617,  617,  617,  617, 
          305,  305,  305,  305,  305,  305,  305,  305,  305,  305, 
          305,  305,  305,  305,  102,  521,  305,  305,  305,  305, 
          617,    0,  305,  617,  617,  305,  617,  617,  305,  617, 
          305,  617,  305,  617,  305,  617,  305,  305,  305,  305, 
          305,  305,  305,  617,  305,  617,  305,    0,  617,  617, 
          617,  617,  617,  617,    0,    0,  623,  617,  305,  617, 
          617,  522,  617,  617,  617,  617,  623,    0,  617,  617, 
          617,  305,    0,    0,    0,    0,    0,    0,    0,    0, 
            0,    0,    0,    0,    0,    0,    0,    0,    0,    0, 
            0,    0,    0,    0,    0,    0,    0,    0,    0,  178, 
          623,  178,  178,  178,  178,  178,    0,    0,    0,    0, 
          178,    0,    0,    0,  623,  623,    0,  125,  622,  623, 
          714,  521,  715,  716,  717,  718,  719,    0,    0,  521, 
            0,  720,    0,  178,  178,    0,    0,    0,    0,    0, 
            0,  178,  178,  178,  178,    0,    0,  623,    0,    0, 
          119,    0,    0,    0,  721,  722,    0,    0,    0,    0, 
            0,    0,  723,  724,  725,  726,  622,    0,    0,    0, 
            0,    0,    0,  622,  622,    0,  387,    0,    0,  618, 
          178,  392,  393,    0,    0,    0,    0,    0,  624,    0, 
            0,  120,  400,  401,    0,    0,    0,    0,  624,  520, 
          622,  727,    0,    0,    0,    0,  520,  520,  402,    0, 
          403,    0,  404,  405,  406,  407,  408,  409,  410,    0, 
          411,    0,    0,    0,    0,    0,    0,    0,    0,    0, 
            0,  179,  624,  179,  179,  179,  179,  179,    0,    0, 
            0,    0,  179,    0,    0,    0,  624,  624,    0,  121, 
            0,  624,  521,  523,    0,    0,    0,    0,    0,  521, 
          521,  523,    0,    0,    0,  179,  179,    0,    0,    0, 
            0,    0,    0,  179,  179,  179,  179,    0,    0,  624, 
            0,    0,    0,    0,    0,    0,    0,    0,    0,    0, 
            0,    0,    0,    0,    0,    0,    0,    0,  522,    0, 
            0,    0,    0,    0,    0,  522,  522,    0,  135,    0, 
            0,    0,  179,  620,  620,  620,    0,  620,  623,  623, 
          623,  620,  620,  623,  623,  623,  620,  623,  620,  620, 
          620,  620,  620,  620,  620,    0,  623,  623,  623,  620, 
          620,  620,  620,  620,  620,  620,  623,  623,  620,  623, 
          623,  623,  623,  623,    0,  620,    0,  135,  620,  620, 
          620,  623,  620,  620,  620,  620,  620,  620,  620,  620, 
          620,  620,  620,  623,  623,  623,  623,  623,  623,  623, 
          623,  623,  623,  623,  623,  623,  623,    0,    0,  623, 
          623,  623,  623,  620,    0,  623,  620,  620,  623,  620, 
          620,  623,  620,  623,  620,  623,  620,  623,  620,  623, 
          623,  623,  623,  623,  623,  623,  620,  623,  623,  623, 
          412,  620,  620,  620,  620,  620,  620,    0,    0,    0, 
          620,  623,  620,  620,    0,  620,  620,  620,  620,  122, 
          136,  620,  620,  620,  623,  621,  621,  621,    0,  621, 
          624,  624,  624,  621,  621,  624,  624,  624,  621,  624, 
          621,  621,  621,  621,  621,  621,  621,    0,  624,  624, 
          624,  621,  621,  621,  621,  621,  621,  621,  624,  624, 
          621,  624,  624,  624,  624,  624,    0,  621,  122,  136, 
          621,  621,  621,  624,  621,  621,  621,  621,  621,  621, 
          621,  621,  621,  621,  621,  624,  624,  624,  624,  624, 
          624,  624,  624,  624,  624,  624,  624,  624,  624,  101, 
          523,  624,  624,  624,  624,  621,    0,  624,  621,  621, 
          624,  621,  621,  624,  621,  624,  621,  624,  621,  624, 
          621,  624,  624,  624,  624,  624,  624,  624,  621,  624, 
          624,  624,    0,  621,  621,  621,  621,  621,  621,    0, 
            0,  314,  621,  624,  621,  621,  521,  621,  621,  621, 
          621,  314,    0,  621,  621,  621,  624,    0,    0,    0, 
            0,    0,    0,    0,    0,    0,    0,    0,    0,    0, 
            0,    0,    0,    0,    0,    0,    0,    0,    0,    0, 
            0,    0,    0,    0,  135,  314,  135,  135,  135,  135, 
          135,    0,    0,    0,    0,  135,    0,    0,    0,  314, 
          314,    0,  127,    0,  314,    0,  512,  387,  388,  389, 
          390,  391,  392,  393,  512,    0,  396,  397,  135,  135, 
            0,    0,    0,  400,  401,    0,  135,  135,  135,  135, 
            0,    0,  314,    0,  778,    0,    0,    0,    0,  402, 
            0,  403,    0,  404,  405,  406,  407,  408,  409,  410, 
            0,  411,    0,    0,  387,  388,  389,  390,  391,  392, 
          393,  394,  395,  396,  397,  135,  398,  399,    0,    0, 
          400,  401,    0,  644,    0,    0,  122,    0,    0,    0, 
            0,    0,    0,  644,    0,    0,  402,    0,  403,    0, 
          404,  405,  406,  407,  408,  409,  410,    0,  411,    0, 
            0,    0,    0,    0,    0,    0,    0,    0,    0,    0, 
            0,    0,    0,    0,    0,    0,  136,  644,  136,  136, 
          136,  136,  136,    0,    0,    0,    0,  136,    0,    0, 
            0,    0,  644,    0,  120,    0,  644,  523,  515,    0, 
            0,    0,    0,    0,  523,  523,  515,    0,    0,    0, 
          136,  136,    0,    0,    0,    0,    0,    0,  136,  136, 
          136,  136,    0,    0,  644,    0,    0,  305,    0,    0, 
            0,    0,    0,    0,    0,    0,    0,    0,    0,    0, 
            0,    0,    0,  521,    0,    0,    0,    0,    0,    0, 
          521,  521,    0,    0,    0,    0,    0,  136,  619,  619, 
          619,    0,  619,  314,  314,  314,  619,  619,  314,  314, 
          314,  619,  314,  619,  619,  619,  619,  619,  619,  619, 
          412,    0,  314,  314,  619,  619,  619,  619,  619,  619, 
          619,  314,  314,  619,  314,  314,  314,  314,  314,    0, 
          619,    0,    0,  619,  619,  619,  314,  619,  619,  619, 
          619,  619,  619,  619,  619,  619,  619,  619,  314,  314, 
          314,  314,  314,  314,  314,  314,  314,  314,  314,  314, 
          314,  314,  103,    0,  314,  314,  314,  314,  619,    0, 
          314,  619,  619,  314,  619,  619,  314,  619,  314,  619, 
          314,  619,  314,  619,  314,  314,  314,  314,  314,  314, 
          314,  619,  314,    0,  314,    0,  619,  619,  619,  619, 
          619,  619,  412,    0,    0,  619,  314,  619,  619,  523, 
          619,  619,  619,  619,  113,    0,  619,  619,  619,  314, 
          281,  281,  281,    0,  281,  644,  644,  644,  281,  281, 
          644,  644,  644,  281,  644,  281,  281,  281,  281,  281, 
          281,  281,    0,  644,  644,  644,  281,  281,  281,  281, 
          281,  281,  281,  644,  644,  281,  644,  644,  644,  644, 
          644,    0,  281,  113,    0,  281,  281,  281,    0,  281, 
          281,  281,  281,  281,  281,  281,  281,  281,  281,  281, 
          644,  644,  644,  644,  644,  644,  644,  644,  644,  644, 
          644,  644,  644,  644,  321,  514,  644,  644,  644,  644, 
          281,    0,  644,  281,  321,  644,  281,  281,  644,  281, 
          644,  281,  644,  281,  644,  281,  644,  644,  644,  644, 
          644,  644,  644,  281,  644,  644,  644,    0,  281,  281, 
          281,  281,  281,  281,    0,    0,    0,  281,  321,  281, 
          281,    0,  281,  281,  281,  281,    0,    0,  281,  281, 
          281,  644,    0,  321,    0,    0,    0,  321,    0,    0, 
            0,    0,    0,    0,  387,  388,  389,  390,  391,  392, 
          393,  394,  395,  396,  397,    0,  398,  399,    0,    0, 
          400,  401,    0,    0,    0,  321,    0,    0,    0,    0, 
            0,    0,    0,    0,    0,    0,  402,  122,  403,    0, 
          404,  405,  406,  407,  408,  409,  410,    0,  411,    0, 
            0,    0,    0,    0,    0,    0,    0,    0,    0,    0, 
            0,    0,    0,    0,    0,    0,  512,    0,    0,    0, 
            0,    0,    0,    0,    0,    0,  512,    0,    0,    0, 
            0,    0,    0,    0,    0,    0,  523,    0,    0,    0, 
            0,    0,    0,  523,  523,    0,  387,  388,  389,  390, 
          391,  392,  393,  394,  395,  396,  397,    0,  398,  399, 
           92,  113,  400,  401,    0,    0,    0,    0,    0,    0, 
            0,    0,    0,    0,    0,  512,    0,  111,  402,  512, 
          403,    0,  404,  405,  406,  407,  408,  409,  410,    0, 
          411,    0,    0,    0,    0,    0,    0,    0,    0,    0, 
            0,    0,    0,    0,    0,    0,    0,  512,    0,    0, 
            0,    0,    0,    0,    0,    0,    0,    0,    0,    0, 
            0,    0,  514,    0,    0,    0,    0,    0,    0,  514, 
          514,    0,    0,    0,    0,    0,    0,    0,    0,    0, 
            0,  281,  281,  281,    0,  281,  321,  321,  321,  281, 
          281,  321,  321,  321,  281,  321,  281,  281,  281,  281, 
          281,  281,  281,    0,    0,  321,  321,  281,  281,  281, 
          281,  281,  281,  281,  321,  321,  281,  321,  321,  321, 
          321,  321,    0,  281,    0,    0,  281,  281,  281,    0, 
          281,  281,  281,  281,  281,  281,  281,  281,  281,  281, 
          281,  321,  321,  321,  321,  321,  321,  321,  321,  321, 
          321,  321,  321,  321,  321,  863,    0,  321,  321,  321, 
          321,  281,    0,  321,  281,    0,  321,  281,  281,  321, 
          281,  321,  281,  321,  281,  321,  281,  321,  321,  321, 
          321,  321,  321,  321,  281,  321,    0,  321,  105,  281, 
          281,  281,  281,  281,  281,    0,    0,    0,  281,    0, 
          281,  281,    0,  281,  281,  281,  281,  124,    0,  281, 
          281,  281,  321,  615,  615,  615,    0,  615,  512,  512, 
          512,  615,  615,    0,  512,  512,  615,  512,  615,  615, 
          615,  615,  615,  615,  615,  618,  615,  858,    0,  615, 
          615,  615,  615,  615,  615,  615,  512,  512,  615,  512, 
          512,  512,  512,  512,  938,  615,  124,  938,  615,  615, 
          615,    0,  615,  615,  615,  615,  615,  615,  615,  615, 
          615,  615,  615,  512,  512,  512,  512,  512,  512,  512, 
          512,  512,  512,  512,  512,  512,  512,    0,  618,  512, 
          512,  512,  512,  615,  524,    0,  615,  615,  512,  615, 
          615,    0,  615,    0,  615,  512,  615,  512,  615,  512, 
          512,  512,  512,  512,  512,  512,  615,  512,  615,  512, 
            0,  615,  615,  615,  615,  615,  615,  926,    0,  515, 
          615,    0,  615,  615,  936,  615,  615,  615,  615,  515, 
            0,  615,  615,  615,  512,    0,  863,    0,    0,  863, 
            0,  938,    0,  863,    0,  611,  606,    0,    0,  612, 
            0,    0,    0,  184,  185,    0,  186,  187,  188,  189, 
          190,  191,  192,   95,    0,  193,  194,    0,    0,    0, 
          195,  196,  197,  198,    0,    0,    0,    0,  515,    0, 
          114,    0,  515,    0,    0,    0,    0,  200,  201,    0, 
          202,  203,  204,  205,  206,  207,  208,  209,  210,  211, 
          212,    0,    0,  213,    0,    0,    0,    0,    0,    0, 
          515,    0,    0,  124,  618,    0,    0,    0, 1012,    0, 
            0,    0,    0,    0,    0, 1020,  214,    0,    0,    0, 
            0,    0,    0,    0,    0,    0,    0,    0,    0,    0, 
            0,    0,    0,  863,    0,  863,    0,  863,    0,    0, 
          863,  622,    0,    0,  124,    0,    0,    0,    0,    0, 
            0,  622,  618,    0,    0,    0,    0,  618,    0,  618, 
          618,    0,    0,    0,    0,  618,    0,    0,    0,    0, 
            0, 1044,    0, 1046,  863, 1047,    0,    0,    0,    0, 
            0,    0,    0,    0,    0,  106,  618,    0,   94,    0, 
            0,    0,    0,    0,    0,    0,    0,    0,    0,    0, 
          622,    0,  125,    0,  622,  618,    0,    0,    0,    0, 
            0,    0,  618,  618,    0,    0,    0, 1079,  618, 1081, 
            0,    0, 1082,    0,    0,    0,    0,    0,    0,    0, 
            0,    0,  622,    0,    0,  514,    0,    0,    0,  618, 
            0,    0, 1087,    0,    0,    0,    0,    0,    0,    0, 
            0,    0,    0,    0,    0,    0, 1094,    0,    0,    0, 
            0,    0,    0,    0,    0,    0,  616,  616,  616,    0, 
          616,  515,  515,  515,  616,  616,    0,  515,  515,  616, 
          515,  616,  616,  616,  616,  616,  616,  616,    0,  616, 
            0,    0,  616,  616,  616,  616,  616,  616,  616,  515, 
          515,  616,  515,  515,  515,  515,  515,  107,  616,   93, 
            0,  616,  616,  616,    0,  616,  616,  616,  616,  616, 
          616,  616,  616,  616,  616,  616,  515,  515,  515,  515, 
          515,  515,  515,  515,  515,  515,  515,  515,  515,  515, 
            0,    0,  515,  515,  515,  515,  616,  527,    0,  616, 
          616,  515,  616,  616,  619,  616,  513,  616,  515,  616, 
          515,  616,  515,  515,  515,  515,  515,  515,  515,  616, 
          515,  616,  515,   96,  616,  616,  616,  616,  616,  616, 
            0,    0,    0,  616,    0,  616,  616,    0,  616,  616, 
          616,  616,  126,    0,  616,  616,  616,  515,  618,  618, 
          618,    0,  618,  622,  622,  622,  618,  618,    0,  622, 
          622,  618,  622,  618,  618,  618,  618,  618,  618,  618, 
          516,  622,    0,  113,  618,  618,  618,  618,  618,  618, 
          618,  622,  622,  618,  622,  622,  622,  622,  622,    0, 
          618,  126,    0,  618,  618,  618,    0,  618,  618,  618, 
          618,  618,  618,  618,  618,  618,  618,  618,  622,  622, 
          622,  622,  622,  622,  622,  622,  622,  622,  622,  622, 
          622,  622,  514,  619,  622,  622,  622,  622,  618,  514, 
          514,  618,  618,  622,  618,  618,    0,  618,    0,  618, 
          622,  618,  622,  618,  622,  622,  622,  622,  622,  622, 
          622,  618,  622,  622,  622,    0,  618,  618,  618,  618, 
          618,  618,    0,    0,  618,  618,    0,  618,  618,    0, 
          618,  618,  618,  618,  618,    0,  618,  618,  618,  622, 
          387,  388,  389,  390,  391,  392,  393,  394,  395,  396, 
          397,    0,  126,  619,  112,    0,  400,  401,    0,    0, 
            0,    0,    0,    0,    0,    0,    0,    0,  105,    0, 
            0,    0,  402,    0,  403,    0,  404,  405,  406,  407, 
          408,  409,  410,  618,  411,  124,    0,  618,  387,  388, 
          389,  390,  391,  392,  393,  394,    0,  396,  397,    0, 
            0,  619,    0,  513,  400,  401,    0,    0,  619,  619, 
          513,  513,    0,    0,  619,  618,    0,    0,  115,    0, 
          402,    0,  403,    0,  404,  405,  406,  407,  408,  409, 
          410,    0,  411,    0,    0,  619,    0,    0,    0,    0, 
            0,    0,    0,    0,    0,    0,    0,    0,    0,    0, 
            0,    0,    0,    0,    0,    0,  619,    0,    0,  126, 
            0,    0,    0,    0,    0,    0,  619,  516,    0,    0, 
            0,    0,  619,    0,  516,  516,    0,    0,    0,    0, 
            0,    0,    0,    0,    0,    0,    0,    0,    0,    0, 
            0,    0,    0,    0,    0,    0,    0,    0,    0,    0, 
          107,    0,    0,  110,    0,    0,    0,    0,    0,    0, 
            0,    0,    0,    0,    0,  619,    0,  126,    0,  619, 
          619,    0,    0,    0,    0,    0,    0,  619,  619,    0, 
            0,    0,    0,  619,    0,    0,    0,    0,    0,    0, 
            0,    0,    0,    0,    0,    0,    0,  619,    0,    0, 
          304,    0,    0,    0,  619,    0,    0,    0,    0,    0, 
            0,    0,    0,    0,    0,    0,    0,    0,    0,    0, 
            0,    0,    0,    0,    0,    0,    0,    0,    0,    0, 
            0,  618,  618,  618,    0,  618,  618,  618,  618,  618, 
          618,    0,  618,  618,  618,  618,  618,  618,  618,  618, 
          618,  618,  618,    0,  618,    0,    0,  618,  618,  618, 
          618,  618,  618,  618,  618,  618,  618,  618,  618,  618, 
          618,  618,  109,  618,    0,    0,  618,  618,  618,    0, 
          618,  618,  618,  618,  618,  618,  618,  618,  618,  618, 
          618,  618,  618,  618,  618,  618,  618,  618,  618,  618, 
          618,  618,  618,  618,  618,    0,    0,  618,  618,  618, 
          618,  618,    0,    0,  618,  618,  618,  618,  618,  315, 
          618,    0,  618,  618,  618,  618,  618,  618,  618,  618, 
          618,  618,  618,  618,  618,  618, 
      };
   }

   private static final short[] yyTable2() {
      return new short[] {

          618,  618,    0,  618,  618,  618,  618,  618,  618,    0, 
            0,  644,  618,    0,  618,  618,    0,  618,  618,  618, 
          618,  644,    0,  618,  618,  618,  618,  619,  619,  619, 
            0,  619,  619,  619,  619,  619,  619,    0,  619,  619, 
          619,  619,  619,  619,  619,  619,  619,  619,  619,    0, 
          619,    0,  129,  619,  619,  619,  619,  619,  619,  619, 
          619,  619,  619,  619,  619,  619,  619,  619,    0,  619, 
          644,    0,  619,  619,  619,    0,  619,  619,  619,  619, 
          619,  619,  619,  619,  619,  619,  619,  619,  619,  619, 
          619,  619,  619,  619,  619,  619,  619,  619,  619,  619, 
          619,  304,    0,  619,  619,  619,  619,  619,  304,  304, 
          619,  619,  619,  619,  619,    0,  619,    0,  619,  619, 
          619,  619,  619,  619,  619,  619,  619,  619,  619,  619, 
          619,  619,  619,  619,    0,  619,  619,  619,  619,  619, 
          619,    0,    0,  314,  619,    0,  619,  619,    0,  619, 
          619,  619,  619,  314,    0,  619,  619,  619,  619,    0, 
            0,    0,    0,    0,    0,    0,    0,    0,    0,    0, 
            0,  128,    0,    0,    0,    0,    0,    0,    0,    0, 
            0,    0,    0,    0,    0,    0,    0,  108,    0,    0, 
            0,    0,    0,    0,    0,    0,    0,    0,    0,    0, 
            0,    0,  314,    0,  127,    0,  314,    0,    0,    0, 
            0,    0,    0,    0,    0,    0,    0,    0,    0,    0, 
          315,    0,    0,    0,    0,    0,    0,  315,  315,    0, 
            0,    0,    0,    0,  314,    0,    0,    0,    0,    0, 
            0,    0,    0,    0,    0,    0,    0,    0,    0,    0, 
            0,    0,    0,    0,    0,    0,    0,    0,    0,    0, 
            0,    0,    0,    0,    0,    0,    0,    0,  281,  281, 
          281,    0,  281,  644,  644,  644,  281,  281,  644,  644, 
          644,  281,  644,  281,  281,  281,  281,  281,  281,  281, 
            0,  644,  644,    0,  281,  281,  281,  281,  281,  281, 
          281,  644,  644,  281,  644,  644,  644,  644,  644,    0, 
          281,    0,    0,  281,  281,  281,    0,  281,  281,  281, 
          281,  281,  281,  281,  281,  281,  281,  281,    0,    0, 
          645,    0,    0,    0,    0,    0,    0,    0,    0,  644, 
          645,    0,    0,    0,    0,    0,  644,  644,  281,    0, 
            0,  281,    0,  644,  281,  281,    0,  281,    0,  281, 
            0,  281,    0,  281,    0,    0,    0,    0,    0,    0, 
            0,  281,    0,  644,  644,    0,  281,  281,  281,  281, 
          281,  281,    0,    0,    0,  281,    0,  281,  281,  645, 
          281,  281,  281,  281,    0,    0,  281,  281,  281,  644, 
          619,  619,  619,    0,  619,  314,  314,  314,  619,  619, 
            0,  314,  314,  619,  314,  619,  619,  619,  619,  619, 
          619,  619,    0,    0,    0,    0,  619,  619,  619,  619, 
          619,  619,  619,  314,  314,  619,  314,  314,  314,  314, 
          314,    0,  619,    0,    0,  619,  619,  619,    0,  619, 
          619,  619,  619,  619,  619,  619,  619,  619,  619,  619, 
          314,  314,  314,  314,  314,  314,  314,  314,  314,  314, 
          314,  314,  314,  314,    0,    0,  314,  314,  314,  314, 
          619,    0,    0,  619,  619,  314,  619,  619,  112,  619, 
          115,  619,  314,  619,  314,  619,  314,  314,  314,  314, 
          314,  314,  314,  619,  314,  436,  314,    0,  619,  619, 
          619,  619,  619,  619,    0,  436,    0,  619,    0,  619, 
          619,    0,  619,  619,  619,  619,    0,    0,  619,  619, 
          619,  314,    0,    0,    0,    0,    0,  112,    0,  115, 
            0,    0,    0,    0,    0,    0,    0,    0,    0,  436, 
            0,    0,    0,    0,    0,    0,    0,    0,    0,    0, 
            0,    0,    0,  436,  436,    0,  123,    0,  436,  513, 
            0,  516,    0,    0,    0,    0,    0,    0,    0,    0, 
            0,    0,    0,    0,    0,    0,    0,    0,    0,    0, 
            0,    0,  645,  645,  645,    0,  436,  645,  645,  645, 
            0,  645,    0,    0,    0,    0,    0,  435,    0,    0, 
            0,  645,  645,    0,    0,    0,    0,  435,    0,    0, 
          645,  645,    0,  645,  645,  645,  645,  645,    0,    0, 
            0,    0,    0,    0,    0,    0,    0,    0,    0,    0, 
            0,    0,    0,    0,    0,    0,    0,    0,    0,    0, 
            0,  435,    0,    0,    0,    0,    0,    0,    0,    0, 
            0,  129,    0,    0,    0,  435,  435,    0,    0,    0, 
          435,    0,  645,    0,    0,  645,    0,  645,    0,    0, 
            0,  100,    0,    0,    0,    0,    0,    0,    0,    0, 
            0,    0,    0,  645,    0,    0,    0,    0,  435,    0, 
            0,    0,    0,    0,    0,    0,    0,    0,    0,    0, 
          129,    0,    0,    0,    0,  100,    0,    0,  645,    0, 
            0,  518,    0,    0,    0,    0,    0,    0,    0,    0, 
            0,  518,  100,    0,    0,    0,    0,    0,    0,    0, 
            0,    0,  304,    0,    0,  112,    0,  115,    0,    0, 
            0,    0,    0,    0,    0,    0,    0,    0,    0,    0, 
            0,    0,  520,    0,    0,  518,    0,  436,  436,  436, 
            0,    0,  436,  436,  436,    0,  436,    0,    0,  518, 
          518,    0,  117,    0,  518,  436,  436,  436,    0,    0, 
            0,    0,    0,    0,    0,  436,  436,    0,  436,  436, 
          436,  436,  436,    0,    0,    0,  513,    0,  516,    0, 
          436,    0,  518,  513,  513,  516,  516,    0,    0,    0, 
            0,    0,  436,  436,  436,  436,  436,  436,  436,  436, 
          436,  436,  436,  436,  436,  436,  128,    0,  436,  436, 
          436,  436,    0,    0,  436,    0,    0,  436,    0,    0, 
          436,    0,  436,    0,  436,    0,  436,    0,  436,  436, 
          436,  436,  436,  436,  436,    0,  436,  436,  436,  435, 
          435,  435,    0,    0,  435,  435,  435,    0,  435,    0, 
          436,    0,    0,    0,    0,  128,    0,  435,  435,  435, 
            0,    0,    0,  436,    0,    0,    0,  435,  435,    0, 
          435,  435,  435,  435,  435,    0,    0,    0,    0,    0, 
            0,    0,  435,  517,    0,    0,    0,  315,  129,    0, 
            0,    0,    0,  517,  435,  435,  435,  435,  435,  435, 
          435,  435,  435,  435,  435,  435,  435,  435,    0,    0, 
          435,  435,  435,  435,    0,    0,  435,    0,    0,  435, 
          100,    0,  435,    0,  435,    0,  435,  517,  435,    0, 
          435,  435,  435,  435,  435,  435,  435,    0,  435,  435, 
          435,  517,  517,    0,  116,    0,  517,    0,    0,  304, 
            0,    0,  435,  518,  518,  518,  304,  304,  518,  518, 
          518,    0,  518,    0,    0,  435,    0,    0,    0,  520, 
            0,    0,  518,  518,  517,    0,  520,  520,    0,    0, 
            0,  518,  518,  100,  518,  518,  518,  518,  518,    0, 
            0,    0,    0,    0,    0,    0,  518,  519,    0,    0, 
            0,    0,    0,    0,    0,    0,    0,  519,  518,  518, 
          518,  518,  518,  518,  518,  518,  518,  518,  518,  518, 
          518,  518,    0,    0,  518,  518,  518,  518,    0,  530, 
          518,    0,    0,  518,    0,    0,  518,    0,  518,  102, 
          518,  519,  518,    0,  518,  518,  518,  518,  518,  518, 
          518,    0,  518,    0,  518,  519,  519,    0,  118,    0, 
          519,    0,    0,  128,    0,    0,  518,    0,    0,    0, 
            0,    0,    0,  102,    0,    0,    0,    0,    0,  518, 
            0,    0,    0,    0,    0,    0,    0,    0,  519,    0, 
          102,    0,    0,    0,    0,    0,    0,    0,    0,  520, 
            0,    0,    0,    0,    0,    0,    0,    0,    0,  520, 
            0,    0,    0,    0,    0,    0,    0,    0,    0,    0, 
          522,    0,    0,    0,  315,    0,    0,    0,    0,    0, 
            0,  315,  315,    0,    0,    0,    0,    0,    0,    0, 
            0,    0,    0,  520,    0,  517,  517,  517,    0,    0, 
          517,  517,  517,    0,  517,    0,    0,  520,  520,    0, 
          119,    0,  520,    0,  517,  517,    0,    0,    0,    0, 
            0,    0,    0,  517,  517,    0,  517,  517,  517,  517, 
          517,    0,    0,    0,    0,    0,    0,    0,  517,    0, 
          520,    0,    0,    0,    0,    0,    0,    0,    0,    0, 
          517,  517,  517,  517,  517,  517,  517,  517,  517,  517, 
          517,  517,  517,  517,    0,    0,  517,  517,  517,  517, 
            0,  529,  517,    0,    0,  517,    0,    0,  517,    0, 
          517,    0,  517,    0,  517,    0,  517,  517,  517,  517, 
          517,  517,  517,    0,  517,    0,  517,    0,    0,    0, 
            0,    0,  260,    0,    0,    0,    0,    0,  517,  519, 
          519,  519,  260,    0,  519,  519,  519,    0,  519,    0, 
            0,  517,    0,    0,    0,    0,    0,    0,  519,  519, 
            0,    0,    0,    0,    0,    0,    0,  519,  519,    0, 
          519,  519,  519,  519,  519,    0,  260,    0,    0,    0, 
            0,    0,  519,    0,    0,    0,    0,    0,  102,    0, 
          260,  260,    0,    0,  519,  519,  519,  519,  519,  519, 
          519,  519,  519,  519,  519,  519,  519,  519,    0,    0, 
          519,  519,  519,  519,    0,  531,  519,    0,    0,  519, 
            0,    0,  519,    0,  519,    0,  519,    0,  519,    0, 
          519,  519,  519,  519,  519,  519,  519,  522,  519,    0, 
          519,  520,  520,  520,  522,  522,  520,  520,  520,    0, 
          520,  102,  519,    0,    0,    0,    0,    0,    0,    0, 
          520,  520,    0,    0,    0,  519,    0,    0,    0,  520, 
          520,    0,  520,  520,  520,  520,  520,    0,    0,    0, 
            0,    0,    0,    0,  520,  522,    0,    0,    0,    0, 
            0,    0,    0,    0,    0,  522,  520,  520,  520,  520, 
          520,  520,  520,  520,  520,  520,  520,  520,  520,  520, 
            0,    0,  520,  520,  520,  520,    0,  532,  520,    0, 
            0,  520,    0,    0,  520,    0,  520,  101,  520,  522, 
          520,    0,  520,  520,  520,  520,  520,  520,  520,    0, 
          520,    0,  520,  522,  522,    0,  121,    0,  522,    0, 
            0,    0,    0,    0,  520,    0,    0,    0,    0,    0, 
            0,  101,    0,    0,    0,    0,    0,  520,    0,    0, 
            0,    0,    0,    0,    0,    0,  522,    0,  101,    0, 
            0,    0,    0,    0,    0,    0,    0,  521,    0,    0, 
            0,    0,    0,    0,  260,  260,  260,  521,    0,  260, 
          260,  260,    0,  260,    0,    0,    0,    0,  521,    0, 
            0,    0,    0,  260,  260,    0,    0,    0,    0,    0, 
            0,    0,  260,  260,    0,  260,  260,  260,  260,  260, 
            0,  521,    0,    0,    0,    0,    0,  260,    0,    0, 
            0,    0,    0,    0,    0,  521,  521,    0,  120,    0, 
          521,    0,    0,    0,    0,  103,    0,    0,    0,    0, 
            0,    0,    0,    0,    0,    0,    0,    0,    0,    0, 
            0,  260,    0,    0,  260,    0,    0,  260,  521,  260, 
            0,    0,    0,    0,   94,    0,    0,    0,    0,  103, 
            0,    0,    0,    0,    0,  260,    0,    0,    0,    0, 
            0,  523,    0,    0,    0,    0,  103,  260,    0,    0, 
            0,  523,    0,    0,    0,    0,    0,    0,   94,    0, 
          260,    0,    0,    0,    0,    0,    0,    0,    0,    0, 
            0,    0,    0,    0,    0,   94,  523,    0,    0,    0, 
            0,    0,    0,    0,    0,  523,    0,  522,  522,  522, 
            0,    0,  522,  522,  522,    0,  522,    0,    0,  523, 
          523,    0,  122,    0,  523,  514,  522,  522,    0,    0, 
            0,    0,    0,    0,    0,  522,  522,    0,  522,  522, 
          522,  522,  522,    0,    0,    0,    0,    0,    0,    0, 
          522,    0,  523,    0,    0,    0,  101,    0,    0,    0, 
            0,    0,  522,  522,  522,  522,  522,  522,  522,  522, 
          522,  522,  522,  522,  522,  522,    0,    0,  522,  522, 
          522,  522,    0,  534,  522,    0,    0,  522,    0,    0, 
          522,    0,  522,    0,  522,    0,  522,    0,  522,  522, 
          522,  522,  522,  522,  522,  521,  522,    0,  522,  521, 
          521,  521,  521,  521,  521,  521,  521,    0,  521,  101, 
          522,    0,    0,    0,    0,    0,    0,    0,  521,  521, 
            0,    0,    0,  522,    0,    0,    0,  521,  521,    0, 
          521,  521,  521,  521,  521,    0,    0,    0,    0,    0, 
            0,    0,  521,  514,    0,    0,    0,    0,    0,    0, 
            0,    0,    0,  514,  521,  521,  521,  521,  521,  521, 
          521,  521,  521,  521,  521,  521,  521,  521,    0,    0, 
          521,  521,  521,  521,  103,  533,  521,    0,    0,  521, 
            0,    0,  521,    0,  521,    0,  521,  514,  521,    0, 
          521,  521,  521,  521,  521,  521,  521,    0,  521,    0, 
          521,  514,  514,   94,  113,    0,  514,    0,    0,    0, 
            0,    0,  521,  523,  523,  523,    0,    0,  523,  523, 
          523,    0,  523,  523,    0,  521,    0,    0,    0,    0, 
          523,  523,  523,  523,  514,    0,    0,  103,    0,    0, 
            0,  523,  523,    0,  523,  523,  523,  523,  523,    0, 
            0,    0,  514,    0,    0,    0,  523,  513,    0,  514, 
          514,    0,    0,    0,    0,    0,   94,  513,  523,  523, 
          523,  523,  523,  523,  523,  523,  523,  523,  523,  523, 
          523,  523,    0,    0,  523,  523,  523,  523,    0,  535, 
          523,    0,    0,  523,    0,    0,  523,    0,  523,   93, 
          523,  513,  523,    0,  523,  523,  523,  523,  523,  523, 
          523,    0,  523,    0,  523,  513,  513,    0,  112,    0, 
          513,    0,    0,    0,    0,    0,  523,    0,    0,    0, 
            0,    0,    0,   93,    0,    0,    0,    0,    0,  523, 
            0,    0,    0,    0,    0,    0,    0,    0,  513,    0, 
           93,    0,    0,    0,    0,    0,    0,    0,    0,  516, 
            0,    0,    0,    0,    0,    0,    0,    0,    0,  516, 
            0,    0,    0,    0,    0,    0,    0,    0,    0,    0, 
          513,    0,    0,    0,    0,    0,    0,    0,    0,    0, 
            0,    0,    0,    0,    0,    0,    0,    0,    0,    0, 
            0,    0,    0,  516,    0,  514,  514,  514,    0,    0, 
          514,  514,  514,    0,  514,    0,    0,  516,  516,    0, 
          115,    0,  516,    0,  514,  514,    0,    0,    0,    0, 
            0,    0,    0,  514,  514,    0,  514,  514,  514,  514, 
          514,    0,    0,    0,    0,    0,    0,    0,  514,    0, 
          516,    0,    0,    0,    0,    0,    0,    0,    0,    0, 
          514,  514,  514,  514,  514,  514,  514,  514,  514,  514, 
          514,  514,  514,  514,    0,    0,  514,  514,  514,  514, 
            0,  526,  514,    0,    0,  514,    0,    0,  514,    0, 
          514,    0,  514,    0,  514,    0,  514,  514,  514,  514, 
          514,  514,  514,    0,  514,    0,  514,    0,    0,    0, 
            0,    0,  229,    0,    0,    0,    0,    0,  514,  513, 
          513,  513,  229,    0,  513,  513,  513,    0,  513,    0, 
            0,  514,    0,    0,    0,    0,    0,    0,  513,  513, 
            0,    0,    0,    0,    0,    0,    0,  513,  513,    0, 
          513,  513,  513,  513,  513,    0,  229,    0,    0,    0, 
            0,    0,  513,    0,    0,    0,    0,    0,   93,    0, 
          229,  229,    0,    0,  513,  513,  513,  513,  513,  513, 
          513,  513,  513,  513,  513,  513,  513,  513,    0,    0, 
          513,  513,  513,  513,    0,  525,  513,    0,    0,  513, 
            0,    0,  513,    0,  513,    0,  513,    0,  513,    0, 
          513,  513,  513,  513,  513,  513,  513,  513,  513,    0, 
          513,  516,  516,  516,  513,  513,  516,  516,  516,    0, 
          516,   93,  513,    0,    0,    0,    0,    0,    0,    0, 
          516,  516,    0,    0,    0,  513,    0,    0,    0,  516, 
          516,    0,  516,  516,  516,  516,  516,    0,    0,    0, 
            0,    0,    0,    0,  516,  644,    0,    0,    0,    0, 
            0,    0,    0,    0,    0,  644,  516,  516,  516,  516, 
          516,  516,  516,  516,  516,  516,  516,  516,  516,  516, 
            0,    0,  516,  516,  516,  516,    0,  528,  516,    0, 
            0,  516,    0,    0,  516,    0,  516,   96,  516,  644, 
          516,    0,  516,  516,  516,  516,  516,  516,  516,    0, 
          516,    0,  516,  644,  644,    0,    0,    0,  644,    0, 
            0,    0,    0,    0,  516,    0,    0,    0,    0,    0, 
            0,   96,    0,    0,    0,    0,    0,  516,    0,    0, 
            0,    0,    0,    0,    0,    0,  644,    0,   96,    0, 
            0,    0,    0,    0,    0,    0,    0,  304,    0,    0, 
            0,    0,    0,    0,  229,  229,  229,  304,    0,  229, 
          229,  229,    0,  229,    0,    0,    0,    0,  516,    0, 
            0,    0,    0,  229,  229,    0,    0,    0,    0,    0, 
            0,    0,  229,  229,    0,  229,  229,  229,  229,  229, 
            0,  304,    0,    0,    0,    0,    0,  229,    0,    0, 
            0,    0,    0,    0,    0,  304,  304,    0,  129,    0, 
          304,    0,    0,    0,    0,  110,    0,    0,    0,    0, 
            0,    0,    0,    0,    0,    0,    0,    0,    0,    0, 
            0,  229,    0,    0,  229,    0,    0,  229,  304,  229, 
            0,    0,    0,    0,    0,    0,    0,    0,    0,  110, 
            0,    0,    0,    0,    0,  229,    0,    0,    0,    0, 
            0,  315,    0,    0,    0,    0,  110,  229,    0,    0, 
            0,  315,    0,    0,    0,    0,    0,    0,    0,    0, 
          229,    0,    0,    0,    0,    0,    0,    0,    0,    0, 
            0,    0,    0,    0,    0,    0,  304,    0,    0,    0, 
            0,    0,    0,    0,    0,  315,    0,  644,  644,  644, 
            0,    0,  644,  644,  644,    0,  644,    0,    0,  315, 
          315,    0,  128,    0,  315,  644,  644,  644,    0,    0, 
            0,    0,    0,    0,    0,  644,  644,    0,  644,  644, 
          644,  644,  644,    0,    0,    0,    0,    0,    0,    0, 
          644,    0,  315,    0,    0,    0,   96,    0,    0,    0, 
            0,    0,  644,  644,  644,  644,  644,  644,  644,  644, 
          644,  644,  644,  644,  644,  644,    0,    0,  644,  644, 
          644,  644,    0,    0,  644,    0,    0,  644,    0,    0, 
          644,    0,  644,    0,  644,    0,  644,    0,  644,  644, 
          644,  644,  644,  644,  644,  516,  644,  644,  644,  304, 
          304,  304,  516,  516,  304,  304,  304,    0,  304,   96, 
          644,    0,    0,    0,    0,    0,    0,    0,  304,  304, 
            0,    0,    0,  644,    0,    0,    0,  304,  304,    0, 
          304,  304,  304,  304,  304,    0,    0,    0,    0,    0, 
            0,    0,  304,  456,    0,    0,    0,    0,    0,    0, 
            0,    0,    0,  456,  304,  304,  304,  304,  304,  304, 
          304,  304,  304,  304,  304,  304,  304,  304,    0,    0, 
          304,  304,  304,  304,  110,    0,  304,    0,    0,  304, 
            0,    0,  304,    0,  304,    0,  304,  456,  304,    0, 
          304,  304,  304,  304,  304,  304,  304,    0,  304,    0, 
          304,  456,  456,    0,    0,    0,  456,    0,    0,    0, 
            0,    0,  304,  315,  315,  315,    0,    0,  315,  315, 
          315,    0,  315,  304,    0,  304,    0,    0,    0,    0, 
          304,  304,  315,  315,  456,    0,    0,  110,    0,    0, 
            0,  315,  315,    0,  315,  315,  315,  315,  315,    0, 
            0,    0,    0,    0,    0,    0,  315,  326,    0,    0, 
            0,    0,    0,    0,    0,    0,    0,  326,  315,  315, 
          315,  315,  315,  315,  315,  315,  315,  315,  315,  315, 
          315,  315,    0,    0,  315,  315,  315,  315,    0,    0, 
          315,    0,    0,  315,    0,    0,  315,    0,  315,    0, 
          315,  326,  315,    0,  315,  315,  315,  315,  315,  315, 
          315,    0,  315,    0,  315,  326,  326,    0,    0,    0, 
          326,    0,    0,    0,    0,    0,  315,    0,    0,    0, 
            0,    0,    0,    0,    0,    0,    0,    0,    0,  315, 
            0,    0,    0,    0,    0,    0,    0,    0,  326,    0, 
            0,    0,    0,    0,    0,    0,    0,    0,    0,  262, 
            0,    0,    0,    0,    0,    0,    0,    0,    0,  262, 
            0,    0,    0,    0,    0,    0,    0,    0,    0,    0, 
            0,    0,    0,    0,    0,    0,    0,    0,    0,    0, 
            0,    0,    0,    0,    0,    0,    0,    0,    0,    0, 
            0,    0,    0,  262,    0,  456,  456,  456,    0,    0, 
          456,  456,  456,    0,  456,    0,    0,  262,  262,    0, 
            0,    0,  262,    0,  456,  456,    0,    0,    0,    0, 
            0,    0,    0,  456,  456,    0,  456,  456,  456,  456, 
          456,    0,    0,    0,    0,    0,    0,    0,  456,    0, 
          358,    0,    0,    0,    0,    0,    0,    0,    0,    0, 
          456,  456,  456,  456,  456,  456,  456,  456,  456,  456, 
          456,  456,  456,  456,    0,    0,  456,  456,  456,  456, 
            0,    0,  456,    0,    0,  456,    0,    0,  456,    0, 
          456,    0,  456,    0,  456,    0,  456,  456,  456,  456, 
          456,  456,  456,    0,  456,    0,  456,    0,    0,    0, 
            0,    0,  228,    0,    0,    0,    0,    0,  456,  326, 
          326,  326,  228,    0,  326,  326,  326,    0,  326,    0, 
            0,  456,    0,    0,    0,    0,    0,    0,  326,  326, 
            0,    0,    0,    0,    0,    0,    0,  326,  326,    0, 
          326,  326,  326,  326,  326,    0,  228,    0,    0,    0, 
            0,    0,  326,    0,    0,    0,  106,    0,    0,    0, 
          228,  228,    0,    0,  326,  326,  326,  326,  326,  326, 
          326,  326,  326,  326,  326,  326,  326,  326,    0,    0, 
          326,  326,  326,  326,    0,    0,  326,    0,    0,  326, 
          106,    0,  326,    0,  326,    0,  326,    0,  326,    0, 
          326,  326,  326,  326,  326,  326,  326,  106,  326,    0, 
          326,  262,  262,  262,    0,    0,  262,  262,  262,    0, 
          262,    0,  326,    0,    0,    0,    0,    0,    0,    0, 
          262,  262,    0,    0,    0,  326,    0,  622,    0,  262, 
          262,    0,  262,  262,  262,  262,  262,    0,    0,    0, 
            0,    0,    0,    0,  262,  321,    0,    0,    0,    0, 
            0,    0,    0,    0,    0,  321,  262,  262,  262,  262, 
          262,  262,  262,  262,  262,  262,  262,  358,  262,  262, 
            0,    0,  262,  262,  358,  358,    0,    0,  262,    0, 
            0,  262,    0,    0,  262,    0,  262,    0,  262,  321, 
          262,    0,  262,  262,  262,  262,  262,  262,  262,    0, 
          262,    0,  262,  321,  321,    0,    0,    0,  321,    0, 
            0,    0,    0,    0,  262,    0,    0,    0,    0,    0, 
            0,    0,    0,    0,    0,    0,    0,  262,    0,    0, 
            0,    0,    0,    0,    0,    0,  321,    0,    0,    0, 
            0,    0,    0,    0,    0,    0,    0,  507,    0,    0, 
            0,    0,    0,    0,  228,  228,  228,  507,    0,  228, 
          228,  228,    0,  228,    0,    0,    0,    0,    0,    0, 
            0,    0,    0,  228,  228,    0,    0,    0,    0,    0, 
            0,    0,  228,  228,    0,  228,  228,  228,  228,  228, 
            0,  507,    0,    0,    0,    0,    0,  228,    0,    0, 
            0,    0,    0,    0,    0,  507,  507,    0,    0,    0, 
          507,    0,    0,    0,    0,  106,  622,    0,    0,    0, 
            0,    0,    0,    0,    0,    0,    0,    0,    0,    0, 
            0,  228,    0,    0,  228,    0,    0,  228,  507,  228, 
            0,    0,    0,    0,    0,    0,    0,    0,    0,    0, 
            0,    0,  354,    0,    0,  228,    0,    0,    0,    0, 
            0,    0,  354,    0,  622,    0,    0,  228,    0,    0, 
            0,  622,  622,    0,    0,    0,    0,  618,  106,    0, 
          228,    0,    0,    0,    0,    0,    0,    0,    0,    0, 
            0,    0,    0,    0,    0,    0,  354,    0,  622,    0, 
            0,    0,    0,    0,    0,    0,    0,  321,  321,  321, 
            0,  354,  321,  321,  321,  354,  321,    0,    0,    0, 
            0,    0,    0,    0,    0,    0,  321,  321,    0,    0, 
            0,    0,    0,    0,    0,  321,  321,    0,  321,  321, 
          321,  321,  321,  354,    0,    0,    0,    0,    0,    0, 
          321,    0,    0,    0,    0,    0,    0,    0,    0,    0, 
            0,    0,  321,  321,  321,  321,  321,  321,  321,  321, 
          321,  321,  321,  321,  321,  321,    0,    0,  321,  321, 
          321,  321,    0,  105,  321,    0,    0,  321,    0,    0, 
          321,    0,  321,    0,  321,  259,  321,    0,  321,  321, 
          321,  321,  321,  321,  321,  259,  321,    0,  321,  507, 
          507,  507,    0,    0,  507,  507,  507,  105,  507,    0, 
          321,    0,    0,    0,    0,    0,    0,    0,  507,  507, 
            0,    0,    0,  321,  105,    0,    0,  507,  507,  259, 
          507,  507,  507,  507,  507,    0,    0,    0,    0,    0, 
            0,    0,  507,  259,  259,    0,    0,    0,  259,    0, 
            0,    0,    0,  515,  618,  507,  507,  507,  507,  507, 
          507,  507,  507,  507,  507,  507,  507,  507,    0,  107, 
          507,  507,  507,  507,    0,    0,  507,    0,    0,  507, 
            0,    0,  507,    0,  507,    0,  507,  515,  507,    0, 
          507,  507,  507,  507,  507,  507,  507,    0,  507,    0, 
          507,    0,  515,  107,  354,  354,  354,    0,    0,  354, 
          354,  354,  507,  354,    0,    0,    0,    0,    0,    0, 
          107,    0,    0,  354,  354,  507,    0,    0,    0,    0, 
            0,    0,  354,  354,  515,  354,  354,  354,  354,  354, 
            0,    0,    0,    0,    0,    0,    0,    0,  355,    0, 
          619,    0,    0,    0,    0,    0,    0,    0,  355,  354, 
          354,  354,  354,  354,  354,  354,  354,  354,  354,  354, 
          354,  354,  354,    0,    0,  354,  354,  354,  354,    0, 
            0,  354,    0,    0,  354,    0,    0,  354,    0,  354, 
            0,  354,  355,  354,    0,  354,  354,  354,  354,  354, 
          354,  354,    0,  354,    0,  354,    0,  355,    0,    0, 
            0,  355,    0,    0,    0,    0,    0,    0,    0,    0, 
            0,    0,    0,    0,    0,    0,    0,    0,    0,    0, 
          354,    0,    0,    0,    0,    0,    0,    0,    0,  355, 
            0,    0,  105,  618,    0,    0,    0,  259,  259,  259, 
          318,    0,  259,  259,  259,    0,  259,    0,    0,    0, 
          318,    0,    0,    0,    0,    0,  259,  259,    0,    0, 
            0,    0,    0,    0,    0,  259,  259,    0,  259,  259, 
          259,  259,  259,    0,    0,    0,    0,    0,    0,    0, 
          259,  618,    0,    0,  318,    0,    0,    0,  618,  618, 
            0,    0,    0,    0,  618,  105,    0,    0,    0,  318, 
          259,    0,    0,  318,  259,  259,    0,    0,    0,    0, 
            0,    0,    0,  616,  259,  618,    0,  259,    0,    0, 
          259,    0,  259,    0,    0,    0,    0,    0,  107,  619, 
            0,  318,    0,    0,    0,    0,    0,    0,  259,    0, 
            0,    0,    0,    0,    0,  240,    0,    0,    0,    0, 
          259,    0,    0,    0,    0,  240,    0,    0,    0,    0, 
            0,  515,    0,  259,    0,    0,    0,    0,  515,  515, 
            0,    0,    0,    0,  616,    0,    0,  619,    0,    0, 
            0,    0,    0,    0,  619,  619,    0,    0,    0,  240, 
          619,  107,  515,    0,    0,  616,    0,    0,    0,    0, 
          355,  355,  355,  240,  240,  355,  355,  355,  240,  355, 
            0,  619,    0,    0,    0,    0,    0,    0,    0,  355, 
          355,    0,    0,    0,    0,    0,    0,    0,  355,  355, 
            0,  355,  355,  355,  355,  355,    0,    0,    0,    0, 
            0,    0,    0,    0,    0,    0,    0,    0,    0,    0, 
            0,    0,    0,    0,    0,  355,  355,  355,  355,  355, 
          355,  355,  355,  355,  355,  355,  355,  355,  355,    0, 
            0,  355,  355,  355,  355,    0,    0,  355,    0,    0, 
          355,    0,    0,  355,    0,  355,    0,  355,  237,  355, 
            0,  355,  355,  355,  355,  355,  355,  355,  237,  355, 
            0,  355,  318,  318,  318,    0,    0,  318,  318,  318, 
            0,  318,    0,    0,    0,    0,    0,    0,    0,    0, 
            0,  318,  318,    0,    0,    0,  355,    0,    0,    0, 
          318,  318,  237,  318,  318,  318,  318,  318,    0,    0, 
            0,    0,    0,    0,    0,    0,  237,  237,    0,    0, 
            0,  237,    0,    0,    0,    0,    0,  318,  318,  318, 
          318,  318,  318,  318,  318,  318,  318,  318,  318,  318, 
          318,    0,    0,  318,  318,  318,  318,    0,    0,  318, 
            0,    0,  318,    0,    0,  318,    0,  318,    0,  318, 
          234,  318,    0,  318,  318,  318,  318,  318,  318,  318, 
          234,  318,    0,  318,    0,    0,    0,  240,  240,  240, 
            0,    0,  240,  240,  240,    0,  240,    0,    0,    0, 
            0,    0,    0,    0,    0,    0,  240,  240,  318,    0, 
            0,    0,    0,    0,  234,  240,  240,    0,  240,  240, 
          240,  240,  240,    0,    0,    0,    0,    0,  234,  234, 
          240,    0,    0,  234,    0,    0,    0,    0,    0,    0, 
            0,    0,    0,  240,  240,  240,  240,  240,  240,  240, 
          240,  240,  240,    0,  240,  240,    0,    0,  240,  240, 
            0,    0,    0,    0,  240,    0,    0,  240,    0,    0, 
          240,    0,  240,    0,  240,  236,  240,    0,  240,  240, 
          240,  240,  240,  240,  240,  236,  240,    0,  240,    0, 
            0,    0,    0,  109,    0,    0,    0,    0,    0,    0, 
          240,    0,    0,    0,    0,    0,    0,    0,    0,    0, 
            0,    0,    0,  240,    0,    0,    0,    0,    0,  236, 
            0,    0,  622,    0,    0,    0,    0,  109,    0,    0, 
          237,  237,  237,  236,  236,  237,  237,  237,  236,  237, 
            0,    0,    0,    0,  109,    0,    0,    0,    0,  237, 
          237,    0,    0,    0,    0,    0,  622,    0,  237,  237, 
            0,  237,  237,  237,  237,  237,    0,    0,    0,    0, 
            0,  622,    0,  237,  315,    0,    0,    0,    0,    0, 
            0,    0,    0,    0,    0,    0,  237,  237,  237,  237, 
          237,  237,  237,  237,  237,  237,    0,  237,  237,    0, 
            0,  237,  237,  622,    0,    0,    0,  237,    0,    0, 
          237,    0,    0,  237,    0,  237,    0,  237,  235,  237, 
            0,  237,  237,  237,  237,  237,  237,  237,  235,  237, 
            0,  237,  234,  234,  234,    0,    0,  234,  234,  234, 
            0,  234,    0,  237,    0,    0,    0,    0,    0,    0, 
            0,  234,  234,    0,    0,    0,  237,    0,    0,    0, 
          234,  234,  235,  234,  234,  234,  234,  234,    0,    0, 
            0,    0,    0,    0,    0,  234,  235,  235,    0,    0, 
            0,  235,    0,    0,    0,    0,    0,    0,  234,  234, 
          234,  234,  234,  234,  234,  234,  234,  234,    0,  234, 
          234,    0,    0,  234,  234,    0,    0,    0,    0,  234, 
            0,    0,  234,    0,    0,  234,    0,  234,    0,  234, 
          238,  234,    0,  234,  234,  234,  234,  234,  234,  234, 
          238,  234,    0,  234,    0,    0,    0,  236,  236,  236, 
            0,    0,  236,  236,  236,  234,  236,    0,    0,    0, 
            0,    0,    0,    0,    0,    0,  236,  236,  234,    0, 
            0,    0,  109,    0,  238,  236,  236,    0,  236,  236, 
          236,  236,  236,    0,    0,    0,    0,    0,  238,  238, 
          236,    0,    0,  238,    0,    0,    0,    0,    0,    0, 
            0,    0,  622,  236,  236,  236,  236,  236,  236,  236, 
          236,  236,  236,    0,  236,  236,    0,    0,  236,  236, 
            0,  315,    0,    0,  236,    0,    0,  236,  315,  315, 
          236,    0,  236,    0,  236,  109,  236,    0,  236,  236, 
          236,  236,  236,  236,  236,  436,  236,    0,  236,    0, 
          622,    0,    0,    0,    0,  436,    0,  622,  622,    0, 
          236,    0,    0,  618,    0,    0,    0,    0,    0,    0, 
            0,    0,    0,  236,    0,    0,    0,    0,    0,    0, 
            0,  622,    0,    0,  622,    0,    0,    0,    0,  104, 
          235,  235,  235,    0,    0,  235,  235,  235,    0,  235, 
            0,    0,    0,    0,  436,    0,  123,    0,  436,  235, 
          235,    0,    0,    0,    0,    0,    0,    0,  235,  235, 
            0,  235,  235,  235,  235,  235,    0,    0,    0,    0, 
            0,    0,    0,  235,    0,    0,  436,    0,    0,    0, 
            0,    0,    0,    0,    0,    0,  235,  235,  235,  235, 
          235,  235,  235,  235,  235,  235,  518,  235,  235,    0, 
            0,  235,  235,    0,    0,    0,  518,  235,    0,    0, 
          235,    0,    0,  235,    0,  235,    0,  235,    0,  235, 
            0,  235,  235,  235,  235,  235,  235,  235,    0,  235, 
            0,  235,  238,  238,  238,    0,    0,  238,  238,  238, 
           98,  238,    0,  235,    0,    0,    0,    0,    0,    0, 
            0,  238,  238,    0,    0,  518,  235,  117,    0,  518, 
          238,  238,    0,  238,  238,  238,  238,  238,    0,    0, 
            0,    0,    0,    0,    0,  238,    0,    0,    0,    0, 
            0,    0,    0,    0,    0,    0,    0,  518,  238,  238, 
          238,  238,  238,  238,  238,  238,  238,  238,  517,  238, 
          238,    0,    0,  238,  238,    0,    0,    0,  517,  238, 
            0,    0,  238,    0,    0,  238,    0,  238,    0,  238, 
            0,  238,    0,  238,  238,  238,  238,  238,  238,  238, 
            0,  238,    0,  238,    0,    0,    0,    0,    0,    0, 
            0,    0,   97,    0,    0,  238,    0,  436,  436,  436, 
            0,    0,    0,  436,  436,    0,  436,  517,  238,  116, 
            0,  517,    0,    0,    0,  436,    0,    0,    0,    0, 
            0,    0,    0,    0,    0,  436,  436,    0,  436,  436, 
          436,  436,  436,    0,    0,    0,    0,    0,    0,  517, 
            0,    0,    0,    0,    0,    0,    0,    0,    0,    0, 
            0,    0,  436,  436,  436,  436,  436,  436,  436,  436, 
          436,  436,  436,  436,  436,  436,    0,    0,  436,  436, 
          436,  436,    0,    0,    0,    0,    0,  436,    0,    0, 
            0,    0,    0,    0,  436,    0,  436,    0,  436,  436, 
          436,  436,  436,  436,  436,    0,  436,  436,  436,    0, 
            0,    0,    0,    0,    0,    0,    0,    0,  518,  518, 
          518,    0,    0,    0,  518,  518,    0,  518,    0,  425, 
            0,    0,    0,  436,    0,    0,    0,    0,    0,  425, 
            0,    0,    0,    0,    0,    0,  518,  518,    0,  518, 
          518,  518,  518,  518,    0,    0,    0,    0,    0,    0, 
            0,    0,    0,    0,    0,    0,    0,    0,    0,    0, 
            0,    0,    0,  518,  518,  518,  518,  518,  518,  518, 
          518,  518,  518,  518,  518,  518,  518,    0,  425,  518, 
          518,  518,  518,    0,  530,    0,    0,    0,  518,    0, 
            0,    0,    0,    0,    0,  518,  519,  518,    0,  518, 
          518,  518,  518,  518,  518,  518,  519,  518,    0,  518, 
          517,  517,  517,    0,    0,    0,  517,  517,    0,  517, 
            0,    0,    0,    0,    0,    0,    0,    0,    0,    0, 
            0,    0,    0,    0,  518,    0,    0,    0,  517,  517, 
           99,  517,  517,  517,  517,  517,    0,    0,    0,    0, 
            0,    0,    0,    0,    0,  519,    0,  118,    0,  519, 
            0,    0,    0,    0,    0,  517,  517,  517,  517,  517, 
          517,  517,  517,  517,  517,  517,  517,  517,  517,    0, 
            0,  517,  517,  517,  517,    0,  529,  519,    0,    0, 
          517,    0,    0,    0,    0,    0,    0,  517,  520,  517, 
            0,  517,  517,  517,  517,  517,  517,  517,  520,  517, 
            0,  517,    0,    0,    0,    0,    0,    0,    0,    0, 
            0,    0,    0,    0,    0,    0,    0,    0,    0,    0, 
            0,    0,    0,    0,    0,    0,  517,    0,    0,    0, 
            0,    0,  100,    0,    0,    0,    0,    0,    0,    0, 
            0,    0,    0,    0,    0,    0,    0,  520,    0,  119, 
            0,  520,    0,    0,    0,    0,    0,    0,    0,    0, 
            0,    0,    0,    0,    0,  522,    0,    0,    0,    0, 
            0,  425,  425,  425,    0,  522,  425,  425,  425,  520, 
          425,    0,    0,    0,    0,    0,    0,    0,    0,    0, 
          425,    0,    0,    0,    0,    0,    0,    0,    0,  425, 
          425,    0,  425,  425,  425,  425,  425,    0,    0,  102, 
            0,    0,    0,    0,    0,    0,    0,    0,    0,    0, 
            0,    0,    0,    0,  522,    0,  121,    0,  522,    0, 
            0,    0,    0,    0,    0,    0,    0,  425,    0,    0, 
            0,    0,    0,    0,  425,  425,    0,    0,  519,  519, 
          519,  425,    0,    0,  519,  519,  522,  519,    0,    0, 
            0,    0,    0,    0,    0,    0,    0,    0,    0,    0, 
            0,    0,  425,    0,    0,   65,  519,  519,    0,  519, 
          519,  519,  519,  519,    0,   65,    0,    0,    0,    0, 
            0,    0,    0,    0,    0,    0,    0,  425,    0,    0, 
            0,    0,    0,  519,  519,  519,  519,  519,  519,  519, 
          519,  519,  519,  519,  519,  519,  519,  292,    0,  519, 
          519,  519,  519,    0,  531,    0,    0,  292,  519,    0, 
            0,    0,    0,    0,   65,  519,    0,  519,    0,  519, 
          519,  519,  519,  519,  519,  519,    0,  519,    0,  519, 
          520,  520,  520,    0,    0,    0,  520,  520,    0,  520, 
            0,  288,    0,    0,    0,    0,    0,    0,    0,    0, 
            0,    0,    0,    0,  519,    0,  292,    0,  520,  520, 
            0,  520,  520,  520,  520,  520,    0,    0,    0,    0, 
            0,    0,    0,    0,    0,    0,    0,    0,    0,    0, 
            0,    0,    0,    0,    0,  520,  520,  520,  520,  520, 
          520,  520,  520,  520,  520,  520,  520,  520,  520,    0, 
            0,  520,  520,  520,  520,    0,  532,  522,  522,  522, 
          520,    0,    0,  522,  522,    0,  522,  520,    0,  520, 
            0,  520,  520,  520,  520,  520,  520,  520,    0,  520, 
            0,  520,    0,    0,    0,  522,  522,    0,  522,  522, 
          522,  522,  522,    0,    0,    0,    0,    0,  521,    0, 
            0,    0,    0,    0,    0,    0,  520,    0,  521,    0, 
            0,    0,  522,  522,  522,  522,  522,  522,  522,  522, 
          522,  522,  522,  522,  522,  522,    0,    0,  522,  522, 
          522,  522,    0,  534,    0,    0,    0,  522,    0,    0, 
            0,    0,  101,    0,  522,    0,  522,    0,  522,  522, 
          522,  522,  522,  522,  522,    0,  522,  521,  522,  120, 
            0,  521,    0,    0,    0,    0,    0,   65,   65,   65, 
            0,    0,   65,   65,   65,  523,   65,    0,    0,    0, 
            0,    0,    0,  522,    0,  523,   65,   65,    0,  521, 
            0,    0,    0,    0,    0,   65,   65,    0,   65,   65, 
           65,   65,   65,    0,    0,    0,    0,    0,    0,  292, 
          292,  292,    0,    0,  292,  292,  292,    0,  292,  103, 
            0,    0,    0,    0,    0,    0,    0,    0,    0,    0, 
            0,    0,    0,    0,  523,    0,  122,    0,  523,    0, 
          292,  292,  292,  292,  292,    0,    0,   65,    0,    0, 
           65,    0,  514,    0,    0,    0,    0,    0,    0,    0, 
            0,    0,  514,    0,    0,    0,  523,    0,   65,    0, 
            0,    0,    0,    0,    0,    0,    0,    0,    0,    0, 
            0,    0,    0,    0,    0,    0,  292,    0,    0,  292, 
            0,    0,    0,   65,    0,    0,   94,    0,    0,    0, 
            0,    0,    0,    0,    0,    0,    0,    0,    0,    0, 
          292,  514,    0,  113,    0,  514,    0,    0,    0,    0, 
            0,    0,    0,    0,    0,    0,    0,    0,    0,    0, 
            0,    0,    0,    0,    0,  292,    0,    0,    0,    0, 
            0,    0,    0,  514,    0,    0,    0,    0,    0,    0, 
            0,    0,    0,    0,    0,    0,    0,    0,    0,    0, 
          521,  521,  521,    0,    0,    0,  521,  521,    0,  521, 
            0,    0,    0,    0,    0,    0,    0,    0,    0,    0, 
            0,    0,    0,    0,    0,    0, 
      };
   }

   private static final short[] yyTable3() {
      return new short[] {

          293,    0,  521,  521,    0,  521,  521,  521,  521,  521, 
          293,    0,    0,    0,    0,    0,    0,    0,  291,    0, 
            0,    0,    0,    0,    0,    0,    0,    0,  291,  521, 
          521,  521,  521,  521,  521,  521,  521,  521,  521,  521, 
          521,  521,  521,    0,  289,  521,  521,  521,  521,    0, 
          533,  523,  523,  523,  521,    0,    0,  523,  523,  293, 
          523,  521,  286,  521,    0,  521,  521,  521,  521,  521, 
          521,  521,    0,  521,    0,  521,    0,  291,    0,  523, 
          523,    0,  523,  523,  523,  523,  523,    0,    0,    0, 
            0,    0,    0,    0,    0,    0,    0,    0,    0,    0, 
          521,    0,    0,    0,    0,    0,  523,  523,  523,  523, 
          523,  523,  523,  523,  523,  523,  523,  523,  523,  523, 
            0,    0,  523,  523,  523,  523,    0,  535,  514,  514, 
          514,  523,    0,    0,  514,  514,    0,  514,  523,    0, 
          523,    0,  523,  523,  523,  523,  523,  523,  523,    0, 
          523,    0,  523,    0,    0,    0,  514,  514,    0,  514, 
          514,  514,  514,  514,    0,    0,    0,    0,    0,  513, 
            0,    0,    0,    0,    0,    0,    0,  523,    0,  513, 
            0,    0,    0,  514,  514,  514,  514,  514,  514,  514, 
          514,  514,  514,  514,  514,  514,  514,    0,    0,  514, 
          514,  514,  514,    0,  526,    0,    0,    0,  514,    0, 
            0,    0,    0,   93,    0,  514,    0,  514,    0,  514, 
          514,  514,  514,  514,  514,  514,    0,  514,  513,  514, 
          112,    0,  513,    0,    0,    0,    0,    0,    0,    0, 
            0,    0,    0,    0,    0,    0,  516,    0,    0,    0, 
            0,    0,    0,    0,  514,    0,  516,    0,    0,    0, 
          513,    0,  293,  293,  293,    0,    0,  293,  293,  293, 
            0,  293,    0,    0,    0,    0,    0,    0,    0,    0, 
          291,  291,  291,    0,    0,    0,  291,  291,    0,  291, 
           96,    0,    0,  293,  293,  293,  293,  293,    0,    0, 
            0,    0,    0,    0,    0,  516,    0,  115,    0,  516, 
            0,  291,  291,  291,  291,  291,    0,    0,    0,    0, 
            0,    0,    0,  304,    0,    0,    0,    0,    0,    0, 
            0,    0,    0,  304,    0,    0,    0,  516,    0,  293, 
            0,    0,  293,    0,    0,    0,    0,    0,    0,    0, 
            0,    0,    0,    0,    0,    0,    0,    0,    0,    0, 
          291,    0,    0,  293,    0,    0,    0,  110,    0,    0, 
            0,    0,    0,    0,    0,    0,    0,    0,    0,    0, 
            0,  291,  304,    0,  129,    0,  304,    0,  293,    0, 
            0,    0,    0,    0,    0,    0,    0,    0,   44,    0, 
            0,    0,    0,    0,    0,    0,  291,    0,   44,    0, 
            0,    0,    0,    0,  304,    0,    0,    0,    0,    0, 
            0,    0,    0,    0,    0,    0,    0,    0,    0,    0, 
            0,  513,  513,  513,    0,    0,    0,  513,  513,    0, 
          513,    0,    0,    0,    0,    0,    0,    0,    0,    0, 
            0,    0,    0,    0,    0,    0,    0,   44,    0,  513, 
          513,    0,  513,  513,  513,  513,  513,    0,    0,    0, 
            0,    0,    0,    0,    0,  232,    0,    0,    0,    0, 
            0,    0,    0,    0,    0,  232,  513,  513,  513,  513, 
          513,  513,  513,  513,  513,  513,  513,  513,  513,  513, 
            0,    0,  513,  513,  513,  513,    0,  525,  516,  516, 
          516,  513,    0,    0,  516,  516,    0,  516,  513,  232, 
          513,    0,  513,  513,  513,  513,  513,  513,  513,    0, 
          513,    0,  513,  232,  232,    0,  516,  516,  232,  516, 
          516,  516,  516,  516,    0,    0,    0,    0,    0,    0, 
            0,    0,    0,    0,    0,    0,    0,  513,    0,    0, 
            0,    0,    0,  516,  516,  516,  516,  516,  516,  516, 
          516,  516,  516,  516,  516,  516,  516,    0,    0,  516, 
          516,  516,  516,    0,  528,  304,  304,  304,  516,    0, 
            0,  304,  304,    0,  304,  516,    0,  516,    0,  516, 
          516,  516,  516,  516,  516,  516,    0,  516,    0,  516, 
            0,    0,    0,  304,  304,    0,  304,  304,  304,  304, 
          304,    0,    0,    0,    0,    0,  315,    0,    0,    0, 
            0,    0,    0,    0,  516,    0,  315,    0,    0,    0, 
          304,  304,  304,  304,  304,  304,  304,  304,  304,  304, 
          304,  304,  304,  304,    0,    0,  304,  304,  304,  304, 
           44,   44,   44,    0,    0,  304,   44,   44,    0,   44, 
          109,    0,  304,    0,  304,    0,  304,  304,  304,  304, 
          304,  304,  304,    0,  304,  315,  304,  128,    0,  315, 
            0,   44,   44,   44,   44,   44,    0,    0,    0,    0, 
            0,    0,    0,    0,    0,    0,    0,    0,    0,    0, 
            0,  304,    0,    0,    0,    0,    0,  315,    0,    0, 
            0,    0,    0,    0,    0,    0,    0,    0,    0,    0, 
            0,    0,    0,    0,    0,    0,    0,  232,  232,  232, 
           44,    0,  232,  232,  232,    0,  232,    0,  233,    0, 
            0,    0,    0,    0,    0,    0,  232,  232,  233,    0, 
            0,   44,    0,    0,    0,  232,  232,    0,  232,  232, 
          232,  232,  232,    0,    0,    0,    0,    0,    0,    0, 
          232,    0,    0,    0,    0,    0,   44,    0,    0,    0, 
            0,    0,  233,  232,  232,  232,  232,  232,  232,  232, 
          232,  232,  232,    0,  232,  232,  233,  233,  232,  232, 
            0,  233,    0,    0,  232,    0,    0,  232,    0,    0, 
          232,    0,  232,    0,    0,    0,  232,    0,    0,    0, 
          232,  232,  232,  232,  232,    0,  232,    0,  232,    0, 
            0,    0,    0,    0,    0,    0,    0,    0,    0,    0, 
          232,    0,    0,    0,    0,    0,    0,    0,    0,    0, 
            0,    0,    0,  232,    0,    0,  256,    0,    0,    0, 
            0,    0,    0,    0,    0,    0,  256,    0,    0,    0, 
            0,    0,    0,    0,    0,    0,    0,    0,  315,  315, 
          315,    0,    0,    0,  315,  315,    0,  315,    0,    0, 
            0,    0,    0,    0,    0,    0,    0,    0,    0,    0, 
          256,    0,    0,    0,    0,    0,  315,  315,    0,  315, 
          315,  315,  315,  315,  256,  256,    0,    0,    0,  256, 
            0,    0,    0,    0,    0,    0,    0,    0,    0,    0, 
            0,    0,    0,  315,  315,  315,  315,  315,  315,  315, 
          315,  315,  315,  315,  315,  315,  315,    0,    0,  315, 
          315,  315,  315,    0,    0,    0,    0,    0,  315,    0, 
            0,    0,    0,    0,    0,  315,    0,  315,    0,  315, 
          315,  315,  315,  315,  315,  315,    0,  315,  257,  315, 
            0,    0,    0,    0,    0,    0,    0,    0,  257,    0, 
            0,    0,    0,    0,    0,    0,    0,    0,    0,    0, 
          233,  233,  233,    0,  315,  233,  233,  233,    0,  233, 
            0,    0,    0,    0,    0,    0,    0,    0,    0,  233, 
          233,    0,  257,    0,    0,    0,    0,    0,  233,  233, 
            0,  233,  233,  233,  233,  233,  257,  257,    0,    0, 
            0,  257,    0,  233,    0,    0,    0,    0,    0,    0, 
            0,    0,    0,    0,    0,    0,  233,  233,  233,  233, 
          233,  233,  233,  233,  233,  233,    0,  233,  233,    0, 
            0,  233,  233,    0,    0,    0,    0,  233,    0,    0, 
          233,    0,    0,  233,    0,  233,    0,    0,    0,  233, 
            0,    0,    0,  233,  233,  233,  233,  233,    0,  233, 
          243,  233,    0,    0,    0,    0,    0,    0,    0,    0, 
          243,    0,    0,  233,    0,    0,    0,    0,  256,  256, 
          256,    0,    0,  256,  256,  256,  233,  256,    0,    0, 
            0,    0,    0,    0,    0,    0,    0,  256,  256,    0, 
            0,    0,    0,    0,  243,    0,  256,  256,    0,  256, 
          256,  256,  256,  256,    0,    0,    0,    0,  243,  243, 
            0,  256,    0,  243,    0,    0,    0,    0,    0,    0, 
            0,    0,    0,    0,  256,  256,  256,  256,  256,  256, 
          256,  256,  256,  256,    0,  256,  256,    0,    0,  256, 
          256,    0,    0,    0,    0,  256,    0,    0,  256,    0, 
            0,  256,    0,  256,    0,    0,    0,  256,    0,    0, 
            0,    0,    0,  256,  256,  256,    0,  256,  241,  256, 
            0,    0,    0,    0,    0,    0,    0,    0,  241,    0, 
            0,  256,    0,    0,    0,    0,    0,    0,    0,    0, 
          257,  257,  257,    0,  256,  257,  257,  257,    0,  257, 
            0,    0,    0,    0,    0,    0,    0,    0,    0,  257, 
          257,    0,  241,    0,    0,    0,    0,    0,  257,  257, 
            0,  257,  257,  257,  257,  257,  241,  241,    0,    0, 
            0,  241,    0,  257,    0,    0,    0,    0,    0,    0, 
            0,    0,    0,    0,    0,    0,  257,  257,  257,  257, 
          257,  257,  257,  257,  257,  257,    0,  257,  257,    0, 
            0,  257,  257,    0,    0,    0,    0,  257,    0,    0, 
          257,  242,    0,  257,    0,  257,    0,    0,    0,  257, 
            0,  242,    0,    0,    0,  257,  257,  257,    0,  257, 
            0,  257,    0,    0,    0,    0,    0,    0,    0,    0, 
            0,    0,    0,  257,    0,    0,    0,    0,    0,    0, 
            0,    0,  243,  243,  243,  242,  257,  243,  243,  243, 
            0,  243,    0,    0,    0,    0,    0,    0,    0,  242, 
          242,  243,  243,    0,  242,    0,    0,    0,    0,    0, 
          243,  243,    0,  243,  243,  243,  243,  243,    0,    0, 
            0,    0,    0,    0,    0,  243,    0,    0,    0,    0, 
            0,    0,    0,    0,    0,    0,    0,    0,  243,  243, 
          243,  243,  243,  243,  243,  243,  243,  243,    0,  243, 
          243,    0,    0,    0,    0,    0,    0,    0,    0,  243, 
            0,    0,  243,    0,    0,  243,    0,  243,    0,    0, 
            0,  243,    0,    0,    0,    0,    0,  243,  243,  243, 
            0,  243,  246,  243,    0,    0,    0,    0,    0,    0, 
            0,    0,  246,    0,    0,  243,    0,    0,    0,    0, 
          241,  241,  241,    0,    0,  241,  241,  241,  243,  241, 
            0,    0,    0,    0,    0,    0,    0,    0,    0,  241, 
          241,    0,    0,    0,    0,    0,  246,    0,  241,  241, 
            0,  241,  241,  241,  241,  241,    0,    0,    0,    0, 
          246,  246,    0,  241,    0,  246,    0,    0,    0,    0, 
            0,    0,    0,    0,    0,    0,  241,  241,  241,  241, 
          241,  241,  241,  241,  241,  241,    0,  241,  241,    0, 
            0,    0,    0,    0,    0,    0,    0,  241,    0,    0, 
          241,    0,    0,  241,  248,  241,    0,    0,    0,    0, 
            0,    0,    0,    0,  248,  241,  241,  241,    0,  241, 
            0,  241,    0,  242,  242,  242,    0,    0,  242,  242, 
          242,    0,  242,  241,    0,    0,    0,    0,    0,    0, 
            0,    0,  242,  242,    0,    0,  241,    0,  248,    0, 
            0,  242,  242,    0,  242,  242,  242,  242,  242,    0, 
            0,    0,  248,  248,    0,    0,  242,  248,    0,    0, 
            0,    0,    0,    0,    0,    0,    0,    0,    0,  242, 
          242,  242,  242,  242,  242,  242,  242,  242,  242,    0, 
          242,  242,    0,    0,    0,    0,    0,    0,    0,    0, 
          242,    0,    0,  242,    0,    0,  242,    0,  242,    0, 
            0,    0,    0,    0,    0,    0,    0,    0,  242,  242, 
          242,    0,  242,    0,  242,    0,    0,    0,    0,    0, 
            0,  247,    0,    0,    0,    0,  242,    0,    0,    0, 
            0,  247,    0,    0,    0,    0,    0,    0,    0,  242, 
            0,    0,    0,    0,    0,    0,    0,    0,    0,    0, 
            0,    0,    0,    0,  246,  246,  246,    0,    0,  246, 
          246,  246,    0,  246,    0,  247,    0,    0,    0,    0, 
            0,    0,    0,  246,  246,    0,    0,    0,    0,  247, 
          247,    0,  246,  246,  247,  246,  246,  246,  246,  246, 
            0,    0,    0,    0,    0,    0,    0,  246,    0,    0, 
            0,    0,    0,    0,    0,    0,    0,    0,    0,    0, 
          246,  246,  246,  246,  246,  246,  246,  246,  246,  246, 
            0,  246,  246,    0,    0,    0,    0,    0,    0,    0, 
            0,  246,    0,    0,  246,  245,    0,  246,    0,  246, 
            0,    0,    0,    0,    0,  245,    0,    0,    0,  246, 
          246,    0,    0,    0,    0,  246,  248,  248,  248,    0, 
            0,  248,  248,  248,    0,  248,    0,  246,    0,    0, 
            0,    0,    0,    0,    0,  248,  248,    0,    0,  245, 
          246,    0,    0,    0,  248,  248,    0,  248,  248,  248, 
          248,  248,    0,  245,  245,    0,    0,    0,  245,  248, 
            0,    0,    0,    0,    0,    0,    0,    0,    0,    0, 
            0,    0,  248,  248,  248,  248,  248,  248,  248,  248, 
          248,  248,    0,  248,  248,    0,    0,    0,    0,    0, 
            0,    0,    0,  248,    0,    0,  248,  249,    0,  248, 
            0,  248,    0,    0,    0,    0,    0,  249,    0,    0, 
            0,  248,  248,    0,    0,    0,    0,  248,    0,    0, 
            0,    0,    0,    0,    0,    0,    0,    0,    0,  248, 
            0,    0,    0,    0,    0,    0,    0,    0,    0,    0, 
            0,  249,  248,  247,  247,  247,    0,    0,  247,  247, 
          247,    0,  247,    0,    0,  249,  249,    0,    0,    0, 
          249,    0,  247,  247,    0,    0,    0,    0,    0,    0, 
            0,  247,  247,    0,  247,  247,  247,  247,  247,    0, 
            0,    0,    0,  250,    0,    0,  247,    0,    0,    0, 
            0,    0,    0,  250,    0,    0,    0,    0,    0,  247, 
          247,  247,  247,  247,  247,  247,  247,  247,  247,    0, 
          247,  247,    0,    0,    0,    0,    0,    0,    0,    0, 
          247,    0,    0,  247,    0,    0,  247,  250,  247,    0, 
            0,    0,    0,    0,    0,    0,    0,    0,  247,  247, 
            0,  250,  250,    0,  247,    0,  250,    0,    0,    0, 
            0,    0,    0,    0,    0,    0,  247,  245,  245,  245, 
            0,    0,  245,  245,  245,    0,  245,    0,    0,  247, 
           63,    0,    0,    0,    0,    0,  245,  245,    0,    0, 
           63,    0,    0,    0,    0,  245,  245,    0,  245,  245, 
          245,  245,  245,    0,    0,    0,    0,    0,    0,    0, 
          245,    0,    0,    0,    0,    0,    0,    0,    0,    0, 
            0,    0,    0,  245,  245,  245,  245,  245,  245,  245, 
          245,  245,  245,  251,  245,  245,    0,    0,    0,   63, 
            0,    0,    0,  251,  245,    0,    0,  245,    0,    0, 
          245,    0,  245,    0,    0,    0,    0,    0,    0,    0, 
            0,    0,  245,  245,    0,    0,    0,    0,  245,  249, 
          249,  249,    0,    0,  249,  249,  249,  251,  249,    0, 
          245,    0,    0,    0,    0,    0,    0,    0,  249,  249, 
            0,  251,  251,  245,    0,    0,  251,  249,  249,    0, 
          249,  249,  249,  249,  249,    0,    0,    0,    0,    0, 
            0,    0,  249,    0,    0,    0,    0,    0,    0,    0, 
            0,    0,    0,    0,    0,    0,    0,    0,    0,    0, 
            0,  249,  249,    0,    0,    0,  249,  249,    0,    0, 
            0,    0,    0,    0,    0,    0,  249,    0,  258,  249, 
            0,    0,  249,    0,  249,  250,  250,  250,  258,    0, 
          250,  250,  250,    0,  250,    0,    0,    0,    0,    0, 
          249,    0,    0,    0,  250,  250,    0,    0,    0,    0, 
            0,    0,  249,  250,  250,    0,  250,  250,  250,  250, 
          250,    0,  258,    0,    0,  249,    0,    0,  250,    0, 
            0,    0,    0,    0,    0,    0,  258,  258,    0,    0, 
            0,  258,    0,    0,    0,    0,    0,  250,  250,    0, 
            0,    0,  250,  250,    0,    0,    0,    0,    0,    0, 
            0,    0,  250,    0,    0,  250,    0,    0,  250,    0, 
          250,    0,   63,   63,   63,    0,    0,   63,   63,   63, 
          252,   63,    0,    0,    0,    0,  250,    0,    0,    0, 
          252,   63,   63,    0,    0,    0,    0,    0,  250,    0, 
           63,   63,    0,   63,   63,   63,   63,   63,    0,    0, 
            0,  250,    0,    0,    0,    0,    0,    0,    0,    0, 
            0,    0,    0,    0,  252,  251,  251,  251,    0,    0, 
          251,  251,  251,    0,  251,    0,    0,    0,  252,  252, 
            0,    0,    0,  252,  251,  251,    0,    0,    0,    0, 
            0,    0,   63,  251,  251,   63,  251,  251,  251,  251, 
          251,    0,    0,    0,    0,    0,    0,    0,  251,  253, 
            0,    0,    0,   63,    0,    0,    0,    0,    0,  253, 
            0,    0,    0,    0,    0,    0,    0,  251,  251,    0, 
            0,    0,  251,  251,    0,    0,    0,    0,   63,    0, 
            0,    0,  251,    0,    0,  251,    0,    0,  251,    0, 
          251,    0,    0,  253,    0,    0,    0,    0,    0,    0, 
            0,    0,    0,    0,    0,    0,  251,  253,  253,    0, 
            0,    0,  253,    0,    0,    0,    0,    0,  251,    0, 
          258,  258,  258,    0,    0,  258,  258,  258,    0,  258, 
            0,  251,    0,    0,    0,  231,    0,    0,    0,  258, 
          258,    0,    0,    0,    0,  231,    0,    0,  258,  258, 
            0,  258,  258,  258,  258,  258,    0,    0,    0,    0, 
            0,    0,    0,  258,    0,    0,    0,    0,    0,    0, 
            0,    0,    0,    0,    0,    0,    0,    0,    0,  231, 
            0,    0,  258,  258,    0,    0,    0,  258,  258,    0, 
            0,    0,    0,  231,  231,    0,    0,  258,  231,    0, 
          258,    0,    0,  258,    0,  258,    0,    0,    0,    0, 
            0,    0,    0,    0,    0,    0,    0,    0,    0,    0, 
            0,  258,  252,  252,  252,    0,    0,  252,  252,  252, 
            0,  252,    0,  258,    0,    0,    0,  226,    0,    0, 
            0,  252,  252,    0,    0,    0,  258,  226,    0,    0, 
          252,  252,    0,  252,  252,  252,  252,  252,    0,    0, 
            0,    0,    0,    0,    0,  252,    0,    0,    0,    0, 
            0,    0,    0,    0,    0,    0,    0,    0,    0,    0, 
            0,  226,    0,    0,  252,  252,    0,    0,    0,  252, 
          252,    0,    0,    0,    0,  226,  226,    0,    0,  252, 
            0,    0,  252,    0,    0,  252,    0,  252,    0,    0, 
            0,  253,  253,  253,    0,    0,  253,  253,  253,    0, 
          253,    0,    0,  252,  227,    0,    0,    0,    0,    0, 
          253,  253,    0,    0,  227,  252,    0,    0,    0,  253, 
          253,    0,  253,  253,  253,  253,  253,    0,  252,    0, 
            0,    0,    0,    0,  253,    0,    0,    0,    0,    0, 
            0,    0,    0,    0,    0,    0,    0,    0,  227,    0, 
            0,    0,    0,  253,  253,    0,    0,    0,  253,  253, 
            0,    0,  227,  227,    0,    0,    0,    0,  253,    0, 
            0,  253,    0,    0,  253,    0,  253,  231,  231,  231, 
            0,    0,  231,  231,  231,    0,  231,    0,    0,    0, 
          224,    0,  253,    0,    0,    0,  231,  231,    0,    0, 
          224,    0,    0,    0,  253,  231,  231,    0,  231,  231, 
          231,  231,  231,    0,    0,    0,    0,  253,    0,    0, 
          231,    0,    0,    0,    0,    0,    0,    0,    0,    0, 
            0,    0,    0,    0,  224,    0,    0,    0,    0,    0, 
            0,    0,    0,    0,    0,    0,    0,    0,  224,  224, 
            0,    0,    0,    0,  231,    0,    0,  231,    0,    0, 
          231,    0,  231,    0,    0,    0,    0,    0,    0,    0, 
            0,    0,    0,    0,    0,    0,    0,  225,  231,  226, 
          226,  226,    0,    0,  226,  226,  226,  225,  226,    0, 
          231,    0,    0,    0,    0,    0,    0,    0,  226,  226, 
            0,    0,    0,  231,    0,    0,    0,  226,  226,    0, 
          226,  226,  226,  226,  226,    0,    0,    0,    0,    0, 
            0,  225,  226,    0,    0,    0,    0,    0,    0,    0, 
            0,    0,    0,    0,    0,  225,  225,    0,    0,    0, 
            0,    0,    0,    0,    0,    0,    0,    0,    0,    0, 
            0,    0,    0,    0,    0,    0,  226,    0,    0,  226, 
            0,    0,  226,    0,  226,    0,  227,  227,  227,    0, 
            0,  227,  227,  227,    0,  227,    0,    0,    0,  220, 
          226,    0,    0,    0,    0,  227,  227,    0,    0,  220, 
            0,    0,  226,    0,  227,  227,    0,  227,  227,  227, 
          227,  227,    0,    0,    0,  226,    0,    0,    0,  227, 
            0,    0,    0,    0,    0,    0,    0,    0,  222,    0, 
            0,    0,    0,  220,    0,    0,    0,    0,  222,    0, 
            0,    0,    0,    0,    0,    0,    0,  220,  220,    0, 
            0,    0,    0,  227,    0,    0,  227,    0,    0,  227, 
            0,  227,  224,  224,  224,    0,    0,  224,  224,  224, 
            0,  224,  222,    0,    0,    0,    0,  227,    0,    0, 
            0,  224,  224,    0,    0,    0,  222,  222,    0,  227, 
          224,  224,    0,  224,  224,  224,  224,  224,    0,    0, 
            0,    0,  227,    0,    0,  224,    0,    0,    0,    0, 
            0,    0,    0,    0,    0,    0,    0,    0,    0,    0, 
            0,    0,    0,    0,    0,    0,    0,    0,    0,    0, 
            0,    0,    0,    0,    0,    0,    0,    0,    0,  224, 
            0,    0,  224,  261,    0,  224,    0,  224,    0,  225, 
          225,  225,    0,  261,  225,  225,  225,    0,  225,    0, 
            0,    0,    0,  224,    0,    0,    0,    0,  225,  225, 
            0,    0,    0,    0,    0,  224,    0,  225,  225,    0, 
          225,  225,  225,  225,  225,    0,    0,  261,  224,    0, 
            0,    0,  225,    0,    0,    0,    0,    0,    0,    0, 
            0,  261,  261,    0,    0,  223,    0,    0,    0,    0, 
            0,    0,    0,    0,    0,  223,    0,    0,    0,    0, 
            0,    0,    0,    0,    0,    0,  225,    0,    0,  225, 
            0,    0,  225,    0,  225,    0,    0,    0,    0,    0, 
            0,    0,    0,    0,    0,    0,    0,    0,    0,  223, 
          225,  220,  220,  220,    0,    0,  220,  220,  220,    0, 
          220,    0,  225,  223,  223,    0,    0,    0,    0,    0, 
          220,  220,    0,    0,    0,  225,    0,    0,    0,  220, 
          220,    0,  220,  220,  220,  220,  220,    0,    0,    0, 
          222,  222,  222,    0,  220,  222,  222,  222,    0,  222, 
            0,    0,    0,    0,    0,    0,    0,  263,    0,  222, 
          222,    0,    0,    0,    0,    0,    0,  263,  222,  222, 
            0,  222,  222,  222,  222,  222,    0,    0,  220,    0, 
            0,  220,    0,  222,  220,    0,  220,    0,    0,    0, 
            0,    0,    0,    0,    0,    0,    0,    0,    0,    0, 
            0,  263,  220,    0,    0,    0,    0,    0,    0,    0, 
            0,    0,    0,    0,  220,    0,  263,  222,    0,  221, 
          222,    0,    0,  222,    0,  222,    0,  220,    0,  221, 
            0,    0,    0,    0,    0,    0,    0,    0,    0,    0, 
            0,  222,    0,    0,    0,    0,    0,    0,    0,    0, 
            0,    0,    0,  222,    0,  261,  261,  261,    0,    0, 
          261,  261,  261,  221,  261,    0,  222,    0,    0,    0, 
            0,    0,    0,    0,  261,  261,    0,  221,  221,    0, 
            0,    0,    0,  261,  261,    0,  261,  261,  261,  261, 
          261,    0,    0,    0,    0,    0,    0,    0,  261,    0, 
            0,    0,    0,    0,    0,    0,    0,    0,    0,    0, 
            0,    0,    0,    0,    0,    0,    0,  223,  223,  223, 
            0,  219,  223,  223,  223,    0,  223,    0,    0,    0, 
            0,  219,  261,    0,    0,  261,  223,  223,  261,    0, 
          261,    0,    0,    0,    0,  223,  223,    0,  223,  223, 
          223,  223,  223,    0,    0,    0,  261,    0,    0,    0, 
          223,    0,    0,    0,    0,  219,    0,    0,  261,    0, 
            0,    0,    0,    0,    0,    0,    0,    0,    0,  219, 
          219,  261,    0,  286,    0,    0,    0,    0,    0,    0, 
            0,    0,    0,  286,  223,    0,    0,  223,    0,    0, 
          223,    0,  223,    0,    0,    0,    0,    0,    0,    0, 
            0,    0,    0,    0,    0,    0,    0,    0,  223,  263, 
          263,  263,    0,    0,  263,  263,  263,  286,  263,    0, 
          223,    0,    0,    0,    0,    0,    0,    0,  263,  263, 
            0,    0,  286,  223,    0,    0,    0,  263,  263,    0, 
          263,  263,  263,  263,  263,    0,    0,    0,    0,    0, 
            0,    0,  263,    0,    0,    0,    0,    0,    0,    0, 
            0,    0,    0,    0,    0,    0,    0,    0,    0,    0, 
            0,  221,  221,  221,    0,  288,  221,  221,  221,    0, 
          221,    0,    0,    0,    0,  288,  263,    0,    0,  263, 
          221,  221,  263,    0,  263,    0,    0,    0,    0,  221, 
          221,    0,  221,  221,  221,  221,    0,    0,    0,    0, 
          263,   54,    0,    0,  221,    0,    0,    0,    0,  288, 
            0,   54,  263,    0,    0,    0,    0,    0,    0,    0, 
            0,    0,    0,    0,  288,  263,    0,    0,    0,    0, 
            0,    0,    0,    0,    0,    0,    0,    0,  221,    0, 
            0,  221,    0,    0,  221,    0,  221,    0,    0,    0, 
            0,   56,    0,    0,    0,    0,    0,    0,    0,    0, 
           54,   56,  221,  219,  219,  219,    0,    0,  219,  219, 
          219,    0,  219,    0,  221,    0,    0,    0,    0,    0, 
            0,    0,  219,  219,    0,    0,    0,  221,   57,    0, 
            0,  219,  219,    0,  219,  219,  219,  219,   57,    0, 
            0,    0,    0,    0,    0,    0,  219,    0,    0,    0, 
           56,    0,    0,    0,    0,    0,    0,    0,    0,    0, 
            0,    0,    0,    0,    0,  286,  286,  286,  219,    0, 
          286,  286,  286,    0,  286,    0,    0,    0,  219,   25, 
          219,    0,    0,  219,  286,  286,  219,   57,  219,   25, 
            0,    0,    0,  286,  286,    0,  286,  286,  286,  286, 
          286,    0,    0,    0,  219,    0,    0,    0,    0,    0, 
            0,    0,  263,    0,    0,    0,  219,    0,    0,    0, 
            0,    0,    0,    0,    0,    0,    0,  219,    0,  219, 
            0,    0,    0,    0,    0,    0,    0,    0,   25,    0, 
            0,    0,    0,    0,    0,  286,    0,    0,  286,    0, 
          286,    0,    0,    0,    0,    0,    0,    0,    0,    0, 
            0,    0,    0,    0,    0,    0,  286,  288,  288,  288, 
            0,    0,  288,  288,  288,    0,  288,    0,    0,    0, 
            0,    0,    0,    0,    0,    0,  288,  288,    0,    0, 
            0,  286,    0,    0,    0,  288,  288,  305,  288,  288, 
          288,  288,  288,   54,   54,   54,    0,    0,   54,   54, 
           54,    0,   54,    0,    0,    0,    0,    0,    0,    0, 
            0,    0,   54,    0,    0,    0,    0,    0,    0,    0, 
            0,   54,   54,    0,   54,   54,   54,   54,   54,    0, 
            0,    0,    0,    0,    0,    0,    0,  288,    0,    0, 
          288,    0,  288,   56,   56,   56,    0,    0,   56,   56, 
           56,    0,   56,    0,    0,    0,    0,    0,  288,    0, 
            0,    0,   56,    0,    0,    0,    0,    0,    0,    0, 
            0,   56,   56,   54,   56,   56,   56,   56,   56,    0, 
           57,   57,   57,  288,    0,   57,   57,   57,    0,   57, 
            0,    0,    0,    0,   54,    0,    0,    0,    0,   57, 
            0,    0,    0,    0,    0,    0,    0,    0,   57,   57, 
            0,   57,   57,   57,   57,   57,    0,    0,    0,   54, 
          219,  219,  219,   56,  130,    0,  219,  219,    0,  219, 
            0,   25,   25,   25,    0,    0,    0,   25,   25,    0, 
           25,    0,    0,    0,   56,    0,    0,    0,  219,  219, 
            0,  219,  219,  219,  219,    0,    0,    0,    0,    0, 
           57,    0,   25,   25,   25,   25,   25,    0,    0,   56, 
            0,    0,    0,  130,    0,    0,    0,    0,    0,    0, 
            0,   57,    0,    0,    0,    0,    0,    0,    0,    0, 
            0,    0,    0,    0,    0,    0,    0,    0,    0,    0, 
          219,    0,    0,    0,    0,  512,   57,    0,    0,    0, 
            0,   25,    0,    0,    0,    0,    0,    0,    0,    0, 
            0,  219,    0,    0,    0,    0,    0,    0,    0,    0, 
            0,    0,   25,    0,    4,    5,    6,    0,    8,    0, 
            0,    0,    9,   10,    0,    0,  219,   11,    0,   12, 
           13,   14,   15,   16,   17,   18,    0,   25,  131,    0, 
           19,   20,   21,  236,  237,  238,  239,    0,    0,  118, 
            0,    0,    0,    0,    0,    0,   28,    0,    0,  240, 
          241,  242,    0,  243,   35,  244,  245,  246,  247,  282, 
           40,   41,   42,   43,    0,    0,    0,    0,    0,    0, 
            0,    0,    0,    0,    0,    0,    0,  131,    0,    0, 
            0,    0,    0,    0,  248,    0,    0,  131,    0,    0, 
           46,   47,    0,   48,    0,  283,    0,  284,    0,   50, 
            0,    0,    0,    0,    0,    0,    0,  285,    0,  515, 
            0,    0,   52,  286,   54,   55,   56,   57,    0,    0, 
            0,   58,    0,   59,   60,    0,   61,   62,   63,   64, 
            0,    0,   66,   67,  287,    0,    0,    0,    0,    0, 
          130,  130,  130,  130,  130,  130,  130,  130,  130,  130, 
          130,    0,    0,  130,  130,    0,  130,  130,  130,  130, 
          130,  130,  130,    0,  615,    0,  130,  130,  130,  130, 
          130,  130,  130,  130,    0,    0,  130,    0,    0,    0, 
            0,    0,  130,  130,    0,  130,  130,  130,  130,    0, 
          130,  130,  130,  130,  130,  130,    0,  130,  130,  130, 
          130,    0,    0,    0,    0,    0,    0,    0,    0,    0, 
            0,    0,  512,    0,    0,  130,    0,    0,    0,  512, 
          512,  130,    0,    0,  130,  615,    0,  130,  130,    0, 
          130,    0,  130,    0,    0,    0,  130,    0,    0,    0, 
            0,  130,    0,    0,  130,    0,  615,  622,    0,  130, 
          130,  130,  130,  130,  130,    0,    0,    0,  130,    0, 
          130,  130,    0,  130,  130,  130,  130,    0,  130,  130, 
          130,    0,    0,    0,  131,  131,  131,  131,  131,  131, 
          131,  131,  131,  131,  131,    0,    0,  131,  131,    0, 
          131,  131,  131,  131,  131,  131,  131,    0,  616,    0, 
          131,  131,  131,  131,  131,  131,  131,  131,    0,    0, 
          131,    0,    0,    0,    0,    0,  131,  131,    0,  131, 
          131,  131,  131,    0,  131,  131,  131,  131,  131,  131, 
            0,  131,  131,  131,  131,    0,    0,    0,    0,    0, 
            0,    0,    0,    0,    0,    0,  515,    0,    0,  131, 
            0,    0,    0,  515,  515,  131,    0,    0,  131,  616, 
            0,  131,  131,    0,  131,    0,  131,    0,    0,    0, 
          131,    0,    0,    0,    0,  131,    0,    0,  131,    0, 
          616,  314,    0,  131,  131,  131,  131,  131,  131,    0, 
            0,    0,  131,    0,  131,  131,    0,  131,  131,  131, 
          131,    0,  131,  131,  131,    0,    0,    0,    0,    0, 
            0,    0,  130,  130,  130,  130,  130,  130,  130,  130, 
          130,  130,  130,    0,    0,  130,  130,    0,  130,  130, 
          130,  130,  130,  130,  130,    0,  622,    0,  131,  130, 
          130,  130,  130,  130,  130,  130,    0,    0,  130,    0, 
            0,    0,    0,    0,  130,  130,    0,  130,  130,  130, 
          130,    0,  130,  130,  130,  130,  130,  130,    0,  130, 
          130,  130,  130,    0,    0,    0,    0,    0,    0,    0, 
            0,    0,    0,    0,  622,    0,    0,  131,    0,    0, 
            0,  622,  622,  130,    0,    0,  130,  618,    0,  130, 
          130,    0,  130,    0,  130,    0,    0,    0,  130,    0, 
            0,    0,    0,  130,    0,    0,  130,    0,  622,  315, 
            0,  130,  130,  130,  130,  130,  130,    0,    0,    0, 
          130,    0,  130,  130,    0,  130,  130,  130,  130,    0, 
          130,  130,  130,    0,    0,    0,  131,  131,  131,  131, 
          131,  131,  131,  131,  131,  131,  131,    0,    0,  131, 
          131,    0,  131,  131,  131,  131,  131,  131,  131,    0, 
          359,    0,    0,  131,  131,  131,  131,  131,  131,  131, 
            0,    0,  131,    0,    0,    0,    0,    0,  131,  131, 
            0,  131,  131,  131,  131,    0,  131,  131,  131,  131, 
          131,  131,    0,  131,  131,  131,  131,    0,    0,    0, 
            0,    0,    0,    0,    0,    0,    0,    0,  314,  359, 
            0,    0,    0,    0,    0,  314,  314,  131,    0,    0, 
          131,  619,    0,  131,  131,    0,  131,    0,  131,    0, 
            0,    0,  131,    0,    0,    0,    0,  131,    0,    0, 
          131,    0,    0,    0,    0,  131,  131,  131,  131,  131, 
          131,    0,    0,    0,  131,    0,  131,  131,    0,  131, 
          131,  131,  131,    0,  131,  131,  131,    0,    0,    0, 
            0,    0,    0,    0,  131,  131,  131,  131,  131,  131, 
          131,  131,  131,  131,  131,    0,    0,  131,  131,    0, 
          131,  131,  131,  131,  131,  131,  131,    0,  306,    0, 
            0,  131,  131,  131,  131,  131,  131,  131,    0,    0, 
          131,    0,    0,    0,    0,    0,  131,  131,    0,  131, 
          131,  131,  131,    0,  131,  131,  131,  131,  131,  131, 
            0,  131,  131,  131,  131,    0,    0,    0,    0,    0, 
            0,    0,    0,    0,    0,    0,  315,  306,    0,    0, 
            0,    0,    0,  315,  315,  131,    0,    0,  131,    0, 
            0,  131,  131,    0,  131,    0,  131,    0,    0,    0, 
          131,    0,    0,    0,    0,  131,    0,    0,  131,    0, 
            0,    0,    0,  131,  131,  131,  131,  131,  131,    0, 
            0,    0,  131,    0,  131,  131,    0,  131,  131,  131, 
          131,    0,  131,  131,  131,    0,  359,  359,  359,  359, 
          359,  359,  359,  359,  359,  359,  359,    0,  359,  359, 
          359,  359,  359,  359,  359,  359,  359,  359,  359,  540, 
            0,    0,    0,  359,  359,  359,  359,  359,  359,  359, 
            0,    0,  359,    0,    0,    0,    0,    0,  359,  359, 
            0,  359,  359,  359,  359,    0,  359,  359,  359,  359, 
          359,  359,    0,  359,  359,  359,  359,    0,    0,    0, 
            0,    0,    0,    0,    0,    0,    0,    0,  540,    0, 
            0,    0,    0,    0,    0,    0,    0,  359,    0,    0, 
          359,    0,    0,  359,  359,    0,  359,    0,  359,    0, 
            0,    0,  359,    0,    0,    0,    0,    0,    0,    0, 
          359,    0,    0,    0,    0,  359,  359,  359,  359,  359, 
          359,    0,    0,    0,  359,    0,  359,  359,    0,  359, 
          359,  359,  359,    0,  359,  359,  359,    0,    0,    0, 
            0,    0,    0,    0,  306,  306,  306,  306,  306,  306, 
          306,  306,  306,  306,  306,    0,    0,  306,  306,    0, 
          306,  306,  306,  306,  306,  306,  306,  644,    0,    0, 
            0,  306,  306,  306,  306,  306,  306,  306,    0,    0, 
          306,    0,    0,    0,    0,    0,  306,  306,    0,  306, 
          306,  306,  306,    0,  306,  306,  306,  306,  306,  306, 
            0,  306,  306,  306,  306,    0,    0,    0,    0,    0, 
            0,    0,    0,    0,    0,    0,  644,    0,    0,    0, 
            0,    0,    0,    0,    0,  306,    0,    0,  306,    0, 
            0,  306,  306,    0,  306,    0,  306,    0,    0,    0, 
          306,    0,    0,    0,    0,    0,    0,    0,  306,    0, 
           18,    0,    0,  306,  306,  306,  306,  306,  306,    0, 
            0,    0,  306,    0,  306,  306,    0,  306,  306,  306, 
          306,    0,  306,  306,  306,  540,  540,  540,  540,  540, 
          540,  540,  540,  540,  540,  540,    0,    0,  540,  540, 
            0,  540,  540,  540,  540,  540,  540,  540,  644,    0, 
            0,    0,  540,  540,  540,  540,  540,  540,  540,    0, 
            0,  540,    0,    0,    0,    0,    0,  540,  540,    0, 
          540,  540,  540,  540,    0,  540,  540,  540,  540,  540, 
          540,    0,  540,  540,  540,  540,    0,    0,    0,    0, 
            0,    0,    0,    0,    0,    0,    0,  644,    0,    0, 
            0,    0,    0,    0,    0,    0,  540,    0,    0,  540, 
            0,    0,  540,  540,    0,  540,    0,  540,    0,    0, 
            0,  540,    0,    0,    0,    0,    0,    0,    0,  540, 
            0,    0,    0,    0,  540,  540,  540,  540,  540,  540, 
            0,    0,    0,  540,    0,  540,  540,    0,  540,  540, 
          540,  540,    0,  540,  540,  540,    0,    0,    0,    0, 
            0,    0,    0,  644,  644,  644,  644,  644,  644,    0, 
            0,  644,  644,  644,    0,    0,    0,  644,  257,  644, 
          644,  644,  644,  644,  644,  644,    0,    0,    0,    0, 
          644,  644,  644,  644,  644,  644,  644,    0,    0,  644, 
            0,    0,    0,    0,    0,  644,  644,    0,  644,  644, 
          644,  644,    0,  644,  644,  644,  644,  644,  644,    0, 
          644,  644,  644,  644,    0,    0,    0,  256,    0,    0, 
            0,    0,    0,    0,    0,    0,    0,    0,    0,    0, 
            0,    0,    0,    0,  644,    0,    0,  644,    0,    0, 
          644,  644,    0,  644,    0,  644,    0,    0,    0,  644, 
            0,    0,    0,    0,    0,    0,    0,  644,    0,    0, 
            0,    0,  644,  644,  644,  644,  644,  644,    0,    0, 
            0,  644,    0,  644,  644,    0,  644,  644,  644,  644, 
            0,  644,  644,  644,  644,  644,  644,  644,  644,  644, 
            0,    0,    0,  644,  644,    0,    0,    0,  644,  308, 
          644,  644,  644,  644,  644,  644,  644,    0,    0,    0, 
            0,  644,  644,  644,  644,  644,  644,  644,    0,    0, 
          644,    0,    0,    0,    0,    0,  644,  644,    0,  644, 
          644,  644,  644,    0,  644,  644,  644,  644,  644,  644, 
            0,  644,  644,  644,  644,    0,    0,    0,    0,    0, 
            0,    0,    0,    0,    0,    0,    0,    0,    0,    0, 
            0,    0,    0,    0,    0,  644,    0,    0,  644,    0, 
            0,  644,  644,    0,  644,    0,  644,    0,    0,    0, 
          644,    0,    0,    0,    0,    0,    0,    0,  644,    0, 
            0,  644,    0,  644,  644,  644,  644,  644,  644,    0, 
            0,    0,  644,    0,  644,  644,    0,  644,  644,  644, 
          644,    0,  644,  644,  644,    4,    5,    6,    0,    8, 
            0,    0,    0,    9,   10,    0,    0,    0,   11,  305, 
           12,   13,   14,   15,   16,   17,   18,    0,    0,    0, 
            0,   19,   20,   21,  236,  237,  238,  239,    0,    0, 
           26,    0,    0,    0,    0,    0,    0,   28,    0,    0, 
          240,  241,  242,    0,  243,   35,  244,  245,  246,  247, 
            0,   40,   41,   42,   43,    0,    0,    0,    0,    0, 
            0,    0,    0,    0,    0,    0,    0,    0,    0,    0, 
            0,    0,    0,    0,    0,  248,    0,    0,  131,    0, 
            0,   46,   47,    0,   48,    0,    0,    0,    0,    0, 
           50,    0,    0,    0,    0,    0,    0,    0,   51,    0, 
            0,    0,    0,   52,   53,   54,   55,   56,   57,    0, 
            0,    0,   58,    0,   59,   60,    0,   61,   62,   63, 
           64,    0,    0,   66,   67,    0,  310,  310,  310,    0, 
          310,    0,    0,    0,  310,  310,    0,    0,    0,  310, 
            0,  310,  310,  310,  310,  310,  310,  310,    0,    0, 
            0,    0,  310,  310,  310,  310,  310,  310,  310,    0, 
            0,  310,    0,    0,    0,    0,    0,    0,  310,    0, 
            0,  310,  310,  310,  350,  310,  310,  310,  310,  310, 
          310,    0,  310,  310,  310,  310,    0,    0,    0,    0, 
            0,    0,    0,    0,    0,    0,    0,    0,    0,    0, 
            0,    0,    0,    0,    0,    0,  310,    0,    0,  310, 
            0,  308,  310,  310,    0,  310,    0,    0,    0,    0, 
            0,  310,    0,    0,    0,    0,    0,    0,    0,  310, 
            0,    0,    0,    0,  310,  310,  310,  310,  310,  310, 
            0,    0,    0,  310,    0,  310,  310,    0,  310,  310, 
          310,  310,    0,    0,  310,  310,    4,    5,    6,    0, 
            8,    0,    0,    0,    9,   10,    0,    0,    0,   11, 
            0,   12,   13,   14,   15,   16,   17,   18,    0,    0, 
            0,    0,   19,   20,   21,  236, 
      };
   }

   private static final short[] yyTable4() {
      return new short[] {

          237,  238,  239,    0,    0,   26,    0,    0,    0,    0, 
            0,    0,   28,    0,    0,  240,  241,  242,  625,  243, 
           35,  244,  245,  246,  247,    0,   40,   41,   42,   43, 
            0,    0,    0,    0,    0,    0,    0,    0,    0,    0, 
            0,    0,    0,    0,    0,    0,    0,    0,    0,    0, 
          248,    0,    0,  131,    0,    0,   46,   47,    0,   48, 
            0,    0,    0,    0,    0,   50,    0,    0,    0,    0, 
            0,    0,    0,   51,    0,    0,    0,    0,   52,   53, 
           54,   55,   56,   57,    0,    0,    0,   58,    0,   59, 
           60,    0,   61,   62,   63,   64,    0,    0,   66,   67, 
            0,    4,    5,    6,    0,    8,    0,    0,    0,    9, 
           10,    0,    0,    0,   11,    0,   12,   13,   14,  109, 
          110,   17,   18,    0,    0,    0,    0,  111,  112,  113, 
          337,  338,  339,  340,    0,    0,  118,    0,    0,    0, 
            0,    0,    0,  119,    0,    0,  341,  342,  343,    0, 
          344,   35,  345,  346,  347,  348,    0,   40,    0,    0, 
          128,    0,    0,  470,    0,    0,    0,    0,    0,    0, 
            0,    0,    0,    0,    0,    0,    0,    0,    0,    0, 
            0,  349,    0,    0,  131,    0,    0,   46,   47,    0, 
           48,    0,    0,    0,    0,    0,    0,    0,    0,    0, 
            0,    0,    0,    0,    0,    0,    0,    0,    0,   52, 
           53,   54,   55,   56,   57,    0,    0,    0,   58,    0, 
           59,   60,    0,   61,   62,   63,   64,    0,    0,   66, 
           67,    4,    5,    6,    0,    8,    0,    0,    0,    9, 
           10,    0,    0,    0,   11,    0,   12,   13,   14,  109, 
          110,   17,   18,    0,    0,    0,    0,  111,  112,  113, 
          337,  338,  339,  340,    0,    0,  118,    0,    0,    0, 
            0,    0,    0,  119,    0,    0,  341,  342,  343,    0, 
          344,   35,  345,  346,  347,  348,    0,   40,    0,    0, 
          128,    0,    0,  470,    0,    0,    0,    0,    0,    0, 
            0,    0,    0,    0,    0,    0,    0,    0,    0,    0, 
            0,  349,    0,    0,  131,    0,    0,   46,   47,    0, 
           48,    0,    0,    0,    0,    0,    0,    0,    0,    0, 
            0,    0,    0,    0,    0,    0,    0,    0,    0,   52, 
           53,   54,   55,   56,   57,    0,    0,    0,   58,    0, 
           59,   60,    0,   61,   62,   63,   64,    0,    0,   66, 
           67,    4,    5,    6,    7,    8,    0,    0,    0,    9, 
           10,    0,    0,    0,   11,    0,   12,   13,   14,   15, 
           16,   17,   18,    0,    0,    0,    0,   19,   20,   21, 
           22,   23,   24,   25,    0,    0,   26,    0,    0,    0, 
            0,    0,   27,   28,   29,   30,   31,   32,   33,    0, 
           34,   35,   36,   37,   38,   39,    0,   40,   41,   42, 
           43,    0,    0,    0,    0,    0,    0,    0,    0,    0, 
            0,    0,    0,    0,    0,    0,    0,    0,    0,    0, 
            0,   44,    0,    0,   45,    0,    0,   46,   47,    0, 
           48,    0,   49,    0,    0,    0,   50,    0,    0,    0, 
            0,    0,    0,    0,   51,    0,    0,    0,    0,   52, 
           53,   54,   55,   56,   57,    0,    0,    0,   58,    0, 
           59,   60,    0,   61,   62,   63,   64,    0,   65,   66, 
           67,    4,    5,    6,    7,  318,    0,    0,    0,    9, 
           10,    0,    0,    0,   11,    0,   12,   13,   14,   15, 
           16,   17,   18,    0,    0,    0,    0,   19,   20,   21, 
           22,   23,   24,   25,    0,    0,   26,    0,    0,    0, 
            0,    0,   27,   28,    0,   30,   31,   32,   33,    0, 
           34,   35,   36,   37,   38,   39,    0,   40,   41,   42, 
           43,    0,    0,    0,    0,    0,    0,    0,    0,    0, 
            0,    0,    0,    0,    0,    0,    0,    0,    0,    0, 
            0,   44,    0,    0,   45,    0,    0,   46,   47,    0, 
           48,    0,   49,    0,    0,    0,   50,    0,    0,    0, 
            0,    0,    0,    0,   51,    0,    0,    0,    0,   52, 
           53,   54,   55,   56,   57,    0,    0,    0,   58,    0, 
           59,   60,    0,   61,   62,   63,   64,    0,   65,   66, 
           67,  139,  140,  141,  142,  143,  144,  145,  146,  147, 
          148,  149,  150,  151,  152,  153,  154,  155,  156,  157, 
          158,  159,  160,  161,  162,    0,    0,  163,  164,  165, 
          166,  167,  168,  169,  170,  171,  172,    0,    0,    0, 
            0,  173,  174,  175,  176,  177,  178,  179,  180,    0, 
          181,  182,  124,  125,  183,  127,    0,    0,  184,  185, 
            0,  186,  187,  188,  189,  190,  191,  192,    0,    0, 
          193,  194,    0,    0,    0,  195,  196,  197,  198,    0, 
            0,    0,    0,    0,    0,  199,    0,    0,    0,    0, 
            0,    0,  200,  201,    0,  202,  203,  204,  205,  206, 
          207,  208,  209,  210,  211,  212,    0,    0,  213,    0, 
            0,    0,    0,    0,    0,    0,    0,    0,    0,    0, 
            0,    0,    0,    0,    0,    0,    0,    0,    0,    0, 
            0,  214,  139,  140,  141,  142,  143,  144,  145,  146, 
          147,  148,  149,  150,  151,  152,  153,  154,  155,  156, 
          157,  158,  159,  160,  161,  162,    0,    0,  163,  164, 
          165,  220,  221,  222,  223,  170,  171,  172,    0,    0, 
            0,    0,  173,  174,  175,  176,  177,  224,  225,  226, 
            0,  227,  182,  357,  358,  228,  359,    0,    0,  184, 
          185,    0,  186,  187,  188,  189,  190,  191,  192,    0, 
            0,  193,  194,    0,    0,    0,  195,  196,  197,  198, 
            0,    0,    0,    0,    0,    0,    0,    0,    0,    0, 
            0,    0,    0,  200,  201,    0,  202,  203,  204,  205, 
          206,  207,  208,  209,  210,  211,  212,    0,    0,  213, 
            0,    0,    0,    0,    0,    0,    0,    0,    0,    0, 
            0,    0,    0,    0,    0,    0,    0,    0,    0,    0, 
            0,    0,  214,  139,  140,  141,  142,  143,  144,  145, 
          146,  147,  148,  149,  150,  151,  152,  153,  154,  155, 
          156,  157,  158,  159,  160,  161,  162,    0,    0,  163, 
          164,  165,  220,  221,  222,  223,  170,  171,  172,    0, 
            0,    0,    0,  173,  174,  175,  176,  177,  224,  225, 
          226,    0,  227,  182,  307,    0,  228,    0,    0,    0, 
          184,  185,    0,  186,  187,  188,  189,  190,  191,  192, 
            0,    0,  193,  194,    0,    0,    0,  195,  196,  197, 
          198,    0,    0,    0,    0,    0,    0,    0,    0,    0, 
            0,    0,    0,    0,  200,  201,    0,  202,  203,  204, 
          205,  206,  207,  208,  209,  210,  211,  212,    0,    0, 
          213,   52,    0,    0,    0,    0,    0,    0,    0,    0, 
            0,    0,    0,    0,    0,    0,    0,    0,    0,    0, 
            0,    0,    0,  214,  139,  140,  141,  142,  143,  144, 
          145,  146,  147,  148,  149,  150,  151,  152,  153,  154, 
          155,  156,  157,  158,  159,  160,  161,  162,    0,    0, 
          163,  164,  165,  220,  221,  222,  223,  170,  171,  172, 
            0,    0,    0,    0,  173,  174,  175,  176,  177,  224, 
          225,  226,    0,  227,  182,    0,    0,  228,    0,    0, 
            0,  184,  185,    0,  186,  187,  188,  189,  190,  191, 
          192,    0,    0,  193,  194,    0,    0,    0,  195,  196, 
          197,  198,    0,    0,    0,    0,    0,    0,    0,    0, 
            0,    0,    0,    0,    0,  200,  201,    0,  202,  203, 
          204,  205,  206,  207,  208,  209,  210,  211,  212,    0, 
            0,  213,   52,    0,    0,    0,    0,    0,    0,    0, 
            0,    0,    0,    0,    0,    0,    0,    0,    0,    0, 
            0,    0,    0,    0,  214,  139,  140,  141,  142,  143, 
          144,  145,  146,  147,  148,  149,  150,  151,  152,  153, 
          154,  155,  156,  157,  158,  159,  160,  161,  162,    0, 
            0,  163,  164,  165,  220,  221,  222,  223,  170,  171, 
          172,    0,    0,    0,    0,  173,  174,  175,  176,  177, 
          224,  225,  226,    0,  227,  182,    0,    0,  228,    0, 
            0,    0,  184,  185,    0,  186,  187,  188,  189,  190, 
          191,  192,    0,    0,  193,  194,    0,    0,    0,  195, 
          196,  197,  198,    0,    0,    0,    0,    0,    0,    0, 
            0,    0,    0,    0,    0,    0,  200,  201,    0,  202, 
          203,  204,  205,  206,  207,  208,  209,  210,  211,  212, 
            0,    0,  213,    0,    0,    0,    0,    0,    0,    0, 
            3,    4,    5,    6,    7,    8,    0,    0,    0,    9, 
           10,    0,    0,    0,   11,  214,   12,   13,   14,   15, 
           16,   17,   18,    0,    0,    0,    0,   19,   20,   21, 
           22,   23,   24,   25,    0,    0,   26,    0,    0,    0, 
            0,    0,   27,   28,   29,   30,   31,   32,   33,    0, 
           34,   35,   36,   37,   38,   39,    0,   40,   41,   42, 
           43,    0,    0,    0,    0,    0,    0,    0,    0,    0, 
            0,    0,    0,    0,    0,    0,    0,    0,    0,    0, 
            0,   44,    0,    0,   45,    0,    0,   46,   47,    0, 
           48,    0,   49,    0,    0,    0,   50,    0,    0,    0, 
            0,    0,    0,    0,   51,    0,    0,    0,    0,   52, 
           53,   54,   55,   56,   57,    0,    0,    0,   58,    0, 
           59,   60,    0,   61,   62,   63,   64,    0,   65,   66, 
           67,  317,    4,    5,    6,    7,  318,    0,    0,    0, 
            9,   10,    0,    0,    0,   11,    0,   12,   13,   14, 
           15,   16,   17,   18,    0,    0,    0,    0,   19,   20, 
           21,   22,   23,   24,   25,    0,    0,   26,    0,    0, 
            0,    0,    0,   27,   28,    0,   30,   31,   32,   33, 
            0,   34,   35,   36,   37,   38,   39,    0,   40,   41, 
           42,   43,    0,    0,    0,    0,    0,    0,    0,    0, 
            0,    0,    0,    0,    0,    0,    0,    0,    0,    0, 
            0,    0,   44,    0,    0,  319,    0,    0,   46,   47, 
            0,   48,    0,   49,    0,    0,    0,   50,    0,    0, 
            0,    0,    0,    0,    0,   51,    0,    0,    0,    0, 
           52,   53,   54,   55,   56,   57,    0,    0,    0,   58, 
            0,   59,   60,    0,   61,   62,   63,   64,    0,   65, 
           66,   67,  317,    4,    5,    6,    7,  318,    0,    0, 
            0,    9,   10,    0,    0,    0,   11,    0,   12,   13, 
           14,   15,   16,   17,   18,    0,    0,    0,    0,   19, 
           20,   21,   22,   23,   24,   25,    0,    0,   26,    0, 
            0,    0,    0,    0,   27,   28,    0,   30,   31,   32, 
           33,    0,   34,   35,   36,   37,   38,   39,    0,   40, 
           41,   42,   43,    0,    0,    0,    0,    0,    0,    0, 
            0,    0,    0,    0,    0,    0,    0,    0,    0,    0, 
            0,    0,    0,   44,    0,    0,   45,    0,    0,   46, 
           47,    0,   48,    0,   49,    0,    0,    0,   50,    0, 
            0,    0,    0,    0,    0,    0,   51,    0,    0,    0, 
            0,   52,   53,   54,   55,   56,   57,    0,    0,    0, 
           58,    0,   59,   60,    0,   61,   62,   63,   64,    0, 
           65,   66,   67,    4,    5,    6,    0,    8,    0,    0, 
            0,    9,   10,    0,    0,    0,   11,    0,   12,   13, 
           14,   15,   16,   17,   18,    0,    0,    0,    0,   19, 
           20,   21,  236,  237,  238,  239,    0,    0,  118,    0, 
            0,    0,    0,    0,    0,   28,    0,    0,  240,  241, 
          242,    0,  243,   35,  244,  245,  246,  247,  282,   40, 
           41,   42,   43,    0,    0,    0,    0,    0,    0,    0, 
            0,    0,    0,    0,    0,    0,    0,    0,    0,    0, 
            0,    0,    0,  248,    0,    0,  131,    0,    0,   46, 
           47,    0,   48,    0,  283,    0,  284,    0,   50,    0, 
            0,    0,    0,    0,    0,    0,  285,    0,    0,    0, 
            0,   52,  286,   54,   55,   56,   57,    0,    0,    0, 
           58,    0,   59,   60,    0,   61,   62,   63,   64,    0, 
            0,   66,   67,  287,    4,    5,    6,    0,    8,    0, 
            0,    0,    9,   10,    0,    0,    0,   11,    0,   12, 
           13,   14,  109,  110,   17,   18,    0,    0,    0,    0, 
          111,  112,  113,  236,  237,  238,  239,    0,    0,  118, 
            0,    0,    0,    0,    0,    0,   28,    0,    0,  240, 
          241,  242,    0,  243,   35,  244,  245,  246,  247,  282, 
           40,   41,   42,   43,    0,    0,    0,    0,    0,    0, 
            0,    0,    0,    0,    0,    0,    0,    0,    0,    0, 
            0,    0,    0,    0,  248,    0,    0,  131,    0,    0, 
           46,   47,    0,   48,    0,  680,    0,  284,    0,   50, 
            0,    0,    0,    0,    0,    0,    0,  285,    0,    0, 
            0,    0,   52,  286,   54,   55,   56,   57,    0,    0, 
            0,   58,    0,   59,   60,    0,   61,   62,   63,   64, 
            0,    0,   66,   67,  287,  281,  281,  281,    0,  281, 
            0,    0,    0,  281,  281,    0,    0,    0,  281,    0, 
          281,  281,  281,  281,  281,  281,  281,    0,    0,    0, 
            0,  281,  281,  281,  281,  281,  281,  281,    0,    0, 
          281,    0,    0,    0,    0,    0,    0,  281,    0,    0, 
          281,  281,  281,    0,  281,  281,  281,  281,  281,  281, 
          281,  281,  281,  281,  281,    0,    0,    0,    0,    0, 
            0,    0,    0,    0,    0,    0,    0,    0,    0,    0, 
            0,    0,    0,    0,    0,  281,    0,    0,  281,    0, 
            0,  281,  281,    0,  281,    0,  281,    0,  281,    0, 
          281,    0,    0,    0,    0,    0,    0,    0,  281,    0, 
            0,    0,    0,  281,  281,  281,  281,  281,  281,    0, 
            0,    0,  281,    0,  281,  281,    0,  281,  281,  281, 
          281,    0,    0,  281,  281,  281,    4,    5,    6,    0, 
            8,    0,    0,    0,    9,   10,    0,    0,    0,   11, 
            0,   12,   13,   14,  109,  110,   17,   18,    0,    0, 
            0,    0,  111,  112,  113,  236,  237,  238,  239,    0, 
            0,  118,    0,    0,    0,    0,    0,    0,   28,    0, 
            0,  240,  241,  242,    0,  243,   35,  244,  245,  246, 
          247,  282,   40,   41,   42,   43,    0,    0,    0,    0, 
            0,    0,    0,    0,    0,    0,    0,    0,    0,    0, 
            0,    0,    0,    0,    0,    0,  248,    0,    0,  131, 
            0,    0,   46,   47,    0,   48,    0,  283,    0,    0, 
            0,   50,    0,    0,    0,    0,    0,    0,    0,  285, 
            0,    0,    0,    0,   52,  286,   54,   55,   56,   57, 
            0,    0,    0,   58,    0,   59,   60,    0,   61,   62, 
           63,   64,    0,    0,   66,   67,  287,    4,    5,    6, 
            0,    8,    0,    0,    0,    9,   10,    0,    0,    0, 
           11,    0,   12,   13,   14,  109,  110,   17,   18,    0, 
            0,    0,    0,  111,  112,  113,  236,  237,  238,  239, 
            0,    0,  118,    0,    0,    0,    0,    0,    0,   28, 
            0,    0,  240,  241,  242,    0,  243,   35,  244,  245, 
          246,  247,  282,   40,   41,   42,   43,    0,    0,    0, 
            0,    0,    0,    0,    0,    0,    0,    0,    0,    0, 
            0,    0,    0,    0,    0,    0,    0,  248,    0,    0, 
          131,    0,    0,   46,   47,    0,   48,    0,    0,    0, 
          284,    0,   50,    0,    0,    0,    0,    0,    0,    0, 
          285,    0,    0,    0,    0,   52,  286,   54,   55,   56, 
           57,    0,    0,    0,   58,    0,   59,   60,    0,   61, 
           62,   63,   64,    0,    0,   66,   67,  287,    4,    5, 
            6,    0,    8,    0,    0,    0,    9,   10,    0,    0, 
            0,   11,    0,   12,   13,   14,  109,  110,   17,   18, 
            0,    0,    0,    0,  111,  112,  113,  236,  237,  238, 
          239,    0,    0,  118,    0,    0,    0,    0,    0,    0, 
           28,    0,    0,  240,  241,  242,    0,  243,   35,  244, 
          245,  246,  247,  282,   40,   41,   42,   43,    0,    0, 
            0,    0,    0,    0,    0,    0,    0,    0,    0,    0, 
            0,    0,    0,    0,    0,    0,    0,    0,  248,    0, 
            0,  131,    0,    0,   46,   47,    0,   48,    0,  680, 
            0,    0,    0,   50,    0,    0,    0,    0,    0,    0, 
            0,  285,    0,    0,    0,    0,   52,  286,   54,   55, 
           56,   57,    0,    0,    0,   58,    0,   59,   60,    0, 
           61,   62,   63,   64,    0,    0,   66,   67,  287,    4, 
            5,    6,    0,    8,    0,    0,    0,    9,   10,    0, 
            0,    0,   11,    0,   12,   13,   14,  109,  110,   17, 
           18,    0,    0,    0,    0,  111,  112,  113,  236,  237, 
          238,  239,    0,    0,  118,    0,    0,    0,    0,    0, 
            0,   28,    0,    0,  240,  241,  242,    0,  243,   35, 
          244,  245,  246,  247,  282,   40,   41,   42,   43,    0, 
            0,    0,    0,    0,    0,    0,    0,    0,    0,    0, 
            0,    0,    0,    0,    0,    0,    0,    0,    0,  248, 
            0,    0,  131,    0,    0,   46,   47,    0,   48,    0, 
            0,    0,    0,    0,   50,    0,    0,    0,    0,    0, 
            0,    0,  285,    0,    0,    0,    0,   52,  286,   54, 
           55,   56,   57,    0,    0,    0,   58,    0,   59,   60, 
            0,   61,   62,   63,   64,    0,    0,   66,   67,  287, 
            4,    5,    6,    7,    8,    0,    0,    0,    9,   10, 
            0,    0,    0,   11,    0,   12,   13,   14,   15,   16, 
           17,   18,    0,    0,    0,    0,   19,   20,   21,   22, 
           23,   24,   25,    0,    0,   26,    0,    0,    0,    0, 
            0,   27,   28,   29,   30,   31,   32,   33,    0,   34, 
           35,   36,   37,   38,   39,    0,   40,   41,   42,   43, 
            0,    0,    0,    0,    0,    0,    0,    0,    0,    0, 
            0,    0,    0,    0,    0,    0,    0,    0,    0,    0, 
           44,    0,    0,   45,    0,    0,   46,   47,    0,   48, 
            0,   49,    0,    0,    0,   50,    0,    0,    0,    0, 
            0,    0,    0,   51,    0,    0,    0,    0,   52,   53, 
           54,   55,   56,   57,    0,    0,    0,   58,    0,   59, 
           60,    0,   61,   62,   63,   64,    0,   65,   66,   67, 
            4,    5,    6,    7,    8,    0,    0,    0,    9,   10, 
            0,    0,    0,   11,    0,   12,   13,   14,   15,   16, 
           17,   18,    0,    0,    0,    0,   19,   20,   21,   22, 
           23,   24,   25,    0,    0,   26,    0,    0,    0,    0, 
            0,   27,   28,    0,   30,   31,   32,   33,    0,   34, 
           35,   36,   37,   38,   39,    0,   40,   41,   42,   43, 
            0,    0,    0,    0,    0,    0,    0,    0,    0,    0, 
            0,    0,    0,    0,    0,    0,    0,    0,    0,    0, 
           44,    0,    0,   45,    0,    0,   46,   47,    0,   48, 
            0,   49,    0,    0,    0,   50,    0,    0,    0,    0, 
            0,    0,    0,   51,    0,    0,    0,    0,   52,   53, 
           54,   55,   56,   57,    0,    0,    0,   58,    0,   59, 
           60,    0,   61,   62,   63,   64,    0,   65,   66,   67, 
            4,    5,    6,    0,    8,    0,    0,    0,    9,   10, 
            0,    0,    0,   11,    0,   12,   13,   14,  109,  110, 
           17,   18,    0,    0,    0,    0,  111,  112,  113,  236, 
          237,  238,  239,    0,    0,  118,    0,    0,    0,    0, 
            0,    0,   28,    0,    0,  240,  241,  242,    0,  243, 
           35,  244,  245,  246,  247,    0,   40,   41,   42,   43, 
            0,    0,    0,    0,    0,    0,    0,    0,    0,    0, 
            0,    0,    0,    0,    0,    0,    0,    0,    0,    0, 
          248,    0,    0,  131,  505,    0,   46,   47,    0,   48, 
            0,    0,    0,    0,    0,   50,    0,    0,    0,    0, 
            0,    0,    0,  285,    0,    0,    0,    0,   52,   53, 
           54,   55,   56,   57,    0,    0,    0,   58,    0,   59, 
           60,    0,   61,   62,   63,   64,    0,    0,   66,   67, 
            4,    5,    6,    0,    8,    0,    0,    0,    9,   10, 
            0,    0,    0,   11,    0,   12,   13,   14,   15,   16, 
           17,   18,    0,    0,    0,    0,   19,   20,   21,  236, 
          237,  238,  239,    0,    0,  118,    0,    0,    0,    0, 
            0,    0,   28,    0,    0,  240,  241,  242,    0,  243, 
           35,  244,  245,  246,  247,    0,   40,   41,   42,   43, 
            0,    0,    0,    0,    0,    0,    0,    0,    0,    0, 
            0,    0,    0,    0,    0,    0,    0,    0,    0,    0, 
          248,    0,    0,  131,    0,    0,   46,   47,    0,   48, 
            0,  616,    0,    0,    0,   50,    0,    0,    0,    0, 
            0,    0,    0,  285,    0,    0,    0,    0,   52,   53, 
           54,   55,   56,   57,    0,    0,    0,   58,    0,   59, 
           60,    0,   61,   62,   63,   64,    0,    0,   66,   67, 
            4,    5,    6,    0,    8,    0,    0,    0,    9,   10, 
            0,    0,    0,   11,    0,   12,   13,   14,  109,  110, 
           17,   18,    0,    0,    0,    0,  111,  112,  113,  236, 
          237,  238,  239,    0,    0,  118,    0,    0,    0,    0, 
            0,    0,   28,    0,    0,  240,  241,  242,    0,  243, 
           35,  244,  245,  246,  247,    0,   40,   41,   42,   43, 
            0,    0,    0,    0,    0,    0,    0,    0,    0,    0, 
            0,    0,    0,    0,    0,    0,    0,    0,    0,    0, 
          248,    0,    0,  131,    0,    0,   46,   47,    0,   48, 
            0,  283,    0,    0,    0,   50,    0,    0,    0,    0, 
            0,    0,    0,  285,    0,    0,    0,    0,   52,   53, 
           54,   55,   56,   57,    0,    0,    0,   58,    0,   59, 
           60,    0,   61,   62,   63,   64,    0,    0,   66,   67, 
            4,    5,    6,    0,    8,    0,    0,    0,    9,   10, 
            0,    0,    0,   11,    0,   12,   13,   14,  109,  110, 
           17,   18,    0,    0,    0,    0,  111,  112,  113,  236, 
          237,  238,  239,    0,    0,  118,    0,    0,    0,    0, 
            0,    0,   28,    0,    0,  240,  241,  242,    0,  243, 
           35,  244,  245,  246,  247,    0,   40,   41,   42,   43, 
            0,    0,    0,    0,    0,    0,    0,    0,    0,    0, 
            0,    0,    0,    0,    0,    0,    0,    0,    0,    0, 
          248,    0,    0,  131,    0,    0,   46,   47,    0,   48, 
            0,  616,    0,    0,    0,   50,    0,    0,    0,    0, 
            0,    0,    0,  285,    0,    0,    0,    0,   52,   53, 
           54,   55,   56,   57,    0,    0,    0,   58,    0,   59, 
           60,    0,   61,   62,   63,   64,    0,    0,   66,   67, 
            4,    5,    6,    0,    8,    0,    0,    0,    9,   10, 
            0,    0,    0,   11,    0,   12,   13,   14,  109,  110, 
           17,   18,    0,    0,    0,    0,  111,  112,  113,  236, 
          237,  238,  239,    0,    0,  118,    0,    0,    0,    0, 
            0,    0,   28,    0,    0,  240,  241,  242,    0,  243, 
           35,  244,  245,  246,  247,    0,   40,   41,   42,   43, 
            0,    0,    0,    0,    0,    0,    0,    0,    0,    0, 
            0,    0,    0,    0,    0,    0,    0,    0,    0,    0, 
          248,    0,    0,  131,    0,    0,   46,   47,    0,   48, 
            0,  889,    0,    0,    0,   50,    0,    0,    0,    0, 
            0,    0,    0,  285,    0,    0,    0,    0,   52,   53, 
           54,   55,   56,   57,    0,    0,    0,   58,    0,   59, 
           60,    0,   61,   62,   63,   64,    0,    0,   66,   67, 
            4,    5,    6,    0,    8,    0,    0,    0,    9,   10, 
            0,    0,    0,   11,    0,   12,   13,   14,  109,  110, 
           17,   18,    0,    0,    0,    0,  111,  112,  113,  236, 
          237,  238,  239,    0,    0,  118,    0,    0,    0,    0, 
            0,    0,   28,    0,    0,  240,  241,  242,    0,  243, 
           35,  244,  245,  246,  247,    0,   40,   41,   42,   43, 
            0,    0,    0,    0,    0,    0,    0,    0,    0,    0, 
            0,    0,    0,    0,    0,    0,    0,    0,    0,    0, 
          248,    0,    0,  131,    0,    0,   46,   47,    0,   48, 
            0,  680,    0,    0,    0,   50,    0,    0,    0,    0, 
            0,    0,    0,  285,    0,    0,    0,    0,   52,   53, 
           54,   55,   56,   57,    0,    0,    0,   58,    0,   59, 
           60,    0,   61,   62,   63,   64,    0,    0,   66,   67, 
          633,  633,  633,    0,  633,    0,    0,    0,  633,  633, 
            0,    0,    0,  633,    0,  633,  633,  633,  633,  633, 
          633,  633,    0,    0,    0,    0,  633,  633,  633,  633, 
          633,  633,  633,    0,    0,  633,    0,    0,    0,    0, 
            0,    0,  633,    0,    0,  633,  633,  633,    0,  633, 
          633,  633,  633,  633,  633,    0,  633,  633,  633,  633, 
            0,    0,    0,    0,    0,    0,    0,    0,    0,    0, 
            0,    0,    0,    0,    0,    0,    0,    0,    0,    0, 
          633,    0,    0,  633,  633,    0,  633,  633,    0,  633, 
            0,    0,    0,    0,    0,  633,    0,    0,    0,    0, 
            0,    0,    0,  633,    0,    0,    0,    0,  633,  633, 
          633,  633,  633,  633,    0,    0,    0,  633,    0,  633, 
          633,    0,  633,  633,  633,  633,    0,    0,  633,  633, 
            4,    5,    6,    0,    8,    0,    0,    0,    9,   10, 
            0,    0,    0,   11,    0,   12,   13,   14,   15,   16, 
           17,   18,    0,    0,    0,    0,   19,   20,   21,  236, 
          237,  238,  239,    0,    0,   26,    0,    0,    0,    0, 
            0,    0,   28,    0,    0,  240,  241,  242,    0,  243, 
           35,  244,  245,  246,  247,    0,   40,   41,   42,   43, 
            0,    0,    0,    0,    0,    0,    0,    0,    0,    0, 
            0,    0,    0,    0,    0,    0,    0,    0,    0,    0, 
          248,    0,    0,  131,    0,    0,   46,   47,    0,   48, 
            0,    0,    0,    0,    0,   50,    0,    0,    0,    0, 
            0,    0,    0,   51,    0,    0,    0,    0,   52,   53, 
           54,   55,   56,   57,    0,    0,    0,   58,    0,   59, 
           60,    0,   61,   62,   63,   64,    0,    0,   66,   67, 
            4,    5,    6,    0,    8,    0,    0,    0,    9,   10, 
            0,    0,    0,   11,    0,   12,   13,   14,  109,  110, 
           17,   18,    0,    0,    0,    0,  111,  112,  113,  236, 
          237,  238,  239,    0,    0,  118,    0,    0,    0,    0, 
            0,    0,   28,    0,    0,  240,  241,  242,    0,  243, 
           35,  244,  245,  246,  247,    0,   40,   41,   42,   43, 
            0,    0,    0,    0,    0,    0,    0,    0,    0,    0, 
            0,    0,    0,    0,    0,    0,    0,    0,    0,    0, 
          248,    0,    0,  131,    0,    0,   46,   47,    0,   48, 
            0,    0,    0,    0,    0,   50,    0,    0,    0,    0, 
            0,    0,    0,  285,    0,    0,    0,    0,   52,   53, 
           54,   55,   56,   57,    0,    0,    0,   58,    0,   59, 
           60,    0,   61,   62,   63,   64,    0,    0,   66,   67, 
            4,    5,    6,    0,    8,    0,    0,    0,    9,   10, 
            0,    0,    0,   11,    0,   12,   13,   14,   15,   16, 
           17,   18,    0,    0,    0,    0,   19,   20,   21,  236, 
          237,  238,  239,    0,    0,  118,    0,    0,    0,    0, 
            0,    0,   28,    0,    0,  240,  241,  242,    0,  243, 
           35,  244,  245,  246,  247,    0,   40,   41,   42,   43, 
            0,    0,    0,    0,    0,    0,    0,    0,    0,    0, 
            0,    0,    0,    0,    0,    0,    0,    0,    0,    0, 
          248,    0,    0,  131,    0,    0,   46,   47,    0,   48, 
            0,    0,    0,    0,    0,   50,    0,    0,    0,    0, 
            0,    0,    0,  285,    0,    0,    0,    0,   52,   53, 
           54,   55,   56,   57,    0,    0,    0,   58,    0,   59, 
           60,    0,   61,   62,   63,   64,    0,    0,   66,   67, 
          633,  633,  633,    0,  633,    0,    0,    0,  633,  633, 
            0,    0,    0,  633,    0,  633,  633,  633,  633,  633, 
          633,  633,    0,    0,    0,    0,  633,  633,  633,  633, 
          633,  633,  633,    0,    0,  633,    0,    0,    0,    0, 
            0,    0,  633,    0,    0,  633,  633,  633,    0,  633, 
          633,  633,  633,  633,  633,    0,  633,  633,  633,  633, 
            0,    0,    0,    0,    0,    0,    0,    0,    0,    0, 
            0,    0,    0,    0,    0,    0,    0,    0,    0,    0, 
          633,    0,    0,  633,    0,    0,  633,  633,    0,  633, 
            0,    0,    0,    0,    0,  633,    0,    0,    0,    0, 
            0,    0,    0,  633,    0,    0,    0,    0,  633,  633, 
          633,  633,  633,  633,    0,    0,    0,  633,    0,  633, 
          633,    0,  633,  633,  633,  633,    0,    0,  633,  633, 
            4,    5,    6,    0,    8,    0,    0,    0,    9,   10, 
            0,    0,    0,   11,    0,   12,   13,   14,  109,  110, 
           17,   18,    0,    0,    0,    0,  111,  112,  113,  114, 
          115,  116,  117,    0,    0,  118,    0,    0,    0,    0, 
            0,    0,  119,    0,    0,  120,  121,  122,    0,  123, 
           35,  124,  125,  126,  127,    0,   40,    0,    0,  128, 
            0,    0,    0,    0,    0,    0,    0,    0,    0,    0, 
            0,    0,    0,    0,    0,    0,  129,    0,    0,    0, 
          130,    0,    0,  131,    0,    0,   46,   47,    0,   48, 
            0,    0,    0,    0,    0,    0,    0,    0,    0,    0, 
            0,    0,    0,    0,    0,    0,    0,    0,   52,   53, 
           54,   55,   56,   57,    0,    0,    0,   58,    0,   59, 
           60,    0,   61,   62,   63,   64,    0,    0,   66,   67, 
            4,    5,    6,    0,    8,    0,    0,    0,    9,   10, 
            0,    0,    0,   11,    0,   12,   13,   14,  109,  110, 
           17,   18,    0,    0,    0,    0,  111,  112,  113,  264, 
          265,  266,  267,    0,    0,  118,    0,    0,    0,    0, 
            0,    0,  119,    0,    0,  268,  269,  270,    0,  271, 
           35,  272,  273,  274,  275,    0,   40,    0,    0,  128, 
            0,    0,    0,    0,    0,    0,    0,    0,    0,    0, 
            0,    0,    0,    0,    0,    0,    0,    0,    0,    0, 
          276,    0,    0,   45,    0,    0,   46,   47,    0,   48, 
            0,   49,    0,    0,    0,    0,    0,    0,    0,    0, 
            0,    0,    0,    0,    0,    0,    0,    0,   52,   53, 
           54,   55,   56,   57,    0,    0,    0,   58,    0,   59, 
           60,    0,   61,   62,   63,   64,    0,    0,   66,   67, 
            4,    5,    6,    0,    8,    0,    0,    0,    9,   10, 
            0,    0,    0,   11,    0,   12,   13,   14,  109,  110, 
           17,   18,    0,    0,    0,    0,  111,  112,  113,  337, 
          338,  339,  340,    0,    0,  118,    0,    0,    0,    0, 
            0,    0,  119,    0,    0,  341,  342,  343,    0,  344, 
           35,  345,  346,  347,  348,    0,   40,    0,    0,  128, 
            0,    0,    0,    0,    0,    0,    0,    0,    0,    0, 
            0,    0,    0,    0,    0,    0,    0,    0,    0,    0, 
          349,    0,    0,  426,    0,    0,   46,   47,    0,   48, 
            0,  427,    0,    0,    0,    0,    0,    0,    0,    0, 
            0,    0,    0,    0,    0,    0,    0,    0,   52,   53, 
           54,   55,   56,   57,    0,    0,    0,   58,    0,   59, 
           60,    0,   61,   62,   63,   64,    0,    0,   66,   67, 
            4,    5,    6,    0,    8,    0,    0,    0,    9,   10, 
            0,    0,    0,   11,    0,   12,   13,   14,  109,  110, 
           17,   18,    0,    0,    0,    0,  111,  112,  113,  114, 
          115,  116,  117,    0,    0,  118,    0,    0,    0,    0, 
            0,    0,  119,    0,    0,  120,  121,  122,    0,  123, 
           35,  124,  125,  126,  127,    0,   40,    0,    0,  128, 
            0,    0,    0,    0,    0,    0,    0,    0,    0,    0, 
            0,    0,    0,    0,    0,    0,    0,    0,    0,    0, 
          130,    0,    0,  131,    0,    0,   46,   47,    0,   48, 
            0,    0,    0,    0,    0,    0,    0,    0,    0,    0, 
            0,    0,    0,    0,    0,    0,    0,    0,   52,   53, 
           54,   55,   56,   57,    0,    0,    0,   58,    0,   59, 
           60,    0,   61,   62,   63,   64,    0,    0,   66,   67, 
            4,    5,    6,    0,    8,    0,    0,    0,    9,   10, 
            0,    0,    0,   11,    0,   12,   13,   14,  109,  110, 
           17,   18,    0,    0,    0,    0,  111,  112,  113,  337, 
          338,  339,  340,    0,    0,  118,    0,    0,    0,    0, 
            0,    0,  119,    0,    0,  341,  342,  343,    0,  344, 
           35,  345,  346,  347,  348,    0,   40,    0,    0,  128, 
            0,    0,    0,    0,    0,    0,    0,    0,    0,    0, 
            0,    0,    0,    0,    0,    0,    0,    0,    0,    0, 
          349,    0,    0,  426,    0,    0,   46,   47,    0,   48, 
            0,    0,    0,    0,    0,    0,    0,    0,    0,    0, 
            0,    0,    0,    0,    0,    0,    0,    0,   52,   53, 
           54,   55,   56,   57,    0,    0,    0,   58,    0,   59, 
           60,    0,   61,   62,   63,   64,    0,    0,   66,   67, 
            4,    5,    6,    0,    8,    0,    0,    0,    9,   10, 
            0,    0,    0,   11,    0,   12,   13,   14,  109,  110, 
           17,   18,    0,    0,    0,    0,  111,  112,  113,  114, 
          115,  116,  117,    0,    0,  118,    0,    0,    0,    0, 
            0,    0,  119,    0,    0,  120,  121,  122,    0,  952, 
           35,  124,  125,  953,  127,    0,   40,    0,    0,  128, 
            0,    0,    0,    0,    0,    0,    0,    0,    0,    0, 
            0,    0,    0,    0,    0,    0,    0,    0,    0,    0, 
          954,    0,    0,  131,    0,    0,   46,   47,    0,   48, 
            0,    0,    0,    0,    0,    0,    0,    0,    0,    0, 
            0,    0,    0,    0,    0,    0,    0,    0,   52,   53, 
           54,   55,   56,   57,    0,    0,    0,   58,    0,   59, 
           60,    0,   61,   62,   63,   64,    0,    0,   66,   67, 
            4,    5,    6,    0,    8,    0,    0,    0,    9,   10, 
            0,    0,    0,   11,    0,   12,   13,   14,  109,  110, 
           17,   18,    0,    0,    0,    0,  111,  112,  113,  968, 
          969,  970,  971,    0,    0,  118,    0,    0,    0,    0, 
            0,    0,  119,    0,    0,  972,  973,  974,    0,  975, 
           35,  976,  977,  978,  979,    0,   40,    0,    0,  128, 
            0,    0,    0,    0,    0,    0,    0,    0,    0,    0, 
            0,    0,    0,    0,    0,    0,    0,    0,    0,    0, 
          980,    0,    0,  131,    0,    0,   46,   47,    0,   48, 
            0,    0,    0,    0,    0,    0,    0,    0,    0,    0, 
            0,    0,    0,    0,    0,    0,    0,    0,   52,   53, 
           54,   55,   56,   57,    0,    0,    0,   58,    0,   59, 
           60,    0,   61,   62,   63,   64,  597,  598,   66,   67, 
          599,    0,    0,    0,  184,  185,    0,  186,  187,  188, 
          189,  190,  191,  192,    0,    0,  193,  194,    0,    0, 
            0,  195,  196,  197,  198,    0,    0,    0,    0,    0, 
            0,  301,    0,    0,    0,    0,    0,    0,  200,  201, 
            0,  202,  203,  204,  205,  206,  207,  208,  209,  210, 
          211,  212,    0,    0,  213,  605,  606,    0,    0,  607, 
            0,    0,    0,  184,  185,    0,  186,  187,  188,  189, 
          190,  191,  192,    0,    0,  193,  194,  214,    0,    0, 
          195,  196,  197,  198,    0,    0,    0,    0,    0,    0, 
          301,    0,    0,    0,    0,    0,    0,  200,  201,    0, 
          202,  203,  204,  205,  206,  207,  208,  209,  210,  211, 
          212,    0,    0,  213,  640,  598,    0,    0,  641,    0, 
            0,    0,  184,  185,    0,  186,  187,  188,  189,  190, 
          191,  192,    0,    0,  193,  194,  214,    0,    0,  195, 
          196,  197,  198,    0,    0,    0,    0,    0,    0,  301, 
            0,    0,    0,    0,    0,    0,  200,  201,    0,  202, 
          203,  204,  205,  206,  207,  208,  209,  210,  211,  212, 
            0,    0,  213,  611,  606,    0,    0,  612,    0,    0, 
            0,  184,  185,    0,  186,  187,  188,  189,  190,  191, 
          192,    0,    0,  193,  194,  214,    0,    0,  195,  196, 
          197,  198,    0,    0,    0,    0,    0,    0,  301,    0, 
            0,    0,    0,    0,    0,  200,  201,    0,  202,  203, 
          204,  205,  206,  207,  208,  209,  210,  211,  212,    0, 
            0,  213,  658,  598,    0,    0,  659,    0,    0,    0, 
          184,  185,    0,  186,  187,  188,  189,  190,  191,  192, 
            0,    0,  193,  194,  214,    0,    0,  195,  196,  197, 
          198,    0,    0,    0,    0,    0,    0,  301,    0,    0, 
            0,    0,    0,    0,  200,  201,    0,  202,  203,  204, 
          205,  206,  207,  208,  209,  210,  211,  212,    0,    0, 
          213,  661,  606,    0,    0,  662,    0,    0,    0,  184, 
          185,    0,  186,  187,  188,  189,  190,  191,  192,    0, 
            0,  193,  194,  214,    0,    0,  195,  196,  197,  198, 
            0,    0,    0,    0,    0,    0,  301,    0,    0,    0, 
            0,    0,    0,  200,  201,    0,  202,  203,  204,  205, 
          206,  207,  208,  209,  210,  211,  212,    0,    0,  213, 
          672,  598,    0,    0,  673,    0,    0,    0,  184,  185, 
            0,  186,  187,  188,  189,  190,  191,  192,    0,    0, 
          193,  194,  214,    0,    0,  195,  196,  197,  198,    0, 
            0,    0,    0,    0,    0,  301,    0,    0,    0,    0, 
            0,    0,  200,  201,    0,  202,  203,  204,  205,  206, 
          207,  208,  209,  210,  211,  212,    0,    0,  213,  675, 
          606,    0,    0,  676,    0,    0,    0,  184,  185,    0, 
          186,  187,  188,  189,  190,  191,  192,    0,    0,  193, 
          194,  214,    0,    0,  195,  196,  197,  198,    0,    0, 
            0,    0,    0,    0,  301,    0,    0,    0,    0,    0, 
            0,  200,  201,    0,  202,  203,  204,  205,  206,  207, 
          208,  209,  210,  211,  212,    0,    0,  213,  703,  598, 
            0,    0,  704,    0,    0,    0,  184,  185,    0,  186, 
          187,  188,  189,  190,  191,  192,    0,    0,  193,  194, 
          214,    0,    0,  195,  196,  197,  198,    0,    0,    0, 
            0,    0,    0,  301,    0,    0,    0,    0,    0,    0, 
          200,  201,    0,  202,  203,  204,  205,  206,  207,  208, 
          209,  210,  211,  212,    0,    0,  213,  706,  606,    0, 
            0,  707,    0,    0,    0,  184,  185,    0,  186,  187, 
          188,  189,  190,  191,  192,    0,    0,  193,  194,  214, 
            0,    0,  195,  196,  197,  198,    0,    0,    0,    0, 
            0,    0,  301,    0,    0,    0,    0,    0,    0,  200, 
          201,    0,  202,  203,  204,  205,  206,  207,  208,  209, 
          210,  211,  212,    0,    0,  213, 1058,  598,    0,    0, 
         1059,    0,    0,    0,  184,  185,    0,  186,  187,  188, 
          189,  190,  191,  192,    0,    0,  193,  194,  214,    0, 
            0,  195,  196,  197,  198,    0,    0,    0,    0,    0, 
            0,  301,    0,    0,    0,    0,    0,    0,  200,  201, 
            0,  202,  203,  204,  205,  206,  207,  208,  209,  210, 
          211,  212,    0,    0,  213, 1063,  598,    0,    0, 1064, 
            0,    0,    0,  184,  185,    0,  186,  187,  188,  189, 
          190,  191,  192,    0,    0,  193,  194,  214,    0,    0, 
          195,  196,  197,  198,    0,    0,    0,    0,    0,    0, 
          301,    0,    0,    0,    0,    0,    0,  200,  201,    0, 
          202,  203,  204,  205,  206,  207,  208,  209,  210,  211, 
          212,    0,    0,  213, 1066,  606,    0,    0, 1067,    0, 
            0,    0,  184,  185,    0,  186,  187,  188,  189,  190, 
          191,  192,    0,    0,  193,  194,  214,    0,    0,  195, 
          196,  197,  198,    0,    0,    0,    0,    0,    0,  301, 
            0,    0,    0,    0,    0,    0,  200,  201,    0,  202, 
          203,  204,  205,  206,  207,  208,  209,  210,  211,  212, 
            0,    0,  213,    0,    0,    0,    0,    0,    0,    0, 
            0,    0,    0,    0,    0,    0,    0,    0,    0,    0, 
            0,    0,    0,    0,    0,  214, 
      };
   }

   private static final short[] yyCheck1() {
      return new short[] {

           26,   51,   28,  293,   75,    2,    3,    4,    5,   89, 
           21,   91,    9,   10,   11,    7,    7,   14,   15,   16, 
          467,   27,   19,  447,  304,  487,    9,   10,   11,   10, 
            4,    5,   45,    6,  425,   27,   27,  644,   10,    3, 
           14,   49,    0,    0,    6,    7,  326,  327,   45,    2, 
            3,  130,   49,  660,   51,  100,  255,  417,   10,  419, 
           10,   10,  422,  472,  526,   27,   59,  674,  418,   15, 
           16,  484,   10,   19,  487,   49,   15,   16,   59,   52, 
           19,   10,    0,  332,  429,   10,  446,  336,   10,   61, 
           52,   10,   10,  119,   10,  445,  107,   10,  705,   10, 
          460,   44,  462,  100,   11,  507,   10,   59,   47,   48, 
           59,  461,   10,  923,  280,  319,  476,   10,   61,   44, 
          948,   59,   44,   10,   10,  475,  100,   10,   44,   43, 
           59,   10,  129,   10,  131,   59,   61,   59,   10,  847, 
           44,   59,   61,   59,   10,   10,  129,   10,   59,   10, 
          510,   10,  280,   10,   21,   59,   10,   15,   16,  345, 
          957,   19,  305,   10,   11,   44,   59,   44,  306,   10, 
           10,  251,  310,   59,  562,  535,   59,  363,   44,   32, 
           59,   44,   59,  837,  534,   44,   10,   59,   10,   47, 
          342,  305,   10,   59,   59,  342,  362,  277,  105,  269, 
           59,  271,   59,  857,   44,   59,  496,  359,  596,  489, 
          325,  921,  359,  341,  128,  639,  341,  497,  498,   59, 
           44,   10,  426,   10, 1052,  649,  688,   10,   10,   10, 
         1040,   10,  375,  313,  362,  515,   32,   59,  235,   10, 
          107,   59,    2,    3,  359,  341,  113,  373,  536,    9, 
           10,   11,  378,   10,   10,   15,   16,  308,   10,   19, 
           10,  375,   44, 1060,   10,  262,  263,  362,   10,   10, 
          137,  351,   61,   10,  687,  688,   10,   59,   59,  262, 
          263,   10,    0,   10,   10,   45,  996,  310,   59,  328, 
           10,   51,   10,   10,  648,  697,  650,  336, 1006,  268, 
          269,  298,   32,  300,  301,  350,  319,  304,  653,  306, 
          262,  263,  264,  373,  311,   44,  268,  269,  378,  271, 
          317,  304,  319,  306,   44,  376,  377,   44,  350,  351, 
           59,  328, 1042,  262,  263,  264,  279,  625,   32,   59, 
          269,   59,   10,  760,  624,  328,   15,   16,   32,  766, 
           19,  258,  298,  350,  300,  301,  267,   32,  105,  298, 
          279,  300,   91, 1073,  317,  444,  359,  647,  722,  129, 
          342,  131,  366,  423,  282,  425,  350,  290,  291,  429, 
          430,   44,  773,  380,  381,  382,  383,  384,  385,  386, 
          342,  779,  682,   61,  338,  345,   61,  380,  381,  382, 
          383,  305,  385,  386,  813,  363,  363,  388,   61,  281, 
          359,  363,  692,  426,  321,  262,  263,  342,  342,  427, 
          700,  418,  305,  342,  341,  363,  423,  267,  425,  426, 
          427,  384,  429,  430,  105,  432,  388,  359,  342,  305, 
          298,  338,  300,  359,  342,  363,  305,  345,  445,  342, 
          447,  448,  701,  427,  341,  917,   10,  419, 1065,  342, 
          422,  458,    2,    3,  461,  448,  913,  666,  432,  466, 
          894,  375,  418,  359,   14,  235,  342,   10,  475,  432, 
          359,  342,  359,  342,  446,    2,    3,    4,    5,  777, 
          279,  536,  375,  359,  359,  357,  267,   14,  504,  445, 
          462,  342,  262,  263,  917,   45,  786,   61,  505,  375, 
          507,  258,  504,  504,  476,  461,  375,   44,  371,  372, 
          342,  518,  505,  305,  305,  378,   59,  817,   45,  475, 
          338,   45,   49,  380,  381,  382,  383,  534,  298,  536, 
          300,  301,  504,  967,  304,  484,  306,  267,  510,  263, 
          264,  311,  419,  342,  507,  342,  836,  317,  345,  319, 
          342,  342,  536,  342,  471,  472,  345,  425,  328,  595, 
          310,  429,   44,  535,  321,  371,  372,  359,  604,  446, 
          625,  721,  378,  100,  523,  342,  342,   44,  534,  600, 
          342,  131,  342,  375,  375,  462,  342,  608,   91,  328, 
          342,  342,  613,  345,   10,  342,  335,  336,  342,  476, 
          468,  279,  341,   61,  131,  342,  342,  131,  644,  339, 
          380,  381,  382,  383,  384,  385,  386,  271,  625,  298, 
          359,  300,  263,  264,  660,  654,  338,    2,  269,   59, 
          920,  638,  639,  510,  845,  664,  652,  279,  674,  310, 
          321,  625,  649,   59,   91,  638,  339,  702,  418,   10, 
          652,  652,  373,  423,   44,  425,  426,  378,  535,  429, 
          430,   44,  432,  362,  342, 1035,   10,   10,  651,  705, 
          677,  373,   47,   48, 1034,  445,  378,  447,  448,  651, 
          652,  831,  832,   44,  677,  235,  290,  291,  458,  770, 
          697,  461,   10,  902,   48,  702,  466,  757,  758,  908, 
           61,   10,   10,  763,  764,  475,   91,  317,  235,  317, 
           10,  338,  342,  773,  750,  279,   59,   61,  702,  345, 
          637,  264,  777,  600,  363,  310,   10,   44,   44,  373, 
           91,  608,  328,  944,  697,  505,  613,  507,  687,  335, 
          336,  373,  373,   61,  554,  752,   44,  754,  518,  756, 
          757,  758,   61,   61,  750,  373,  763,  764,  347,   59, 
           44,  311,   10,  341,  534,  642,  773,  317,  645,  319, 
          777,  921,  341,  923,   91,    2,    3,   61,  342,  262, 
          264,  638,  342,  264,  311,  653,    0,  311,  795, 1000, 
          317,  798,  319,  777,  267,  319,   10,  379,  380,  381, 
          382,  837,  809,  810,  811,  798,  750,   91,   44,  308, 
          309,   59,  311,   44,  363,  362,   10,   32,   45,  879, 
          677,  857,   10,  350,  306,  328,  308,  309,  310,  311, 
           32,   44,  335,  336,  384,  842,  843, 1048,  648,  306, 
          650,  308,  309,  310,  311,   59,  996,  997,  264,  845, 
          338,  847,  338,  269,   61,  271,  264,  384,   91,  338, 
          867,  338,   10,   44,  781,   10,   44,   61,  638,  639, 
          269,  788,  879,   61,  338,  425,  426,  376,  377,  649, 
          264,  328,  432,  867,  913,  338,  893,  894,  335,  336, 
         1040,  981, 1042,  338,  901,  338,  813,  447,  905,  426, 
          427,  363,  426,  306,  131,  432,  306,  677,  458,   44, 
           44,   59,  722,   44,   59,   44,  466,  901,  279,  280, 
          447,   44,  317, 1073,  267,  306,  301,  697,  282,  283, 
          284,  458,   58,  287,  458,  279,   91,   61,  345,  466, 
          750,  798,  466,  328,  744,  735,  736,   91,  944,  956, 
          335,  336,  948,  264,  349,   44,  338,  507,  353,  354, 
          967,  279,  279,  280,  264,  264,  339,  328,  518,  269, 
          279,  279,  956,  263,  335,  336,  264,  984,  338,  986, 
          507,  342,  752,  264,  754,  992,  756,  757,  758,  363, 
         1026,  518,   44,  763,  764,  279,  339,  941,  342,   91, 
          944,  362,  946,  773,  948,  371,  372,   44,  235,  536, 
         1006,  328,  378,   44,  262,  263,  264,  383,  335,  336, 
          268,  269,   44,  271,  342,  795,   44, 1034,  798, 1065, 
          359,   61, 1039,  342,  342,  845,   59,  847,  264,  809, 
          810,  811,  363,  418,  328,  362,  264,  264,  262,  263, 
          264,  335,  336,  267,  268,  269, 1052,  271,  342,   91, 
          264,  861,   10, 1035,   44,  342,  866,  281,  282,  554, 
          445,  425,  842,  843,   44,  429,  290,  291, 1034,  293, 
          294,  295,  296,  297,  311,  279,  461,  264,  363,  639, 
          317,  279,  319,   44,  342,  328,   44,   44,  625,  649, 
          475,   10,  335,  336, 1048,   44, 1050,  359, 1052,  879, 
          310, 1055,  639,   61,  468,  363,  264,  363,  263,  264, 
          268,  269,  649,  893,  894,  264,   10,   44,  342,  483, 
          484,  345,  507,  487,  944,  905,  926,  310,  948,  939, 
          388,  931,  932,   91,  934, 1089,  936,  697,  342,  363, 
           59,   44,   44,   44,  342,   44,  264,  384, 1035,  534, 
           44,  264,   44,  648,   44,  650,   10,   44,  388,  523, 
          697,   44,  526,  328,  388,  702,  271,   61,  345,  363, 
          335,  336,   91,  710,  328,  341,  561,  562,  371,  372, 
          373,  335,  336,  271,  345,  378, 1006,  967,  363,  426, 
           44,  342,  752,  345,  754,  432,  756,   91,  345,   10, 
           58,   91,   71,  264,  984,  286,  986,   61, 1018,   54, 
          447,  596,  992,  773,  373,  752,   10,  754,  752,  756, 
          754,  458,  756,  608,    5,  312,  328,  722,  613,  466, 
          317, 1037, 1052,  335,  336,  795,   10,   91,  518,    6, 
          777,  613,  900,  845, 1044, 1045, 1046, 1047,   59,  809, 
          810,  811,  616,  750, 1034,  849,  946,  944,  795, 1039, 
          645,  795,   10,  350,  351,   59,  769,   14,  653,  654, 
          507,  656,  809,  810,  811,  809,  810,  811, 1036,  664, 
          831,  518,  842,  843,   -1,   59,  328, 1087,    0,  653, 
           -1, 1060,   -1,  335,  336,   -1,   44,   91,   10,   -1, 
          387,   -1,   10,   -1,   -1,  842,  843,   -1,  842,  843, 
           -1,   -1,  697,   61,  678,   -1,  680,   91,   -1,   -1, 
           10,  279,  280,  687,  688,   -1,   -1,   -1,    0,   -1, 
          867,   -1,   44,  893,  894,  371,  372,  373,   10,   -1, 
           -1,  901,  378,   91,   -1,  905,   58,   59,  267,   -1, 
           -1,   63,   -1,  389,   -1,   -1,  893,  894,   -1,  893, 
           -1,  280,   -1,   -1,  901,   -1,   -1,   -1,  905,   59, 
          328,  905,   44,  371,  372,  373,   -1,  335,  336,   -1, 
          378,  745,   -1,  341,  342,  279,   -1,   59,   -1,  279, 
          280,   -1,   -1,   -1,  779,  306,   -1,  308,  309,  310, 
          311,   91,  639,   -1,  362,   -1,   -1,  967,  793,  328, 
          774,  796,  649,   -1,   -1,   10,  335,  336,    0,  956, 
           -1,   -1,  341,   -1,  984,  279,  986,   10,   10,  340, 
          967,   -1,  992,   -1,  328,   15,   16,  348,  328,   19, 
           -1,  335,  336,  362,  808,  335,  336,  984,  342,  986, 
          984,  341,  986,   -1,   -1,  992,   -1,   -1,  992,   -1, 
          697,   41,   42,   -1,   59,   -1,   -1,   47,   48,   -1, 
           50,   51,  362,  267,  328,   -1,   59,   59,   -1, 1039, 
           -1,  335,  336,   -1,  317,   -1,  280,  341,  342,   -1, 
           -1,  855,   -1,  267,   -1,  306,   91,  308,  309,  310, 
          311,  312, 1039,   -1,    0, 1039,  317,   -1,   91,   -1, 
           -1,  371,  372,  373,   10,  752,  349,  754,  378,  756, 
          353,  354,  355,  356,   -1,  889,   10,   -1,  913,  340, 
           -1,  279,   -1,   -1,  328,   -1,   -1,  348,  349,  350, 
          351,  335,  336,   -1,   -1,   -1,   -1,  341,   -1,   -1, 
          262,  263,  264,  917,  328,  267,  268,  269,  795,  271, 
           44,  335,  336,   59,  371,  372,  373,  341,  362,  281, 
          282,  378,  809,  810,  811,   59,  387,  267,  290,  291, 
          328,  293,  294,  295,  296,  297,   -1,  335,  336,   -1, 
          262,  263,  264,  305,  342,  267,  268,  269,  306,  271, 
          308,  309,  310,  311,  312,  842,  843,   91,   -1,  317, 
           -1,   -1,  324,  325,   -1,   10,   -1,  329,  330,   -1, 
           -1,  293,  294,  295,  296,  297,   44,  339,   -1,   -1, 
          342,   -1,  340,  345,   -1,  347,   -1,   -1,  328,   -1, 
          348,  349,  350,  351,   -1,  335,  336,   -1,   -1, 1034, 
           -1,  363, 1037,  371,  372,  373,  893,  894,   -1,   -1, 
          378,   -1,   -1,  375,   59,   -1,   -1,  339,  905,   -1, 
          342,   -1,  267,   91,   -1,   -1,  388,   -1,   -1,  387, 
          262,  263,  264,   -1,  267,  267,  268,  269,   -1,  271, 
           -1,  363,   -1,  371,  372,  373,   91,  280,    0,  281, 
          378,   -1,  282,  283,  284,  285,   -1,  287,   10,   -1, 
           -1,  293,  294,  295,  296,  297,  388,   -1,  298,   -1, 
          300,  301,  363,   -1,  365,   -1,  367,   -1,   -1,  309, 
          967,   -1,  373,  328,   -1,   -1,  371,  372,  373,   -1, 
          335,  336,   44,  378,   -1,  328,   -1,  984,   -1,  986, 
           -1,   -1,  335,  336,   -1,  992,   -1,   59,  341,   -1, 
          342,   63,  371,  372,  373,   -1,  262,  263,  264,  378, 
           -1,  267,  268,  269,   -1,  271,   -1,   -1,   -1,  362, 
           -1,  363,   -1,   -1,   -1,  281,   -1,   -1,  306,   91, 
          308,  309,  310,  311,  290,  291,  280,  293,  294,  295, 
          296,  297, 1039,   -1,   -1,   -1,  388,  387,  388,  389, 
          390,  391,  392,  393,  394,  395,  396,  397,  398,  399, 
          400,  401,  402,  403,  404,  405,  406,  407,  408,  409, 
          410,  411,  412,  293,  294,  295,  296,  297,  418,   -1, 
          481,   -1,   -1,  423,  328,  425,  342,   -1,   10,  429, 
          430,  335,  336,   -1,   -1,   -1,   -1,  341,   -1,   44, 
           -1,  279,   -1,   10,    0,  445,  306,  363,  308,  309, 
          310,  311,  267,   -1,   10,  359,   -1,   -1,  362,   44, 
           -1,  461,   44,  463,  464,  465,   -1,   -1,  468,   -1, 
           -1,   -1,  388,   -1,   -1,  475,   -1,   -1,  317,   61, 
          340,   -1,   -1,  483,  484,   -1,   91,  487,   44,  550, 
          328,   -1,   59,   -1,  333,  334,  557,  335,  336,   -1, 
           -1,   -1,   58,   59,   -1,   -1,   91,   63,   -1,   91, 
          349,  511,  512,  328,  353,  354,  355,  356,   -1,   -1, 
          335,  336,   -1,  523,   91,   -1,  526,   10,   -1,   -1, 
           -1,   -1,   -1,   -1,  534,  257,  258,  259,   -1,  261, 
          262,  263,  264,  265,  266,  267,  268,  269,  270,  271, 
          272,  273,  274,  275,  276,  277,  278,   -1,  280,  281, 
          282,  283,  284,  285,  286,  287,  288,  289,  290,  291, 
          292,  293,  294,  295,  296,  297,   59,  299,   -1,   -1, 
          302,  303,  304,   -1,  306,  307,  308,  309,  310,  311, 
          312,  313,  314,  315,  316,  317,  318,  319,  320,  321, 
          322,  323,  324,  325,  326,  327,  328,  329,  330,   44, 
           -1,  333,  334,  335,  336,  337,  616,  339,  340,   -1, 
          342,  343,  344,  345,  346,  347,  348,  349,  350,  351, 
          352,  353,  354,  355,  356,  357,  358,  359,  360,  361, 
          362,  363,   -1,  365,  366,  367,  368,  369,  370,   -1, 
           -1,   -1,  374,  653,  376,  377,   91,  379,  380,  381, 
          382,    0,   -1,  385,  386,  387,  388,   -1,   -1,   -1, 
           -1,   10,   -1,   -1,  279,   -1,   -1,   -1,  678,   -1, 
          680,   -1,   -1,   -1,   -1,  858,   -1,  687,  688,  862, 
           -1,   -1,   -1,   -1,  279,  280,   -1,  279,  280,   -1, 
          267,   -1,   -1,   -1,   -1,   44,  262,  263,  264,   -1, 
           -1,  267,  268,  269,   -1,  271,   -1,   -1,   -1,   58, 
           59,   -1,   61,  328,   63,  281,  282,   -1,   -1,   -1, 
          335,  336,   -1,   -1,  290,  291,  341,  293,  294,  295, 
          296,  297,   -1,  328,   -1,  745,  328,   -1,   -1,  305, 
          335,  336,   91,  335,  336,   -1,  341,  757,  758,  341, 
          342,  328,   -1,  763,  764,   -1,   -1,   -1,  335,  336, 
           -1,   -1,  772,  773,  774,   -1,   -1,  362,  778,  317, 
          362,   -1,   -1,  339,   -1,   -1,  342,   -1,   -1,  345, 
           -1,  347,   -1,    0,   -1,  333,  334,   -1,   -1,   -1, 
           -1,  801,  802,   10,  804,  805,   -1,  363,  808,   -1, 
           -1,  349,   -1,  351,   -1,  353,  354,  355,  356,  375, 
           -1,   -1,   -1,  306,   -1,  308,  309,  310,  311,  312, 
           -1,   -1,  388,   -1,  317,   -1,   -1,   44,   -1, 1012, 
           -1,   -1, 1015, 1016,  279,  328, 1019, 1020,   -1,   -1, 
           -1,   58,   59,  336,   61,  855,   63,  340,  341,   -1, 
           -1,   -1,   -1,   -1,   -1,  348,  349,  350,  351,   -1, 
           -1,   -1,  872,   -1,   -1,   -1,   -1,   -1,   -1,  879, 
           -1,   -1,   -1,   -1,   91,   -1,   -1,   -1,   -1,  889, 
           -1,   -1,   -1,  328,   -1,   -1,   -1,   -1,   -1,   -1, 
          335,  336,   -1,   -1,  387,   -1, 1079, 1080, 1081, 1082, 
          910,   -1,   -1,   10,   -1,   -1,   -1,  917,  257,  258, 
          259, 1094,  261,  262,  263,  264,  265,  266,  267,  268, 
          269,  270,  271,  272,  273,  274,  275,  276,  277,  278, 
           -1,  280,  281,  282,  283,  284,  285,  286,  287,  288, 
          289,  290,  291,  292,  293,  294,  295,  296,  297,   -1, 
          299,   -1,   59,  302,  303,  304,  305,  306,  307,  308, 
          309,  310,  311,  312,  313,  314,  315,  316,  317,  318, 
          319,  320,  321,  322,  323,  324,  325,  326,  327,  328, 
          329,  330,   -1,   -1,  333,  334,  335,  336,  337,  338, 
          339,  340,  341,  342,  343,  344,  345,  346,  347,  348, 
          349,  350,  351,  352,  353,  354,  355,  356,  357,  358, 
          359,  360,  361,  362,  363,  750,  365,  366,  367,  368, 
          369,  370,   -1,   -1, 1034,  374,  375,  376,  377,   -1, 
          379,  380,  381,  382,   10,   10,  385,  386,  387,  388, 
          257,  258,  259,   -1,  261,  262,  263,  264,  265,  266, 
          267,  268,  269,  270,  271,  272,  273,  274,  275,  276, 
          277,  278,   -1,  280,  281,  282,  283,  284,  285,  286, 
          287,  288,  289,  290,  291,  292,  293,  294,  295,  296, 
          297,   -1,  299,   59,   59,  302,  303,  304,  305,  306, 
          307,  308,  309,  310,  311,  312,  313,  314,  315,  316, 
          317,  318,  319,  320,  321,  322,  323,  324,  325,  326, 
          327,  328,  329,  330,  849,   91,  333,  334,  335,  336, 
          337,  338,  339,  340,  341,  342,  343,  344,  345,  346, 
          347,  348,  349,  350,  351,  352,  353,  354,  355,  356, 
          357,  358,  359,  360,  361,  362,  363,   -1,  365,  366, 
          367,  368,  369,  370,   -1,   -1,    0,  374,  375,  376, 
          377,   -1,  379,  380,  381,  382,   10,   -1,  385,  386, 
          387,  388,   -1,  262,  263,  264,   -1,   -1,   -1,  268, 
          269,   -1,  271,  262,  263,  264,   -1,   -1,   -1,  268, 
          269,   -1,  271,  306,   -1,  308,  309,  310,  311,  306, 
           44,  308,  309,  310,  311,  312,  941,   -1,   -1,  944, 
          317,  946,   -1,  948,   58,   59,   -1,   61,   -1,   63, 
          306,  328,  308,  309,  310,  311,  312,  340,   -1,  336, 
           -1,  317,   -1,  340,  341,  348,   -1,   -1,   -1,  317, 
           -1,  348,  349,  350,  351,   -1,   -1,   91,   -1,   -1, 
           -1,   -1,   -1,  342,  340,  333,  334,   -1,   -1,   -1, 
           -1,   -1,   -1,  342,  350,  351,   -1,   -1,   -1,   -1, 
           -1,  349,   -1,  351,  363,  353,  354,  355,  356,   -1, 
          387,  359,   -1,  361,  363,   -1,   -1,   -1,    0,   -1, 
           -1,  267,   -1,   -1,   -1,   -1,   -1,   -1,   10,  388, 
           -1,  387,   -1,   -1,  280,   -1,   -1,   -1,   -1,  388, 
           -1,   -1,   -1, 1048,   -1, 1050,   -1, 1052,   -1,   -1, 
         1055,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,  306,   44,  308,  309,  310,  311,  312,   -1,   -1, 
           -1,   -1,  317,   -1,   -1,   -1,   58,   59,   -1,   61, 
           -1,   63,  328,  328, 1089,   -1,   -1,   -1,   -1,  335, 
          336,  336,   -1,   -1,   -1,  340,  341,   -1,   -1,   -1, 
           -1,   -1,   -1,  348,  349,  350,  351,   -1,   -1,   91, 
           -1,   -1,   -1,   -1,   -1,   -1,  362,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   10,   -1, 
           -1,   -1,  387,  257,  258,  259,   -1,  261,  262,  263, 
          264,  265,  266,  267,  268,  269,  270,  271,  272,  273, 
          274,  275,  276,  277,  278,   -1,  280,  281,  282,  283, 
          284,  285,  286,  287,  288,  289,  290,  291,  292,  293, 
          294,  295,  296,  297,   -1,  299,   -1,   59,  302,  303, 
          304,  305,  306,  307,  308,  309,  310,  311,  312,  313, 
          314,  315,  316,  317,  318,  319,  320,  321,  322,  323, 
          324,  325,  326,  327,  328,  329,  330,   -1,   -1,  333, 
          334,  335,  336,  337,   -1,  339,  340,  341,  342,  343, 
          344,  345,  346,  347,  348,  349,  350,  351,  352,  353, 
          354,  355,  356,  357,  358,  359,  360,  361,  362,  363, 
           63,  365,  366,  367,  368,  369,  370,   -1,   -1,   -1, 
          374,  375,  376,  377,   -1,  379,  380,  381,  382,   10, 
           10,  385,  386,  387,  388,  257,  258,  259,   -1,  261, 
          262,  263,  264,  265,  266,  267,  268,  269,  270,  271, 
          272,  273,  274,  275,  276,  277,  278,   -1,  280,  281, 
          282,  283,  284,  285,  286,  287,  288,  289,  290,  291, 
          292,  293,  294,  295,  296,  297,   -1,  299,   59,   59, 
          302,  303,  304,  305,  306,  307,  308,  309,  310,  311, 
          312,  313,  314,  315,  316,  317,  318,  319,  320,  321, 
          322,  323,  324,  325,  326,  327,  328,  329,  330,   44, 
           91,  333,  334,  335,  336,  337,   -1,  339,  340,  341, 
          342,  343,  344,  345,  346,  347,  348,  349,  350,  351, 
          352,  353,  354,  355,  356,  357,  358,  359,  360,  361, 
          362,  363,   -1,  365,  366,  367,  368,  369,  370,   -1, 
           -1,    0,  374,  375,  376,  377,   91,  379,  380,  381, 
          382,   10,   -1,  385,  386,  387,  388,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,  306,   -1,  308, 
          309,  310,  311,  312,   -1,   -1,   -1,   -1,  317,   -1, 
           -1,   -1,   -1,   -1,  306,   44,  308,  309,  310,  311, 
          312,   -1,   -1,   -1,   -1,  317,   -1,   -1,   -1,   58, 
           59,  340,   61,   -1,   63,   -1,  328,   -1,   -1,  348, 
          349,  350,  351,   -1,  336,   -1,   -1,   -1,  340,  341, 
           -1,   -1,   -1,   -1,   -1,   -1,  348,  349,  350,  351, 
           -1,   -1,   91,   -1,  297,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  387,   -1, 
           -1,   -1,   -1,   -1,  317,  318,  319,  320,  321,  322, 
          323,  324,  325,  326,  327,  387,  329,  330,   -1,   -1, 
          333,  334,   -1,    0,   -1,   -1,  267,   -1,   -1,   -1, 
           -1,   -1,   -1,   10,   -1,   -1,  349,   -1,  351,   -1, 
          353,  354,  355,  356,  357,  358,  359,   -1,  361,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,  306,   44,  308,  309, 
          310,  311,  312,   -1,   -1,   -1,   -1,  317,   -1,   -1, 
           -1,   58,   59,   -1,  279,   -1,   63,  328,  328,   -1, 
           -1,   -1,   -1,   -1,  335,  336,  336,   -1,   -1,   -1, 
          340,  341,   -1,   -1,   -1,   -1,   -1,   -1,  348,  349, 
          350,  351,   -1,   -1,   91,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,  328,   -1,   -1,   -1,   -1,   -1,   -1, 
          335,  336,   -1,   10,   -1,   -1,   -1,  387,  257,  258, 
          259,   -1,  261,  262,  263,  264,  265,  266,  267,  268, 
          269,  270,  271,  272,  273,  274,  275,  276,  277,  278, 
           -1,  280,  281,  282,  283,  284,  285,  286,  287,  288, 
          289,  290,  291,  292,  293,  294,  295,  296,  297,   -1, 
          299,   -1,   59,  302,  303,  304,  305,  306,  307,  308, 
          309,  310,  311,  312,  313,  314,  315,  316,  317,  318, 
          319,  320,  321,  322,  323,  324,  325,  326,  327,  328, 
          329,  330,   44,   -1,  333,  334,  335,  336,  337,   -1, 
          339,  340,  341,  342,  343,  344,  345,  346,  347,  348, 
          349,  350,  351,  352,  353,  354,  355,  356,  357,  358, 
          359,  360,  361,  362,  363,   44,  365,  366,  367,  368, 
          369,  370,   -1,   -1,   -1,  374,  375,  376,  377,   91, 
          379,  380,  381,  382,   10,   10,  385,  386,  387,  388, 
          257,  258,  259,   -1,  261,  262,  263,  264,  265,  266, 
          267,  268,  269,  270,  271,  272,  273,  274,  275,  276, 
          277,  278,   91,  280,  281,  282,  283,  284,  285,  286, 
          287,  288,  289,  290,  291,  292,  293,  294,  295,  296, 
          297,   -1,  299,   59,   59,  302,  303,  304,  305,  306, 
          307,  308,  309,  310,  311,  312,  313,  314,  315,  316, 
          317,  318,  319,  320,  321,  322,  323,  324,  325,  326, 
          327,  328,  329,  330,   44,   91,  333,  334,  335,  336, 
          337,   -1,  339,  340,  341,  342,  343,  344,  345,  346, 
          347,  348,  349,  350,  351,  352,  353,  354,  355,  356, 
          357,  358,  359,  360,  361,  362,  363,   -1,  365,  366, 
          367,  368,  369,  370,   -1,   -1,    0,  374,  375,  376, 
          377,   91,  379,  380,  381,  382,   10,   -1,  385,  386, 
          387,  388,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  306, 
           44,  308,  309,  310,  311,  312,   -1,   -1,   -1,   -1, 
          317,   -1,   -1,   -1,   58,   59,   -1,  279,  280,   63, 
          306,  328,  308,  309,  310,  311,  312,   -1,   -1,  336, 
           -1,  317,   -1,  340,  341,   -1,   -1,   -1,   -1,   -1, 
           -1,  348,  349,  350,  351,   -1,   -1,   91,   -1,   -1, 
          279,   -1,   -1,   -1,  340,  341,   -1,   -1,   -1,   -1, 
           -1,   -1,  348,  349,  350,  351,  328,   -1,   -1,   -1, 
           -1,   -1,   -1,  335,  336,   -1,  317,   -1,   -1,  341, 
          387,  322,  323,   -1,   -1,   -1,   -1,   -1,    0,   -1, 
           -1,  267,  333,  334,   -1,   -1,   -1,   -1,   10,  328, 
          362,  387,   -1,   -1,   -1,   -1,  335,  336,  349,   -1, 
          351,   -1,  353,  354,  355,  356,  357,  358,  359,   -1, 
          361,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,  306,   44,  308,  309,  310,  311,  312,   -1,   -1, 
           -1,   -1,  317,   -1,   -1,   -1,   58,   59,   -1,  279, 
           -1,   63,  328,  328,   -1,   -1,   -1,   -1,   -1,  335, 
          336,  336,   -1,   -1,   -1,  340,  341,   -1,   -1,   -1, 
           -1,   -1,   -1,  348,  349,  350,  351,   -1,   -1,   91, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  328,   -1, 
           -1,   -1,   -1,   -1,   -1,  335,  336,   -1,   10,   -1, 
           -1,   -1,  387,  257,  258,  259,   -1,  261,  262,  263, 
          264,  265,  266,  267,  268,  269,  270,  271,  272,  273, 
          274,  275,  276,  277,  278,   -1,  280,  281,  282,  283, 
          284,  285,  286,  287,  288,  289,  290,  291,  292,  293, 
          294,  295,  296,  297,   -1,  299,   -1,   59,  302,  303, 
          304,  305,  306,  307,  308,  309,  310,  311,  312,  313, 
          314,  315,  316,  317,  318,  319,  320,  321,  322,  323, 
          324,  325,  326,  327,  328,  329,  330,   -1,   -1,  333, 
          334,  335,  336,  337,   -1,  339,  340,  341,  342,  343, 
          344,  345,  346,  347,  348,  349,  350,  351,  352,  353, 
          354,  355,  356,  357,  358,  359,  360,  361,  362,  363, 
           63,  365,  366,  367,  368,  369,  370,   -1,   -1,   -1, 
          374,  375,  376,  377,   -1,  379,  380,  381,  382,   10, 
           10,  385,  386,  387,  388,  257,  258,  259,   -1,  261, 
          262,  263,  264,  265,  266,  267,  268,  269,  270,  271, 
          272,  273,  274,  275,  276,  277,  278,   -1,  280,  281, 
          282,  283,  284,  285,  286,  287,  288,  289,  290,  291, 
          292,  293,  294,  295,  296,  297,   -1,  299,   59,   59, 
          302,  303,  304,  305,  306,  307,  308,  309,  310,  311, 
          312,  313,  314,  315,  316,  317,  318,  319,  320,  321, 
          322,  323,  324,  325,  326,  327,  328,  329,  330,   44, 
           91,  333,  334,  335,  336,  337,   -1,  339,  340,  341, 
          342,  343,  344,  345,  346,  347,  348,  349,  350,  351, 
          352,  353,  354,  355,  356,  357,  358,  359,  360,  361, 
          362,  363,   -1,  365,  366,  367,  368,  369,  370,   -1, 
           -1,    0,  374,  375,  376,  377,   91,  379,  380,  381, 
          382,   10,   -1,  385,  386,  387,  388,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,  306,   44,  308,  309,  310,  311, 
          312,   -1,   -1,   -1,   -1,  317,   -1,   -1,   -1,   58, 
           59,   -1,   61,   -1,   63,   -1,  328,  317,  318,  319, 
          320,  321,  322,  323,  336,   -1,  326,  327,  340,  341, 
           -1,   -1,   -1,  333,  334,   -1,  348,  349,  350,  351, 
           -1,   -1,   91,   -1,  297,   -1,   -1,   -1,   -1,  349, 
           -1,  351,   -1,  353,  354,  355,  356,  357,  358,  359, 
           -1,  361,   -1,   -1,  317,  318,  319,  320,  321,  322, 
          323,  324,  325,  326,  327,  387,  329,  330,   -1,   -1, 
          333,  334,   -1,    0,   -1,   -1,  267,   -1,   -1,   -1, 
           -1,   -1,   -1,   10,   -1,   -1,  349,   -1,  351,   -1, 
          353,  354,  355,  356,  357,  358,  359,   -1,  361,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,  306,   44,  308,  309, 
          310,  311,  312,   -1,   -1,   -1,   -1,  317,   -1,   -1, 
           -1,   -1,   59,   -1,  279,   -1,   63,  328,  328,   -1, 
           -1,   -1,   -1,   -1,  335,  336,  336,   -1,   -1,   -1, 
          340,  341,   -1,   -1,   -1,   -1,   -1,   -1,  348,  349, 
          350,  351,   -1,   -1,   91,   -1,   -1,   10,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,  328,   -1,   -1,   -1,   -1,   -1,   -1, 
          335,  336,   -1,   -1,   -1,   -1,   -1,  387,  257,  258, 
          259,   -1,  261,  262,  263,  264,  265,  266,  267,  268, 
          269,  270,  271,  272,  273,  274,  275,  276,  277,  278, 
           63,   -1,  281,  282,  283,  284,  285,  286,  287,  288, 
          289,  290,  291,  292,  293,  294,  295,  296,  297,   -1, 
          299,   -1,   -1,  302,  303,  304,  305,  306,  307,  308, 
          309,  310,  311,  312,  313,  314,  315,  316,  317,  318, 
          319,  320,  321,  322,  323,  324,  325,  326,  327,  328, 
          329,  330,   44,   -1,  333,  334,  335,  336,  337,   -1, 
          339,  340,  341,  342,  343,  344,  345,  346,  347,  348, 
          349,  350,  351,  352,  353,  354,  355,  356,  357,  358, 
          359,  360,  361,   -1,  363,   -1,  365,  366,  367,  368, 
          369,  370,   63,   -1,   -1,  374,  375,  376,  377,   91, 
          379,  380,  381,  382,   10,   -1,  385,  386,  387,  388, 
          257,  258,  259,   -1,  261,  262,  263,  264,  265,  266, 
          267,  268,  269,  270,  271,  272,  273,  274,  275,  276, 
          277,  278,   -1,  280,  281,  282,  283,  284,  285,  286, 
          287,  288,  289,  290,  291,  292,  293,  294,  295,  296, 
          297,   -1,  299,   59,   -1,  302,  303,  304,   -1,  306, 
          307,  308,  309,  310,  311,  312,  313,  314,  315,  316, 
          317,  318,  319,  320,  321,  322,  323,  324,  325,  326, 
          327,  328,  329,  330,    0,   91,  333,  334,  335,  336, 
          337,   -1,  339,  340,   10,  342,  343,  344,  345,  346, 
          347,  348,  349,  350,  351,  352,  353,  354,  355,  356, 
          357,  358,  359,  360,  361,  362,  363,   -1,  365,  366, 
          367,  368,  369,  370,   -1,   -1,   -1,  374,   44,  376, 
          377,   -1,  379,  380,  381,  382,   -1,   -1,  385,  386, 
          387,  388,   -1,   59,   -1,   -1,   -1,   63,   -1,   -1, 
           -1,   -1,   -1,   -1,  317,  318,  319,  320,  321,  322, 
          323,  324,  325,  326,  327,   -1,  329,  330,   -1,   -1, 
          333,  334,   -1,   -1,   -1,   91,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,  349,  279,  351,   -1, 
          353,  354,  355,  356,  357,  358,  359,   -1,  361,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,    0,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   10,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,  328,   -1,   -1,   -1, 
           -1,   -1,   -1,  335,  336,   -1,  317,  318,  319,  320, 
          321,  322,  323,  324,  325,  326,  327,   -1,  329,  330, 
           44,  267,  333,  334,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   59,   -1,   61,  349,   63, 
          351,   -1,  353,  354,  355,  356,  357,  358,  359,   -1, 
          361,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   91,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,  328,   -1,   -1,   -1,   -1,   -1,   -1,  335, 
          336,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,  257,  258,  259,   -1,  261,  262,  263,  264,  265, 
          266,  267,  268,  269,  270,  271,  272,  273,  274,  275, 
          276,  277,  278,   -1,   -1,  281,  282,  283,  284,  285, 
          286,  287,  288,  289,  290,  291,  292,  293,  294,  295, 
          296,  297,   -1,  299,   -1,   -1,  302,  303,  304,   -1, 
          306,  307,  308,  309,  310,  311,  312,  313,  314,  315, 
          316,  317,  318,  319,  320,  321,  322,  323,  324,  325, 
          326,  327,  328,  329,  330,  750,   -1,  333,  334,  335, 
          336,  337,   -1,  339,  340,   -1,  342,  343,  344,  345, 
          346,  347,  348,  349,  350,  351,  352,  353,  354,  355, 
          356,  357,  358,  359,  360,  361,   -1,  363,   44,  365, 
          366,  367,  368,  369,  370,   -1,   -1,   -1,  374,   -1, 
          376,  377,   -1,  379,  380,  381,  382,   10,   -1,  385, 
          386,  387,  388,  257,  258,  259,   -1,  261,  262,  263, 
          264,  265,  266,   -1,  268,  269,  270,  271,  272,  273, 
          274,  275,  276,  277,  278,   91,  280,  750,   -1,  283, 
          284,  285,  286,  287,  288,  289,  290,  291,  292,  293, 
          294,  295,  296,  297,  849,  299,   59,  852,  302,  303, 
          304,   -1,  306,  307,  308,  309,  310,  311,  312,  313, 
          314,  315,  316,  317,  318,  319,  320,  321,  322,  323, 
          324,  325,  326,  327,  328,  329,  330,   -1,   91,  333, 
          334,  335,  336,  337,  338,   -1,  340,  341,  342,  343, 
          344,   -1,  346,   -1,  348,  349,  350,  351,  352,  353, 
          354,  355,  356,  357,  358,  359,  360,  361,  362,  363, 
           -1,  365,  366,  367,  368,  369,  370,  840,   -1,    0, 
          374,   -1,  376,  377,  847,  379,  380,  381,  382,   10, 
           -1,  385,  386,  387,  388,   -1,  941,   -1,   -1,  944, 
           -1,  946,   -1,  948,   -1,  306,  307,   -1,   -1,  310, 
           -1,   -1,   -1,  314,  315,   -1,  317,  318,  319,  320, 
          321,  322,  323,   44,   -1,  326,  327,   -1,   -1,   -1, 
          331,  332,  333,  334,   -1,   -1,   -1,   -1,   59,   -1, 
           61,   -1,   63,   -1,   -1,   -1,   -1,  348,  349,   -1, 
          351,  352,  353,  354,  355,  356,  357,  358,  359,  360, 
          361,   -1,   -1,  364,   -1,   -1,   -1,   -1,   -1,   -1, 
           91,   -1,   -1,  279,  280,   -1,   -1,   -1,  941,   -1, 
           -1,   -1,   -1,   -1,   -1,  948,  387,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1, 1048,   -1, 1050,   -1, 1052,   -1,   -1, 
         1055,    0,   -1,   -1,  267,   -1,   -1,   -1,   -1,   -1, 
           -1,   10,  328,   -1,   -1,   -1,   -1,  280,   -1,  335, 
          336,   -1,   -1,   -1,   -1,  341,   -1,   -1,   -1,   -1, 
           -1, 1004,   -1, 1006, 1089, 1008,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   44,  362,   -1,   44,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           59,   -1,   61,   -1,   63,  328,   -1,   -1,   -1,   -1, 
           -1,   -1,  335,  336,   -1,   -1,   -1, 1050,  341, 1052, 
           -1,   -1, 1055,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   91,   -1,   -1,   91,   -1,   -1,   -1,  362, 
           -1,   -1, 1075,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1, 1089,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,  257,  258,  259,   -1, 
          261,  262,  263,  264,  265,  266,   -1,  268,  269,  270, 
          271,  272,  273,  274,  275,  276,  277,  278,   -1,  280, 
           -1,   -1,  283,  284,  285,  286,  287,  288,  289,  290, 
          291,  292,  293,  294,  295,  296,  297,   44,  299,   44, 
           -1,  302,  303,  304,   -1,  306,  307,  308,  309,  310, 
          311,  312,  313,  314,  315,  316,  317,  318,  319,  320, 
          321,  322,  323,  324,  325,  326,  327,  328,  329,  330, 
           -1,   -1,  333,  334,  335,  336,  337,  338,   -1,  340, 
          341,  342,  343,  344,   91,  346,   91,  348,  349,  350, 
          351,  352,  353,  354,  355,  356,  357,  358,  359,  360, 
          361,  362,  363,   44,  365,  366,  367,  368,  369,  370, 
           -1,   -1,   -1,  374,   -1,  376,  377,   -1,  379,  380, 
          381,  382,   10,   -1,  385,  386,  387,  388,  257,  258, 
          259,   -1,  261,  262,  263,  264,  265,  266,   -1,  268, 
          269,  270,  271,  272,  273,  274,  275,  276,  277,  278, 
           91,  280,   -1,  279,  283,  284,  285,  286,  287,  288, 
          289,  290,  291,  292,  293,  294,  295,  296,  297,   -1, 
          299,   59,   -1,  302,  303,  304,   -1,  306,  307,  308, 
          309,  310,  311,  312,  313,  314,  315,  316,  317,  318, 
          319,  320,  321,  322,  323,  324,  325,  326,  327,  328, 
          329,  330,  328,   91,  333,  334,  335,  336,  337,  335, 
          336,  340,  341,  342,  343,  344,   -1,  346,   -1,  348, 
          349,  350,  351,  352,  353,  354,  355,  356,  357,  358, 
          359,  360,  361,  362,  363,   -1,  365,  366,  367,  368, 
          369,  370,   -1,   -1,    0,  374,   -1,  376,  377,   -1, 
          379,  380,  381,  382,   10,   -1,  385,  386,  387,  388, 
          317,  318,  319,  320,  321,  322,  323,  324,  325,  326, 
          327,   -1,  279,  280,  279,   -1,  333,  334,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   44,   -1, 
           -1,   -1,  349,   -1,  351,   -1,  353,  354,  355,  356, 
          357,  358,  359,   59,  361,   61,   -1,   63,  317,  318, 
          319,  320,  321,  322,  323,  324,   -1,  326,  327,   -1, 
           -1,  328,   -1,  328,  333,  334,   -1,   -1,  335,  336, 
          335,  336,   -1,   -1,  341,   91,   -1,   -1,  279,   -1, 
          349,   -1,  351,   -1,  353,  354,  355,  356,  357,  358, 
          359,   -1,  361,   -1,   -1,  362,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,    0,   -1,   -1,  267, 
           -1,   -1,   -1,   -1,   -1,   -1,   10,  328,   -1,   -1, 
           -1,   -1,  280,   -1,  335,  336,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           44,   -1,   -1,   44,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   59,   -1,   61,   -1,   63, 
          328,   -1,   -1,   -1,   -1,   -1,   -1,  335,  336,   -1, 
           -1,   -1,   -1,  341,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   91,   -1,   -1, 
           91,   -1,   -1,   -1,  362,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,  257,  258,  259,   -1,  261,  262,  263,  264,  265, 
          266,   -1,  268,  269,  270,  271,  272,  273,  274,  275, 
          276,  277,  278,   -1,  280,   -1,   -1,  283,  284,  285, 
          286,  287,  288,  289,  290,  291,  292,  293,  294,  295, 
          296,  297,   44,  299,   -1,   -1,  302,  303,  304,   -1, 
          306,  307,  308,  309,  310,  311,  312,  313,  314,  315, 
          316,  317,  318,  319,  320,  321,  322,  323,  324,  325, 
          326,  327,  328,  329,  330,   -1,   -1,  333,  334,  335, 
          336,  337,   -1,   -1,  340,  341,  342,  343,  344,   91, 
          346,   -1,  348,  349,  350,  351,  352,  353,  354,  355, 
          356,  357,  358,  359,  360,  361, 
      };
   }

   private static final short[] yyCheck2() {
      return new short[] {

          362,  363,   -1,  365,  366,  367,  368,  369,  370,   -1, 
           -1,    0,  374,   -1,  376,  377,   -1,  379,  380,  381, 
          382,   10,   -1,  385,  386,  387,  388,  257,  258,  259, 
           -1,  261,  262,  263,  264,  265,  266,   -1,  268,  269, 
          270,  271,  272,  273,  274,  275,  276,  277,  278,   -1, 
          280,   -1,  279,  283,  284,  285,  286,  287,  288,  289, 
          290,  291,  292,  293,  294,  295,  296,  297,   -1,  299, 
           59,   -1,  302,  303,  304,   -1,  306,  307,  308,  309, 
          310,  311,  312,  313,  314,  315,  316,  317,  318,  319, 
          320,  321,  322,  323,  324,  325,  326,  327,  328,  329, 
          330,  328,   -1,  333,  334,  335,  336,  337,  335,  336, 
          340,  341,  342,  343,  344,   -1,  346,   -1,  348,  349, 
          350,  351,  352,  353,  354,  355,  356,  357,  358,  359, 
          360,  361,  362,  363,   -1,  365,  366,  367,  368,  369, 
          370,   -1,   -1,    0,  374,   -1,  376,  377,   -1,  379, 
          380,  381,  382,   10,   -1,  385,  386,  387,  388,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,  279,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   44,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   59,   -1,   61,   -1,   63,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
          328,   -1,   -1,   -1,   -1,   -1,   -1,  335,  336,   -1, 
           -1,   -1,   -1,   -1,   91,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  257,  258, 
          259,   -1,  261,  262,  263,  264,  265,  266,  267,  268, 
          269,  270,  271,  272,  273,  274,  275,  276,  277,  278, 
           -1,  280,  281,   -1,  283,  284,  285,  286,  287,  288, 
          289,  290,  291,  292,  293,  294,  295,  296,  297,   -1, 
          299,   -1,   -1,  302,  303,  304,   -1,  306,  307,  308, 
          309,  310,  311,  312,  313,  314,  315,  316,   -1,   -1, 
            0,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  328, 
           10,   -1,   -1,   -1,   -1,   -1,  335,  336,  337,   -1, 
           -1,  340,   -1,  342,  343,  344,   -1,  346,   -1,  348, 
           -1,  350,   -1,  352,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,  360,   -1,  362,  363,   -1,  365,  366,  367,  368, 
          369,  370,   -1,   -1,   -1,  374,   -1,  376,  377,   59, 
          379,  380,  381,  382,   -1,   -1,  385,  386,  387,  388, 
          257,  258,  259,   -1,  261,  262,  263,  264,  265,  266, 
           -1,  268,  269,  270,  271,  272,  273,  274,  275,  276, 
          277,  278,   -1,   -1,   -1,   -1,  283,  284,  285,  286, 
          287,  288,  289,  290,  291,  292,  293,  294,  295,  296, 
          297,   -1,  299,   -1,   -1,  302,  303,  304,   -1,  306, 
          307,  308,  309,  310,  311,  312,  313,  314,  315,  316, 
          317,  318,  319,  320,  321,  322,  323,  324,  325,  326, 
          327,  328,  329,  330,   -1,   -1,  333,  334,  335,  336, 
          337,   -1,   -1,  340,  341,  342,  343,  344,   10,  346, 
           10,  348,  349,  350,  351,  352,  353,  354,  355,  356, 
          357,  358,  359,  360,  361,    0,  363,   -1,  365,  366, 
          367,  368,  369,  370,   -1,   10,   -1,  374,   -1,  376, 
          377,   -1,  379,  380,  381,  382,   -1,   -1,  385,  386, 
          387,  388,   -1,   -1,   -1,   -1,   -1,   59,   -1,   59, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   44, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   58,   59,   -1,   61,   -1,   63,   91, 
           -1,   91,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,  262,  263,  264,   -1,   91,  267,  268,  269, 
           -1,  271,   -1,   -1,   -1,   -1,   -1,    0,   -1,   -1, 
           -1,  281,  282,   -1,   -1,   -1,   -1,   10,   -1,   -1, 
          290,  291,   -1,  293,  294,  295,  296,  297,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   44,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   10,   -1,   -1,   -1,   58,   59,   -1,   -1,   -1, 
           63,   -1,  342,   -1,   -1,  345,   -1,  347,   -1,   -1, 
           -1,   10,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,  363,   -1,   -1,   -1,   -1,   91,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           59,   -1,   -1,   -1,   -1,   44,   -1,   -1,  388,   -1, 
           -1,    0,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   10,   61,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   91,   -1,   -1,  267,   -1,  267,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   91,   -1,   -1,   44,   -1,  262,  263,  264, 
           -1,   -1,  267,  268,  269,   -1,  271,   -1,   -1,   58, 
           59,   -1,   61,   -1,   63,  280,  281,  282,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,  290,  291,   -1,  293,  294, 
          295,  296,  297,   -1,   -1,   -1,  328,   -1,  328,   -1, 
          305,   -1,   91,  335,  336,  335,  336,   -1,   -1,   -1, 
           -1,   -1,  317,  318,  319,  320,  321,  322,  323,  324, 
          325,  326,  327,  328,  329,  330,   10,   -1,  333,  334, 
          335,  336,   -1,   -1,  339,   -1,   -1,  342,   -1,   -1, 
          345,   -1,  347,   -1,  349,   -1,  351,   -1,  353,  354, 
          355,  356,  357,  358,  359,   -1,  361,  362,  363,  262, 
          263,  264,   -1,   -1,  267,  268,  269,   -1,  271,   -1, 
          375,   -1,   -1,   -1,   -1,   59,   -1,  280,  281,  282, 
           -1,   -1,   -1,  388,   -1,   -1,   -1,  290,  291,   -1, 
          293,  294,  295,  296,  297,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,  305,    0,   -1,   -1,   -1,   91,  267,   -1, 
           -1,   -1,   -1,   10,  317,  318,  319,  320,  321,  322, 
          323,  324,  325,  326,  327,  328,  329,  330,   -1,   -1, 
          333,  334,  335,  336,   -1,   -1,  339,   -1,   -1,  342, 
          279,   -1,  345,   -1,  347,   -1,  349,   44,  351,   -1, 
          353,  354,  355,  356,  357,  358,  359,   -1,  361,  362, 
          363,   58,   59,   -1,   61,   -1,   63,   -1,   -1,  328, 
           -1,   -1,  375,  262,  263,  264,  335,  336,  267,  268, 
          269,   -1,  271,   -1,   -1,  388,   -1,   -1,   -1,  328, 
           -1,   -1,  281,  282,   91,   -1,  335,  336,   -1,   -1, 
           -1,  290,  291,  342,  293,  294,  295,  296,  297,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,  305,    0,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   10,  317,  318, 
          319,  320,  321,  322,  323,  324,  325,  326,  327,  328, 
          329,  330,   -1,   -1,  333,  334,  335,  336,   -1,  338, 
          339,   -1,   -1,  342,   -1,   -1,  345,   -1,  347,   10, 
          349,   44,  351,   -1,  353,  354,  355,  356,  357,  358, 
          359,   -1,  361,   -1,  363,   58,   59,   -1,   61,   -1, 
           63,   -1,   -1,  267,   -1,   -1,  375,   -1,   -1,   -1, 
           -1,   -1,   -1,   44,   -1,   -1,   -1,   -1,   -1,  388, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   91,   -1, 
           61,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,    0, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   10, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           91,   -1,   -1,   -1,  328,   -1,   -1,   -1,   -1,   -1, 
           -1,  335,  336,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   44,   -1,  262,  263,  264,   -1,   -1, 
          267,  268,  269,   -1,  271,   -1,   -1,   58,   59,   -1, 
           61,   -1,   63,   -1,  281,  282,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,  290,  291,   -1,  293,  294,  295,  296, 
          297,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  305,   -1, 
           91,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
          317,  318,  319,  320,  321,  322,  323,  324,  325,  326, 
          327,  328,  329,  330,   -1,   -1,  333,  334,  335,  336, 
           -1,  338,  339,   -1,   -1,  342,   -1,   -1,  345,   -1, 
          347,   -1,  349,   -1,  351,   -1,  353,  354,  355,  356, 
          357,  358,  359,   -1,  361,   -1,  363,   -1,   -1,   -1, 
           -1,   -1,    0,   -1,   -1,   -1,   -1,   -1,  375,  262, 
          263,  264,   10,   -1,  267,  268,  269,   -1,  271,   -1, 
           -1,  388,   -1,   -1,   -1,   -1,   -1,   -1,  281,  282, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,  290,  291,   -1, 
          293,  294,  295,  296,  297,   -1,   44,   -1,   -1,   -1, 
           -1,   -1,  305,   -1,   -1,   -1,   -1,   -1,  279,   -1, 
           58,   59,   -1,   -1,  317,  318,  319,  320,  321,  322, 
          323,  324,  325,  326,  327,  328,  329,  330,   -1,   -1, 
          333,  334,  335,  336,   -1,  338,  339,   -1,   -1,  342, 
           -1,   -1,  345,   -1,  347,   -1,  349,   -1,  351,   -1, 
          353,  354,  355,  356,  357,  358,  359,  328,  361,   -1, 
          363,  262,  263,  264,  335,  336,  267,  268,  269,   -1, 
          271,  342,  375,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
          281,  282,   -1,   -1,   -1,  388,   -1,   -1,   -1,  290, 
          291,   -1,  293,  294,  295,  296,  297,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,  305,    0,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   10,  317,  318,  319,  320, 
          321,  322,  323,  324,  325,  326,  327,  328,  329,  330, 
           -1,   -1,  333,  334,  335,  336,   -1,  338,  339,   -1, 
           -1,  342,   -1,   -1,  345,   -1,  347,   10,  349,   44, 
          351,   -1,  353,  354,  355,  356,  357,  358,  359,   -1, 
          361,   -1,  363,   58,   59,   -1,   61,   -1,   63,   -1, 
           -1,   -1,   -1,   -1,  375,   -1,   -1,   -1,   -1,   -1, 
           -1,   44,   -1,   -1,   -1,   -1,   -1,  388,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   91,   -1,   61,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,    0,   -1,   -1, 
           -1,   -1,   -1,   -1,  262,  263,  264,   10,   -1,  267, 
          268,  269,   -1,  271,   -1,   -1,   -1,   -1,   91,   -1, 
           -1,   -1,   -1,  281,  282,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,  290,  291,   -1,  293,  294,  295,  296,  297, 
           -1,   44,   -1,   -1,   -1,   -1,   -1,  305,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   58,   59,   -1,   61,   -1, 
           63,   -1,   -1,   -1,   -1,   10,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,  339,   -1,   -1,  342,   -1,   -1,  345,   91,  347, 
           -1,   -1,   -1,   -1,   10,   -1,   -1,   -1,   -1,   44, 
           -1,   -1,   -1,   -1,   -1,  363,   -1,   -1,   -1,   -1, 
           -1,    0,   -1,   -1,   -1,   -1,   61,  375,   -1,   -1, 
           -1,   10,   -1,   -1,   -1,   -1,   -1,   -1,   44,   -1, 
          388,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   61,   91,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   44,   -1,  262,  263,  264, 
           -1,   -1,  267,  268,  269,   -1,  271,   -1,   -1,   58, 
           59,   -1,   61,   -1,   63,   91,  281,  282,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,  290,  291,   -1,  293,  294, 
          295,  296,  297,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
          305,   -1,   91,   -1,   -1,   -1,  279,   -1,   -1,   -1, 
           -1,   -1,  317,  318,  319,  320,  321,  322,  323,  324, 
          325,  326,  327,  328,  329,  330,   -1,   -1,  333,  334, 
          335,  336,   -1,  338,  339,   -1,   -1,  342,   -1,   -1, 
          345,   -1,  347,   -1,  349,   -1,  351,   -1,  353,  354, 
          355,  356,  357,  358,  359,  328,  361,   -1,  363,  262, 
          263,  264,  335,  336,  267,  268,  269,   -1,  271,  342, 
          375,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  281,  282, 
           -1,   -1,   -1,  388,   -1,   -1,   -1,  290,  291,   -1, 
          293,  294,  295,  296,  297,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,  305,    0,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   10,  317,  318,  319,  320,  321,  322, 
          323,  324,  325,  326,  327,  328,  329,  330,   -1,   -1, 
          333,  334,  335,  336,  279,  338,  339,   -1,   -1,  342, 
           -1,   -1,  345,   -1,  347,   -1,  349,   44,  351,   -1, 
          353,  354,  355,  356,  357,  358,  359,   -1,  361,   -1, 
          363,   58,   59,  279,   61,   -1,   63,   -1,   -1,   -1, 
           -1,   -1,  375,  262,  263,  264,   -1,   -1,  267,  268, 
          269,   -1,  271,  328,   -1,  388,   -1,   -1,   -1,   -1, 
          335,  336,  281,  282,   91,   -1,   -1,  342,   -1,   -1, 
           -1,  290,  291,   -1,  293,  294,  295,  296,  297,   -1, 
           -1,   -1,  328,   -1,   -1,   -1,  305,    0,   -1,  335, 
          336,   -1,   -1,   -1,   -1,   -1,  342,   10,  317,  318, 
          319,  320,  321,  322,  323,  324,  325,  326,  327,  328, 
          329,  330,   -1,   -1,  333,  334,  335,  336,   -1,  338, 
          339,   -1,   -1,  342,   -1,   -1,  345,   -1,  347,   10, 
          349,   44,  351,   -1,  353,  354,  355,  356,  357,  358, 
          359,   -1,  361,   -1,  363,   58,   59,   -1,   61,   -1, 
           63,   -1,   -1,   -1,   -1,   -1,  375,   -1,   -1,   -1, 
           -1,   -1,   -1,   44,   -1,   -1,   -1,   -1,   -1,  388, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   91,   -1, 
           61,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,    0, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   10, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           91,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   44,   -1,  262,  263,  264,   -1,   -1, 
          267,  268,  269,   -1,  271,   -1,   -1,   58,   59,   -1, 
           61,   -1,   63,   -1,  281,  282,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,  290,  291,   -1,  293,  294,  295,  296, 
          297,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  305,   -1, 
           91,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
          317,  318,  319,  320,  321,  322,  323,  324,  325,  326, 
          327,  328,  329,  330,   -1,   -1,  333,  334,  335,  336, 
           -1,  338,  339,   -1,   -1,  342,   -1,   -1,  345,   -1, 
          347,   -1,  349,   -1,  351,   -1,  353,  354,  355,  356, 
          357,  358,  359,   -1,  361,   -1,  363,   -1,   -1,   -1, 
           -1,   -1,    0,   -1,   -1,   -1,   -1,   -1,  375,  262, 
          263,  264,   10,   -1,  267,  268,  269,   -1,  271,   -1, 
           -1,  388,   -1,   -1,   -1,   -1,   -1,   -1,  281,  282, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,  290,  291,   -1, 
          293,  294,  295,  296,  297,   -1,   44,   -1,   -1,   -1, 
           -1,   -1,  305,   -1,   -1,   -1,   -1,   -1,  279,   -1, 
           58,   59,   -1,   -1,  317,  318,  319,  320,  321,  322, 
          323,  324,  325,  326,  327,  328,  329,  330,   -1,   -1, 
          333,  334,  335,  336,   -1,  338,  339,   -1,   -1,  342, 
           -1,   -1,  345,   -1,  347,   -1,  349,   -1,  351,   -1, 
          353,  354,  355,  356,  357,  358,  359,  328,  361,   -1, 
          363,  262,  263,  264,  335,  336,  267,  268,  269,   -1, 
          271,  342,  375,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
          281,  282,   -1,   -1,   -1,  388,   -1,   -1,   -1,  290, 
          291,   -1,  293,  294,  295,  296,  297,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,  305,    0,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   10,  317,  318,  319,  320, 
          321,  322,  323,  324,  325,  326,  327,  328,  329,  330, 
           -1,   -1,  333,  334,  335,  336,   -1,  338,  339,   -1, 
           -1,  342,   -1,   -1,  345,   -1,  347,   10,  349,   44, 
          351,   -1,  353,  354,  355,  356,  357,  358,  359,   -1, 
          361,   -1,  363,   58,   59,   -1,   -1,   -1,   63,   -1, 
           -1,   -1,   -1,   -1,  375,   -1,   -1,   -1,   -1,   -1, 
           -1,   44,   -1,   -1,   -1,   -1,   -1,  388,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   91,   -1,   61,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,    0,   -1,   -1, 
           -1,   -1,   -1,   -1,  262,  263,  264,   10,   -1,  267, 
          268,  269,   -1,  271,   -1,   -1,   -1,   -1,   91,   -1, 
           -1,   -1,   -1,  281,  282,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,  290,  291,   -1,  293,  294,  295,  296,  297, 
           -1,   44,   -1,   -1,   -1,   -1,   -1,  305,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   58,   59,   -1,   61,   -1, 
           63,   -1,   -1,   -1,   -1,   10,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,  339,   -1,   -1,  342,   -1,   -1,  345,   91,  347, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   44, 
           -1,   -1,   -1,   -1,   -1,  363,   -1,   -1,   -1,   -1, 
           -1,    0,   -1,   -1,   -1,   -1,   61,  375,   -1,   -1, 
           -1,   10,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
          388,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   91,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   44,   -1,  262,  263,  264, 
           -1,   -1,  267,  268,  269,   -1,  271,   -1,   -1,   58, 
           59,   -1,   61,   -1,   63,  280,  281,  282,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,  290,  291,   -1,  293,  294, 
          295,  296,  297,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
          305,   -1,   91,   -1,   -1,   -1,  279,   -1,   -1,   -1, 
           -1,   -1,  317,  318,  319,  320,  321,  322,  323,  324, 
          325,  326,  327,  328,  329,  330,   -1,   -1,  333,  334, 
          335,  336,   -1,   -1,  339,   -1,   -1,  342,   -1,   -1, 
          345,   -1,  347,   -1,  349,   -1,  351,   -1,  353,  354, 
          355,  356,  357,  358,  359,  328,  361,  362,  363,  262, 
          263,  264,  335,  336,  267,  268,  269,   -1,  271,  342, 
          375,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  281,  282, 
           -1,   -1,   -1,  388,   -1,   -1,   -1,  290,  291,   -1, 
          293,  294,  295,  296,  297,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,  305,    0,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   10,  317,  318,  319,  320,  321,  322, 
          323,  324,  325,  326,  327,  328,  329,  330,   -1,   -1, 
          333,  334,  335,  336,  279,   -1,  339,   -1,   -1,  342, 
           -1,   -1,  345,   -1,  347,   -1,  349,   44,  351,   -1, 
          353,  354,  355,  356,  357,  358,  359,   -1,  361,   -1, 
          363,   58,   59,   -1,   -1,   -1,   63,   -1,   -1,   -1, 
           -1,   -1,  375,  262,  263,  264,   -1,   -1,  267,  268, 
          269,   -1,  271,  328,   -1,  388,   -1,   -1,   -1,   -1, 
          335,  336,  281,  282,   91,   -1,   -1,  342,   -1,   -1, 
           -1,  290,  291,   -1,  293,  294,  295,  296,  297,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,  305,    0,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   10,  317,  318, 
          319,  320,  321,  322,  323,  324,  325,  326,  327,  328, 
          329,  330,   -1,   -1,  333,  334,  335,  336,   -1,   -1, 
          339,   -1,   -1,  342,   -1,   -1,  345,   -1,  347,   -1, 
          349,   44,  351,   -1,  353,  354,  355,  356,  357,  358, 
          359,   -1,  361,   -1,  363,   58,   59,   -1,   -1,   -1, 
           63,   -1,   -1,   -1,   -1,   -1,  375,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  388, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   91,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,    0, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   10, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   44,   -1,  262,  263,  264,   -1,   -1, 
          267,  268,  269,   -1,  271,   -1,   -1,   58,   59,   -1, 
           -1,   -1,   63,   -1,  281,  282,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,  290,  291,   -1,  293,  294,  295,  296, 
          297,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  305,   -1, 
           91,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
          317,  318,  319,  320,  321,  322,  323,  324,  325,  326, 
          327,  328,  329,  330,   -1,   -1,  333,  334,  335,  336, 
           -1,   -1,  339,   -1,   -1,  342,   -1,   -1,  345,   -1, 
          347,   -1,  349,   -1,  351,   -1,  353,  354,  355,  356, 
          357,  358,  359,   -1,  361,   -1,  363,   -1,   -1,   -1, 
           -1,   -1,    0,   -1,   -1,   -1,   -1,   -1,  375,  262, 
          263,  264,   10,   -1,  267,  268,  269,   -1,  271,   -1, 
           -1,  388,   -1,   -1,   -1,   -1,   -1,   -1,  281,  282, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,  290,  291,   -1, 
          293,  294,  295,  296,  297,   -1,   44,   -1,   -1,   -1, 
           -1,   -1,  305,   -1,   -1,   -1,   10,   -1,   -1,   -1, 
           58,   59,   -1,   -1,  317,  318,  319,  320,  321,  322, 
          323,  324,  325,  326,  327,  328,  329,  330,   -1,   -1, 
          333,  334,  335,  336,   -1,   -1,  339,   -1,   -1,  342, 
           44,   -1,  345,   -1,  347,   -1,  349,   -1,  351,   -1, 
          353,  354,  355,  356,  357,  358,  359,   61,  361,   -1, 
          363,  262,  263,  264,   -1,   -1,  267,  268,  269,   -1, 
          271,   -1,  375,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
          281,  282,   -1,   -1,   -1,  388,   -1,   91,   -1,  290, 
          291,   -1,  293,  294,  295,  296,  297,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,  305,    0,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   10,  317,  318,  319,  320, 
          321,  322,  323,  324,  325,  326,  327,  328,  329,  330, 
           -1,   -1,  333,  334,  335,  336,   -1,   -1,  339,   -1, 
           -1,  342,   -1,   -1,  345,   -1,  347,   -1,  349,   44, 
          351,   -1,  353,  354,  355,  356,  357,  358,  359,   -1, 
          361,   -1,  363,   58,   59,   -1,   -1,   -1,   63,   -1, 
           -1,   -1,   -1,   -1,  375,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,  388,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   91,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,    0,   -1,   -1, 
           -1,   -1,   -1,   -1,  262,  263,  264,   10,   -1,  267, 
          268,  269,   -1,  271,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,  281,  282,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,  290,  291,   -1,  293,  294,  295,  296,  297, 
           -1,   44,   -1,   -1,   -1,   -1,   -1,  305,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   58,   59,   -1,   -1,   -1, 
           63,   -1,   -1,   -1,   -1,  279,  280,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,  339,   -1,   -1,  342,   -1,   -1,  345,   91,  347, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,    0,   -1,   -1,  363,   -1,   -1,   -1,   -1, 
           -1,   -1,   10,   -1,  328,   -1,   -1,  375,   -1,   -1, 
           -1,  335,  336,   -1,   -1,   -1,   -1,  341,  342,   -1, 
          388,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   44,   -1,  362,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,  262,  263,  264, 
           -1,   59,  267,  268,  269,   63,  271,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,  281,  282,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,  290,  291,   -1,  293,  294, 
          295,  296,  297,   91,   -1,   -1,   -1,   -1,   -1,   -1, 
          305,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,  317,  318,  319,  320,  321,  322,  323,  324, 
          325,  326,  327,  328,  329,  330,   -1,   -1,  333,  334, 
          335,  336,   -1,   10,  339,   -1,   -1,  342,   -1,   -1, 
          345,   -1,  347,   -1,  349,    0,  351,   -1,  353,  354, 
          355,  356,  357,  358,  359,   10,  361,   -1,  363,  262, 
          263,  264,   -1,   -1,  267,  268,  269,   44,  271,   -1, 
          375,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  281,  282, 
           -1,   -1,   -1,  388,   61,   -1,   -1,  290,  291,   44, 
          293,  294,  295,  296,  297,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,  305,   58,   59,   -1,   -1,   -1,   63,   -1, 
           -1,   -1,   -1,   10,   91,  318,  319,  320,  321,  322, 
          323,  324,  325,  326,  327,  328,  329,  330,   -1,   10, 
          333,  334,  335,  336,   -1,   -1,  339,   -1,   -1,  342, 
           -1,   -1,  345,   -1,  347,   -1,  349,   44,  351,   -1, 
          353,  354,  355,  356,  357,  358,  359,   -1,  361,   -1, 
          363,   -1,   59,   44,  262,  263,  264,   -1,   -1,  267, 
          268,  269,  375,  271,   -1,   -1,   -1,   -1,   -1,   -1, 
           61,   -1,   -1,  281,  282,  388,   -1,   -1,   -1,   -1, 
           -1,   -1,  290,  291,   91,  293,  294,  295,  296,  297, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,    0,   -1, 
           91,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   10,  317, 
          318,  319,  320,  321,  322,  323,  324,  325,  326,  327, 
          328,  329,  330,   -1,   -1,  333,  334,  335,  336,   -1, 
           -1,  339,   -1,   -1,  342,   -1,   -1,  345,   -1,  347, 
           -1,  349,   44,  351,   -1,  353,  354,  355,  356,  357, 
          358,  359,   -1,  361,   -1,  363,   -1,   59,   -1,   -1, 
           -1,   63,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
          388,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   91, 
           -1,   -1,  279,  280,   -1,   -1,   -1,  262,  263,  264, 
            0,   -1,  267,  268,  269,   -1,  271,   -1,   -1,   -1, 
           10,   -1,   -1,   -1,   -1,   -1,  281,  282,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,  290,  291,   -1,  293,  294, 
          295,  296,  297,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
          305,  328,   -1,   -1,   44,   -1,   -1,   -1,  335,  336, 
           -1,   -1,   -1,   -1,  341,  342,   -1,   -1,   -1,   59, 
          325,   -1,   -1,   63,  329,  330,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,  280,  339,  362,   -1,  342,   -1,   -1, 
          345,   -1,  347,   -1,   -1,   -1,   -1,   -1,  279,  280, 
           -1,   91,   -1,   -1,   -1,   -1,   -1,   -1,  363,   -1, 
           -1,   -1,   -1,   -1,   -1,    0,   -1,   -1,   -1,   -1, 
          375,   -1,   -1,   -1,   -1,   10,   -1,   -1,   -1,   -1, 
           -1,  328,   -1,  388,   -1,   -1,   -1,   -1,  335,  336, 
           -1,   -1,   -1,   -1,  341,   -1,   -1,  328,   -1,   -1, 
           -1,   -1,   -1,   -1,  335,  336,   -1,   -1,   -1,   44, 
          341,  342,  359,   -1,   -1,  362,   -1,   -1,   -1,   -1, 
          262,  263,  264,   58,   59,  267,  268,  269,   63,  271, 
           -1,  362,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  281, 
          282,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  290,  291, 
           -1,  293,  294,  295,  296,  297,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,  317,  318,  319,  320,  321, 
          322,  323,  324,  325,  326,  327,  328,  329,  330,   -1, 
           -1,  333,  334,  335,  336,   -1,   -1,  339,   -1,   -1, 
          342,   -1,   -1,  345,   -1,  347,   -1,  349,    0,  351, 
           -1,  353,  354,  355,  356,  357,  358,  359,   10,  361, 
           -1,  363,  262,  263,  264,   -1,   -1,  267,  268,  269, 
           -1,  271,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,  281,  282,   -1,   -1,   -1,  388,   -1,   -1,   -1, 
          290,  291,   44,  293,  294,  295,  296,  297,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   58,   59,   -1,   -1, 
           -1,   63,   -1,   -1,   -1,   -1,   -1,  317,  318,  319, 
          320,  321,  322,  323,  324,  325,  326,  327,  328,  329, 
          330,   -1,   -1,  333,  334,  335,  336,   -1,   -1,  339, 
           -1,   -1,  342,   -1,   -1,  345,   -1,  347,   -1,  349, 
            0,  351,   -1,  353,  354,  355,  356,  357,  358,  359, 
           10,  361,   -1,  363,   -1,   -1,   -1,  262,  263,  264, 
           -1,   -1,  267,  268,  269,   -1,  271,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,  281,  282,  388,   -1, 
           -1,   -1,   -1,   -1,   44,  290,  291,   -1,  293,  294, 
          295,  296,  297,   -1,   -1,   -1,   -1,   -1,   58,   59, 
          305,   -1,   -1,   63,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,  318,  319,  320,  321,  322,  323,  324, 
          325,  326,  327,   -1,  329,  330,   -1,   -1,  333,  334, 
           -1,   -1,   -1,   -1,  339,   -1,   -1,  342,   -1,   -1, 
          345,   -1,  347,   -1,  349,    0,  351,   -1,  353,  354, 
          355,  356,  357,  358,  359,   10,  361,   -1,  363,   -1, 
           -1,   -1,   -1,   10,   -1,   -1,   -1,   -1,   -1,   -1, 
          375,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,  388,   -1,   -1,   -1,   -1,   -1,   44, 
           -1,   -1,   10,   -1,   -1,   -1,   -1,   44,   -1,   -1, 
          262,  263,  264,   58,   59,  267,  268,  269,   63,  271, 
           -1,   -1,   -1,   -1,   61,   -1,   -1,   -1,   -1,  281, 
          282,   -1,   -1,   -1,   -1,   -1,   44,   -1,  290,  291, 
           -1,  293,  294,  295,  296,  297,   -1,   -1,   -1,   -1, 
           -1,   59,   -1,  305,   91,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,  318,  319,  320,  321, 
          322,  323,  324,  325,  326,  327,   -1,  329,  330,   -1, 
           -1,  333,  334,   91,   -1,   -1,   -1,  339,   -1,   -1, 
          342,   -1,   -1,  345,   -1,  347,   -1,  349,    0,  351, 
           -1,  353,  354,  355,  356,  357,  358,  359,   10,  361, 
           -1,  363,  262,  263,  264,   -1,   -1,  267,  268,  269, 
           -1,  271,   -1,  375,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,  281,  282,   -1,   -1,   -1,  388,   -1,   -1,   -1, 
          290,  291,   44,  293,  294,  295,  296,  297,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,  305,   58,   59,   -1,   -1, 
           -1,   63,   -1,   -1,   -1,   -1,   -1,   -1,  318,  319, 
          320,  321,  322,  323,  324,  325,  326,  327,   -1,  329, 
          330,   -1,   -1,  333,  334,   -1,   -1,   -1,   -1,  339, 
           -1,   -1,  342,   -1,   -1,  345,   -1,  347,   -1,  349, 
            0,  351,   -1,  353,  354,  355,  356,  357,  358,  359, 
           10,  361,   -1,  363,   -1,   -1,   -1,  262,  263,  264, 
           -1,   -1,  267,  268,  269,  375,  271,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,  281,  282,  388,   -1, 
           -1,   -1,  279,   -1,   44,  290,  291,   -1,  293,  294, 
          295,  296,  297,   -1,   -1,   -1,   -1,   -1,   58,   59, 
          305,   -1,   -1,   63,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,  280,  318,  319,  320,  321,  322,  323,  324, 
          325,  326,  327,   -1,  329,  330,   -1,   -1,  333,  334, 
           -1,  328,   -1,   -1,  339,   -1,   -1,  342,  335,  336, 
          345,   -1,  347,   -1,  349,  342,  351,   -1,  353,  354, 
          355,  356,  357,  358,  359,    0,  361,   -1,  363,   -1, 
          328,   -1,   -1,   -1,   -1,   10,   -1,  335,  336,   -1, 
          375,   -1,   -1,  341,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,  388,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,  359,   -1,   -1,  362,   -1,   -1,   -1,   -1,   44, 
          262,  263,  264,   -1,   -1,  267,  268,  269,   -1,  271, 
           -1,   -1,   -1,   -1,   59,   -1,   61,   -1,   63,  281, 
          282,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  290,  291, 
           -1,  293,  294,  295,  296,  297,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,  305,   -1,   -1,   91,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,  318,  319,  320,  321, 
          322,  323,  324,  325,  326,  327,    0,  329,  330,   -1, 
           -1,  333,  334,   -1,   -1,   -1,   10,  339,   -1,   -1, 
          342,   -1,   -1,  345,   -1,  347,   -1,  349,   -1,  351, 
           -1,  353,  354,  355,  356,  357,  358,  359,   -1,  361, 
           -1,  363,  262,  263,  264,   -1,   -1,  267,  268,  269, 
           44,  271,   -1,  375,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,  281,  282,   -1,   -1,   59,  388,   61,   -1,   63, 
          290,  291,   -1,  293,  294,  295,  296,  297,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,  305,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   91,  318,  319, 
          320,  321,  322,  323,  324,  325,  326,  327,    0,  329, 
          330,   -1,   -1,  333,  334,   -1,   -1,   -1,   10,  339, 
           -1,   -1,  342,   -1,   -1,  345,   -1,  347,   -1,  349, 
           -1,  351,   -1,  353,  354,  355,  356,  357,  358,  359, 
           -1,  361,   -1,  363,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   44,   -1,   -1,  375,   -1,  262,  263,  264, 
           -1,   -1,   -1,  268,  269,   -1,  271,   59,  388,   61, 
           -1,   63,   -1,   -1,   -1,  280,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,  290,  291,   -1,  293,  294, 
          295,  296,  297,   -1,   -1,   -1,   -1,   -1,   -1,   91, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,  317,  318,  319,  320,  321,  322,  323,  324, 
          325,  326,  327,  328,  329,  330,   -1,   -1,  333,  334, 
          335,  336,   -1,   -1,   -1,   -1,   -1,  342,   -1,   -1, 
           -1,   -1,   -1,   -1,  349,   -1,  351,   -1,  353,  354, 
          355,  356,  357,  358,  359,   -1,  361,  362,  363,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  262,  263, 
          264,   -1,   -1,   -1,  268,  269,   -1,  271,   -1,    0, 
           -1,   -1,   -1,  388,   -1,   -1,   -1,   -1,   -1,   10, 
           -1,   -1,   -1,   -1,   -1,   -1,  290,  291,   -1,  293, 
          294,  295,  296,  297,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,  317,  318,  319,  320,  321,  322,  323, 
          324,  325,  326,  327,  328,  329,  330,   -1,   59,  333, 
          334,  335,  336,   -1,  338,   -1,   -1,   -1,  342,   -1, 
           -1,   -1,   -1,   -1,   -1,  349,    0,  351,   -1,  353, 
          354,  355,  356,  357,  358,  359,   10,  361,   -1,  363, 
          262,  263,  264,   -1,   -1,   -1,  268,  269,   -1,  271, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,  388,   -1,   -1,   -1,  290,  291, 
           44,  293,  294,  295,  296,  297,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   59,   -1,   61,   -1,   63, 
           -1,   -1,   -1,   -1,   -1,  317,  318,  319,  320,  321, 
          322,  323,  324,  325,  326,  327,  328,  329,  330,   -1, 
           -1,  333,  334,  335,  336,   -1,  338,   91,   -1,   -1, 
          342,   -1,   -1,   -1,   -1,   -1,   -1,  349,    0,  351, 
           -1,  353,  354,  355,  356,  357,  358,  359,   10,  361, 
           -1,  363,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,  388,   -1,   -1,   -1, 
           -1,   -1,   44,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   59,   -1,   61, 
           -1,   63,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,    0,   -1,   -1,   -1,   -1, 
           -1,  262,  263,  264,   -1,   10,  267,  268,  269,   91, 
          271,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
          281,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  290, 
          291,   -1,  293,  294,  295,  296,  297,   -1,   -1,   44, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   59,   -1,   61,   -1,   63,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,  328,   -1,   -1, 
           -1,   -1,   -1,   -1,  335,  336,   -1,   -1,  262,  263, 
          264,  342,   -1,   -1,  268,  269,   91,  271,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,  363,   -1,   -1,    0,  290,  291,   -1,  293, 
          294,  295,  296,  297,   -1,   10,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,  388,   -1,   -1, 
           -1,   -1,   -1,  317,  318,  319,  320,  321,  322,  323, 
          324,  325,  326,  327,  328,  329,  330,    0,   -1,  333, 
          334,  335,  336,   -1,  338,   -1,   -1,   10,  342,   -1, 
           -1,   -1,   -1,   -1,   59,  349,   -1,  351,   -1,  353, 
          354,  355,  356,  357,  358,  359,   -1,  361,   -1,  363, 
          262,  263,  264,   -1,   -1,   -1,  268,  269,   -1,  271, 
           -1,   44,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,  388,   -1,   59,   -1,  290,  291, 
           -1,  293,  294,  295,  296,  297,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,  317,  318,  319,  320,  321, 
          322,  323,  324,  325,  326,  327,  328,  329,  330,   -1, 
           -1,  333,  334,  335,  336,   -1,  338,  262,  263,  264, 
          342,   -1,   -1,  268,  269,   -1,  271,  349,   -1,  351, 
           -1,  353,  354,  355,  356,  357,  358,  359,   -1,  361, 
           -1,  363,   -1,   -1,   -1,  290,  291,   -1,  293,  294, 
          295,  296,  297,   -1,   -1,   -1,   -1,   -1,    0,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,  388,   -1,   10,   -1, 
           -1,   -1,  317,  318,  319,  320,  321,  322,  323,  324, 
          325,  326,  327,  328,  329,  330,   -1,   -1,  333,  334, 
          335,  336,   -1,  338,   -1,   -1,   -1,  342,   -1,   -1, 
           -1,   -1,   44,   -1,  349,   -1,  351,   -1,  353,  354, 
          355,  356,  357,  358,  359,   -1,  361,   59,  363,   61, 
           -1,   63,   -1,   -1,   -1,   -1,   -1,  262,  263,  264, 
           -1,   -1,  267,  268,  269,    0,  271,   -1,   -1,   -1, 
           -1,   -1,   -1,  388,   -1,   10,  281,  282,   -1,   91, 
           -1,   -1,   -1,   -1,   -1,  290,  291,   -1,  293,  294, 
          295,  296,  297,   -1,   -1,   -1,   -1,   -1,   -1,  262, 
          263,  264,   -1,   -1,  267,  268,  269,   -1,  271,   44, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   59,   -1,   61,   -1,   63,   -1, 
          293,  294,  295,  296,  297,   -1,   -1,  342,   -1,   -1, 
          345,   -1,    0,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   10,   -1,   -1,   -1,   91,   -1,  363,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,  339,   -1,   -1,  342, 
           -1,   -1,   -1,  388,   -1,   -1,   44,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
          363,   59,   -1,   61,   -1,   63,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,  388,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   91,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
          262,  263,  264,   -1,   -1,   -1,  268,  269,   -1,  271, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1, 
      };
   }

   private static final short[] yyCheck3() {
      return new short[] {

            0,   -1,  290,  291,   -1,  293,  294,  295,  296,  297, 
           10,   -1,   -1,   -1,   -1,   -1,   -1,   -1,    0,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   10,  317, 
          318,  319,  320,  321,  322,  323,  324,  325,  326,  327, 
          328,  329,  330,   -1,   44,  333,  334,  335,  336,   -1, 
          338,  262,  263,  264,  342,   -1,   -1,  268,  269,   59, 
          271,  349,   44,  351,   -1,  353,  354,  355,  356,  357, 
          358,  359,   -1,  361,   -1,  363,   -1,   59,   -1,  290, 
          291,   -1,  293,  294,  295,  296,  297,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
          388,   -1,   -1,   -1,   -1,   -1,  317,  318,  319,  320, 
          321,  322,  323,  324,  325,  326,  327,  328,  329,  330, 
           -1,   -1,  333,  334,  335,  336,   -1,  338,  262,  263, 
          264,  342,   -1,   -1,  268,  269,   -1,  271,  349,   -1, 
          351,   -1,  353,  354,  355,  356,  357,  358,  359,   -1, 
          361,   -1,  363,   -1,   -1,   -1,  290,  291,   -1,  293, 
          294,  295,  296,  297,   -1,   -1,   -1,   -1,   -1,    0, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,  388,   -1,   10, 
           -1,   -1,   -1,  317,  318,  319,  320,  321,  322,  323, 
          324,  325,  326,  327,  328,  329,  330,   -1,   -1,  333, 
          334,  335,  336,   -1,  338,   -1,   -1,   -1,  342,   -1, 
           -1,   -1,   -1,   44,   -1,  349,   -1,  351,   -1,  353, 
          354,  355,  356,  357,  358,  359,   -1,  361,   59,  363, 
           61,   -1,   63,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,    0,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,  388,   -1,   10,   -1,   -1,   -1, 
           91,   -1,  262,  263,  264,   -1,   -1,  267,  268,  269, 
           -1,  271,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
          262,  263,  264,   -1,   -1,   -1,  268,  269,   -1,  271, 
           44,   -1,   -1,  293,  294,  295,  296,  297,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   59,   -1,   61,   -1,   63, 
           -1,  293,  294,  295,  296,  297,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,    0,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   10,   -1,   -1,   -1,   91,   -1,  339, 
           -1,   -1,  342,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
          342,   -1,   -1,  363,   -1,   -1,   -1,   44,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,  363,   59,   -1,   61,   -1,   63,   -1,  388,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,    0,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,  388,   -1,   10,   -1, 
           -1,   -1,   -1,   -1,   91,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,  262,  263,  264,   -1,   -1,   -1,  268,  269,   -1, 
          271,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   59,   -1,  290, 
          291,   -1,  293,  294,  295,  296,  297,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,    0,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   10,  317,  318,  319,  320, 
          321,  322,  323,  324,  325,  326,  327,  328,  329,  330, 
           -1,   -1,  333,  334,  335,  336,   -1,  338,  262,  263, 
          264,  342,   -1,   -1,  268,  269,   -1,  271,  349,   44, 
          351,   -1,  353,  354,  355,  356,  357,  358,  359,   -1, 
          361,   -1,  363,   58,   59,   -1,  290,  291,   63,  293, 
          294,  295,  296,  297,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,  388,   -1,   -1, 
           -1,   -1,   -1,  317,  318,  319,  320,  321,  322,  323, 
          324,  325,  326,  327,  328,  329,  330,   -1,   -1,  333, 
          334,  335,  336,   -1,  338,  262,  263,  264,  342,   -1, 
           -1,  268,  269,   -1,  271,  349,   -1,  351,   -1,  353, 
          354,  355,  356,  357,  358,  359,   -1,  361,   -1,  363, 
           -1,   -1,   -1,  290,  291,   -1,  293,  294,  295,  296, 
          297,   -1,   -1,   -1,   -1,   -1,    0,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,  388,   -1,   10,   -1,   -1,   -1, 
          317,  318,  319,  320,  321,  322,  323,  324,  325,  326, 
          327,  328,  329,  330,   -1,   -1,  333,  334,  335,  336, 
          262,  263,  264,   -1,   -1,  342,  268,  269,   -1,  271, 
           44,   -1,  349,   -1,  351,   -1,  353,  354,  355,  356, 
          357,  358,  359,   -1,  361,   59,  363,   61,   -1,   63, 
           -1,  293,  294,  295,  296,  297,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,  388,   -1,   -1,   -1,   -1,   -1,   91,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,  262,  263,  264, 
          342,   -1,  267,  268,  269,   -1,  271,   -1,    0,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,  281,  282,   10,   -1, 
           -1,  363,   -1,   -1,   -1,  290,  291,   -1,  293,  294, 
          295,  296,  297,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
          305,   -1,   -1,   -1,   -1,   -1,  388,   -1,   -1,   -1, 
           -1,   -1,   44,  318,  319,  320,  321,  322,  323,  324, 
          325,  326,  327,   -1,  329,  330,   58,   59,  333,  334, 
           -1,   63,   -1,   -1,  339,   -1,   -1,  342,   -1,   -1, 
          345,   -1,  347,   -1,   -1,   -1,  351,   -1,   -1,   -1, 
          355,  356,  357,  358,  359,   -1,  361,   -1,  363,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
          375,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,  388,   -1,   -1,    0,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   10,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  262,  263, 
          264,   -1,   -1,   -1,  268,  269,   -1,  271,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           44,   -1,   -1,   -1,   -1,   -1,  290,  291,   -1,  293, 
          294,  295,  296,  297,   58,   59,   -1,   -1,   -1,   63, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,  317,  318,  319,  320,  321,  322,  323, 
          324,  325,  326,  327,  328,  329,  330,   -1,   -1,  333, 
          334,  335,  336,   -1,   -1,   -1,   -1,   -1,  342,   -1, 
           -1,   -1,   -1,   -1,   -1,  349,   -1,  351,   -1,  353, 
          354,  355,  356,  357,  358,  359,   -1,  361,    0,  363, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   10,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
          262,  263,  264,   -1,  388,  267,  268,  269,   -1,  271, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  281, 
          282,   -1,   44,   -1,   -1,   -1,   -1,   -1,  290,  291, 
           -1,  293,  294,  295,  296,  297,   58,   59,   -1,   -1, 
           -1,   63,   -1,  305,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,  318,  319,  320,  321, 
          322,  323,  324,  325,  326,  327,   -1,  329,  330,   -1, 
           -1,  333,  334,   -1,   -1,   -1,   -1,  339,   -1,   -1, 
          342,   -1,   -1,  345,   -1,  347,   -1,   -1,   -1,  351, 
           -1,   -1,   -1,  355,  356,  357,  358,  359,   -1,  361, 
            0,  363,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           10,   -1,   -1,  375,   -1,   -1,   -1,   -1,  262,  263, 
          264,   -1,   -1,  267,  268,  269,  388,  271,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,  281,  282,   -1, 
           -1,   -1,   -1,   -1,   44,   -1,  290,  291,   -1,  293, 
          294,  295,  296,  297,   -1,   -1,   -1,   -1,   58,   59, 
           -1,  305,   -1,   63,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,  318,  319,  320,  321,  322,  323, 
          324,  325,  326,  327,   -1,  329,  330,   -1,   -1,  333, 
          334,   -1,   -1,   -1,   -1,  339,   -1,   -1,  342,   -1, 
           -1,  345,   -1,  347,   -1,   -1,   -1,  351,   -1,   -1, 
           -1,   -1,   -1,  357,  358,  359,   -1,  361,    0,  363, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   10,   -1, 
           -1,  375,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
          262,  263,  264,   -1,  388,  267,  268,  269,   -1,  271, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  281, 
          282,   -1,   44,   -1,   -1,   -1,   -1,   -1,  290,  291, 
           -1,  293,  294,  295,  296,  297,   58,   59,   -1,   -1, 
           -1,   63,   -1,  305,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,  318,  319,  320,  321, 
          322,  323,  324,  325,  326,  327,   -1,  329,  330,   -1, 
           -1,  333,  334,   -1,   -1,   -1,   -1,  339,   -1,   -1, 
          342,    0,   -1,  345,   -1,  347,   -1,   -1,   -1,  351, 
           -1,   10,   -1,   -1,   -1,  357,  358,  359,   -1,  361, 
           -1,  363,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,  375,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,  262,  263,  264,   44,  388,  267,  268,  269, 
           -1,  271,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   58, 
           59,  281,  282,   -1,   63,   -1,   -1,   -1,   -1,   -1, 
          290,  291,   -1,  293,  294,  295,  296,  297,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,  305,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  318,  319, 
          320,  321,  322,  323,  324,  325,  326,  327,   -1,  329, 
          330,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  339, 
           -1,   -1,  342,   -1,   -1,  345,   -1,  347,   -1,   -1, 
           -1,  351,   -1,   -1,   -1,   -1,   -1,  357,  358,  359, 
           -1,  361,    0,  363,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   10,   -1,   -1,  375,   -1,   -1,   -1,   -1, 
          262,  263,  264,   -1,   -1,  267,  268,  269,  388,  271, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  281, 
          282,   -1,   -1,   -1,   -1,   -1,   44,   -1,  290,  291, 
           -1,  293,  294,  295,  296,  297,   -1,   -1,   -1,   -1, 
           58,   59,   -1,  305,   -1,   63,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,  318,  319,  320,  321, 
          322,  323,  324,  325,  326,  327,   -1,  329,  330,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,  339,   -1,   -1, 
          342,   -1,   -1,  345,    0,  347,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   10,  357,  358,  359,   -1,  361, 
           -1,  363,   -1,  262,  263,  264,   -1,   -1,  267,  268, 
          269,   -1,  271,  375,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,  281,  282,   -1,   -1,  388,   -1,   44,   -1, 
           -1,  290,  291,   -1,  293,  294,  295,  296,  297,   -1, 
           -1,   -1,   58,   59,   -1,   -1,  305,   63,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  318, 
          319,  320,  321,  322,  323,  324,  325,  326,  327,   -1, 
          329,  330,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
          339,   -1,   -1,  342,   -1,   -1,  345,   -1,  347,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  357,  358, 
          359,   -1,  361,   -1,  363,   -1,   -1,   -1,   -1,   -1, 
           -1,    0,   -1,   -1,   -1,   -1,  375,   -1,   -1,   -1, 
           -1,   10,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  388, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,  262,  263,  264,   -1,   -1,  267, 
          268,  269,   -1,  271,   -1,   44,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,  281,  282,   -1,   -1,   -1,   -1,   58, 
           59,   -1,  290,  291,   63,  293,  294,  295,  296,  297, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,  305,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
          318,  319,  320,  321,  322,  323,  324,  325,  326,  327, 
           -1,  329,  330,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,  339,   -1,   -1,  342,    0,   -1,  345,   -1,  347, 
           -1,   -1,   -1,   -1,   -1,   10,   -1,   -1,   -1,  357, 
          358,   -1,   -1,   -1,   -1,  363,  262,  263,  264,   -1, 
           -1,  267,  268,  269,   -1,  271,   -1,  375,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,  281,  282,   -1,   -1,   44, 
          388,   -1,   -1,   -1,  290,  291,   -1,  293,  294,  295, 
          296,  297,   -1,   58,   59,   -1,   -1,   -1,   63,  305, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,  318,  319,  320,  321,  322,  323,  324,  325, 
          326,  327,   -1,  329,  330,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,  339,   -1,   -1,  342,    0,   -1,  345, 
           -1,  347,   -1,   -1,   -1,   -1,   -1,   10,   -1,   -1, 
           -1,  357,  358,   -1,   -1,   -1,   -1,  363,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  375, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   44,  388,  262,  263,  264,   -1,   -1,  267,  268, 
          269,   -1,  271,   -1,   -1,   58,   59,   -1,   -1,   -1, 
           63,   -1,  281,  282,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,  290,  291,   -1,  293,  294,  295,  296,  297,   -1, 
           -1,   -1,   -1,    0,   -1,   -1,  305,   -1,   -1,   -1, 
           -1,   -1,   -1,   10,   -1,   -1,   -1,   -1,   -1,  318, 
          319,  320,  321,  322,  323,  324,  325,  326,  327,   -1, 
          329,  330,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
          339,   -1,   -1,  342,   -1,   -1,  345,   44,  347,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  357,  358, 
           -1,   58,   59,   -1,  363,   -1,   63,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,  375,  262,  263,  264, 
           -1,   -1,  267,  268,  269,   -1,  271,   -1,   -1,  388, 
            0,   -1,   -1,   -1,   -1,   -1,  281,  282,   -1,   -1, 
           10,   -1,   -1,   -1,   -1,  290,  291,   -1,  293,  294, 
          295,  296,  297,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
          305,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,  318,  319,  320,  321,  322,  323,  324, 
          325,  326,  327,    0,  329,  330,   -1,   -1,   -1,   59, 
           -1,   -1,   -1,   10,  339,   -1,   -1,  342,   -1,   -1, 
          345,   -1,  347,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,  357,  358,   -1,   -1,   -1,   -1,  363,  262, 
          263,  264,   -1,   -1,  267,  268,  269,   44,  271,   -1, 
          375,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  281,  282, 
           -1,   58,   59,  388,   -1,   -1,   63,  290,  291,   -1, 
          293,  294,  295,  296,  297,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,  305,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,  324,  325,   -1,   -1,   -1,  329,  330,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,  339,   -1,    0,  342, 
           -1,   -1,  345,   -1,  347,  262,  263,  264,   10,   -1, 
          267,  268,  269,   -1,  271,   -1,   -1,   -1,   -1,   -1, 
          363,   -1,   -1,   -1,  281,  282,   -1,   -1,   -1,   -1, 
           -1,   -1,  375,  290,  291,   -1,  293,  294,  295,  296, 
          297,   -1,   44,   -1,   -1,  388,   -1,   -1,  305,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   58,   59,   -1,   -1, 
           -1,   63,   -1,   -1,   -1,   -1,   -1,  324,  325,   -1, 
           -1,   -1,  329,  330,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,  339,   -1,   -1,  342,   -1,   -1,  345,   -1, 
          347,   -1,  262,  263,  264,   -1,   -1,  267,  268,  269, 
            0,  271,   -1,   -1,   -1,   -1,  363,   -1,   -1,   -1, 
           10,  281,  282,   -1,   -1,   -1,   -1,   -1,  375,   -1, 
          290,  291,   -1,  293,  294,  295,  296,  297,   -1,   -1, 
           -1,  388,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   44,  262,  263,  264,   -1,   -1, 
          267,  268,  269,   -1,  271,   -1,   -1,   -1,   58,   59, 
           -1,   -1,   -1,   63,  281,  282,   -1,   -1,   -1,   -1, 
           -1,   -1,  342,  290,  291,  345,  293,  294,  295,  296, 
          297,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  305,    0, 
           -1,   -1,   -1,  363,   -1,   -1,   -1,   -1,   -1,   10, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,  324,  325,   -1, 
           -1,   -1,  329,  330,   -1,   -1,   -1,   -1,  388,   -1, 
           -1,   -1,  339,   -1,   -1,  342,   -1,   -1,  345,   -1, 
          347,   -1,   -1,   44,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,  363,   58,   59,   -1, 
           -1,   -1,   63,   -1,   -1,   -1,   -1,   -1,  375,   -1, 
          262,  263,  264,   -1,   -1,  267,  268,  269,   -1,  271, 
           -1,  388,   -1,   -1,   -1,    0,   -1,   -1,   -1,  281, 
          282,   -1,   -1,   -1,   -1,   10,   -1,   -1,  290,  291, 
           -1,  293,  294,  295,  296,  297,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,  305,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   44, 
           -1,   -1,  324,  325,   -1,   -1,   -1,  329,  330,   -1, 
           -1,   -1,   -1,   58,   59,   -1,   -1,  339,   63,   -1, 
          342,   -1,   -1,  345,   -1,  347,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,  363,  262,  263,  264,   -1,   -1,  267,  268,  269, 
           -1,  271,   -1,  375,   -1,   -1,   -1,    0,   -1,   -1, 
           -1,  281,  282,   -1,   -1,   -1,  388,   10,   -1,   -1, 
          290,  291,   -1,  293,  294,  295,  296,  297,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,  305,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   44,   -1,   -1,  324,  325,   -1,   -1,   -1,  329, 
          330,   -1,   -1,   -1,   -1,   58,   59,   -1,   -1,  339, 
           -1,   -1,  342,   -1,   -1,  345,   -1,  347,   -1,   -1, 
           -1,  262,  263,  264,   -1,   -1,  267,  268,  269,   -1, 
          271,   -1,   -1,  363,    0,   -1,   -1,   -1,   -1,   -1, 
          281,  282,   -1,   -1,   10,  375,   -1,   -1,   -1,  290, 
          291,   -1,  293,  294,  295,  296,  297,   -1,  388,   -1, 
           -1,   -1,   -1,   -1,  305,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   44,   -1, 
           -1,   -1,   -1,  324,  325,   -1,   -1,   -1,  329,  330, 
           -1,   -1,   58,   59,   -1,   -1,   -1,   -1,  339,   -1, 
           -1,  342,   -1,   -1,  345,   -1,  347,  262,  263,  264, 
           -1,   -1,  267,  268,  269,   -1,  271,   -1,   -1,   -1, 
            0,   -1,  363,   -1,   -1,   -1,  281,  282,   -1,   -1, 
           10,   -1,   -1,   -1,  375,  290,  291,   -1,  293,  294, 
          295,  296,  297,   -1,   -1,   -1,   -1,  388,   -1,   -1, 
          305,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   44,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   58,   59, 
           -1,   -1,   -1,   -1,  339,   -1,   -1,  342,   -1,   -1, 
          345,   -1,  347,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,    0,  363,  262, 
          263,  264,   -1,   -1,  267,  268,  269,   10,  271,   -1, 
          375,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  281,  282, 
           -1,   -1,   -1,  388,   -1,   -1,   -1,  290,  291,   -1, 
          293,  294,  295,  296,  297,   -1,   -1,   -1,   -1,   -1, 
           -1,   44,  305,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   58,   59,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,  339,   -1,   -1,  342, 
           -1,   -1,  345,   -1,  347,   -1,  262,  263,  264,   -1, 
           -1,  267,  268,  269,   -1,  271,   -1,   -1,   -1,    0, 
          363,   -1,   -1,   -1,   -1,  281,  282,   -1,   -1,   10, 
           -1,   -1,  375,   -1,  290,  291,   -1,  293,  294,  295, 
          296,  297,   -1,   -1,   -1,  388,   -1,   -1,   -1,  305, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,    0,   -1, 
           -1,   -1,   -1,   44,   -1,   -1,   -1,   -1,   10,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   58,   59,   -1, 
           -1,   -1,   -1,  339,   -1,   -1,  342,   -1,   -1,  345, 
           -1,  347,  262,  263,  264,   -1,   -1,  267,  268,  269, 
           -1,  271,   44,   -1,   -1,   -1,   -1,  363,   -1,   -1, 
           -1,  281,  282,   -1,   -1,   -1,   58,   59,   -1,  375, 
          290,  291,   -1,  293,  294,  295,  296,  297,   -1,   -1, 
           -1,   -1,  388,   -1,   -1,  305,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  339, 
           -1,   -1,  342,    0,   -1,  345,   -1,  347,   -1,  262, 
          263,  264,   -1,   10,  267,  268,  269,   -1,  271,   -1, 
           -1,   -1,   -1,  363,   -1,   -1,   -1,   -1,  281,  282, 
           -1,   -1,   -1,   -1,   -1,  375,   -1,  290,  291,   -1, 
          293,  294,  295,  296,  297,   -1,   -1,   44,  388,   -1, 
           -1,   -1,  305,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   58,   59,   -1,   -1,    0,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   10,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,  339,   -1,   -1,  342, 
           -1,   -1,  345,   -1,  347,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   44, 
          363,  262,  263,  264,   -1,   -1,  267,  268,  269,   -1, 
          271,   -1,  375,   58,   59,   -1,   -1,   -1,   -1,   -1, 
          281,  282,   -1,   -1,   -1,  388,   -1,   -1,   -1,  290, 
          291,   -1,  293,  294,  295,  296,  297,   -1,   -1,   -1, 
          262,  263,  264,   -1,  305,  267,  268,  269,   -1,  271, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,    0,   -1,  281, 
          282,   -1,   -1,   -1,   -1,   -1,   -1,   10,  290,  291, 
           -1,  293,  294,  295,  296,  297,   -1,   -1,  339,   -1, 
           -1,  342,   -1,  305,  345,   -1,  347,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   44,  363,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,  375,   -1,   59,  339,   -1,    0, 
          342,   -1,   -1,  345,   -1,  347,   -1,  388,   -1,   10, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,  363,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,  375,   -1,  262,  263,  264,   -1,   -1, 
          267,  268,  269,   44,  271,   -1,  388,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,  281,  282,   -1,   58,   59,   -1, 
           -1,   -1,   -1,  290,  291,   -1,  293,  294,  295,  296, 
          297,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  305,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,  262,  263,  264, 
           -1,    0,  267,  268,  269,   -1,  271,   -1,   -1,   -1, 
           -1,   10,  339,   -1,   -1,  342,  281,  282,  345,   -1, 
          347,   -1,   -1,   -1,   -1,  290,  291,   -1,  293,  294, 
          295,  296,  297,   -1,   -1,   -1,  363,   -1,   -1,   -1, 
          305,   -1,   -1,   -1,   -1,   44,   -1,   -1,  375,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   58, 
           59,  388,   -1,    0,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   10,  339,   -1,   -1,  342,   -1,   -1, 
          345,   -1,  347,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  363,  262, 
          263,  264,   -1,   -1,  267,  268,  269,   44,  271,   -1, 
          375,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  281,  282, 
           -1,   -1,   59,  388,   -1,   -1,   -1,  290,  291,   -1, 
          293,  294,  295,  296,  297,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,  305,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,  262,  263,  264,   -1,    0,  267,  268,  269,   -1, 
          271,   -1,   -1,   -1,   -1,   10,  339,   -1,   -1,  342, 
          281,  282,  345,   -1,  347,   -1,   -1,   -1,   -1,  290, 
          291,   -1,  293,  294,  295,  296,   -1,   -1,   -1,   -1, 
          363,    0,   -1,   -1,  305,   -1,   -1,   -1,   -1,   44, 
           -1,   10,  375,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   59,  388,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  339,   -1, 
           -1,  342,   -1,   -1,  345,   -1,  347,   -1,   -1,   -1, 
           -1,    0,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           59,   10,  363,  262,  263,  264,   -1,   -1,  267,  268, 
          269,   -1,  271,   -1,  375,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,  281,  282,   -1,   -1,   -1,  388,    0,   -1, 
           -1,  290,  291,   -1,  293,  294,  295,  296,   10,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,  305,   -1,   -1,   -1, 
           59,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,  262,  263,  264,    0,   -1, 
          267,  268,  269,   -1,  271,   -1,   -1,   -1,   10,    0, 
          339,   -1,   -1,  342,  281,  282,  345,   59,  347,   10, 
           -1,   -1,   -1,  290,  291,   -1,  293,  294,  295,  296, 
          297,   -1,   -1,   -1,  363,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   44,   -1,   -1,   -1,  375,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   59,   -1,  388, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   59,   -1, 
           -1,   -1,   -1,   -1,   -1,  342,   -1,   -1,  345,   -1, 
          347,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,  363,  262,  263,  264, 
           -1,   -1,  267,  268,  269,   -1,  271,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,  281,  282,   -1,   -1, 
           -1,  388,   -1,   -1,   -1,  290,  291,   10,  293,  294, 
          295,  296,  297,  262,  263,  264,   -1,   -1,  267,  268, 
          269,   -1,  271,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,  281,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,  290,  291,   -1,  293,  294,  295,  296,  297,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,  342,   -1,   -1, 
          345,   -1,  347,  262,  263,  264,   -1,   -1,  267,  268, 
          269,   -1,  271,   -1,   -1,   -1,   -1,   -1,  363,   -1, 
           -1,   -1,  281,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,  290,  291,  342,  293,  294,  295,  296,  297,   -1, 
          262,  263,  264,  388,   -1,  267,  268,  269,   -1,  271, 
           -1,   -1,   -1,   -1,  363,   -1,   -1,   -1,   -1,  281, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  290,  291, 
           -1,  293,  294,  295,  296,  297,   -1,   -1,   -1,  388, 
          262,  263,  264,  342,   10,   -1,  268,  269,   -1,  271, 
           -1,  262,  263,  264,   -1,   -1,   -1,  268,  269,   -1, 
          271,   -1,   -1,   -1,  363,   -1,   -1,   -1,  290,  291, 
           -1,  293,  294,  295,  296,   -1,   -1,   -1,   -1,   -1, 
          342,   -1,  293,  294,  295,  296,  297,   -1,   -1,  388, 
           -1,   -1,   -1,   59,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,  363,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
          342,   -1,   -1,   -1,   -1,   91,  388,   -1,   -1,   -1, 
           -1,  342,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,  363,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,  363,   -1,  257,  258,  259,   -1,  261,   -1, 
           -1,   -1,  265,  266,   -1,   -1,  388,  270,   -1,  272, 
          273,  274,  275,  276,  277,  278,   -1,  388,   10,   -1, 
          283,  284,  285,  286,  287,  288,  289,   -1,   -1,  292, 
           -1,   -1,   -1,   -1,   -1,   -1,  299,   -1,   -1,  302, 
          303,  304,   -1,  306,  307,  308,  309,  310,  311,  312, 
          313,  314,  315,  316,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   59,   -1,   -1, 
           -1,   -1,   -1,   -1,  337,   -1,   -1,  340,   -1,   -1, 
          343,  344,   -1,  346,   -1,  348,   -1,  350,   -1,  352, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,  360,   -1,   91, 
           -1,   -1,  365,  366,  367,  368,  369,  370,   -1,   -1, 
           -1,  374,   -1,  376,  377,   -1,  379,  380,  381,  382, 
           -1,   -1,  385,  386,  387,   -1,   -1,   -1,   -1,   -1, 
          256,  257,  258,  259,  260,  261,  262,  263,  264,  265, 
          266,   -1,   -1,  269,  270,   -1,  272,  273,  274,  275, 
          276,  277,  278,   -1,  280,   -1,   10,  283,  284,  285, 
          286,  287,  288,  289,   -1,   -1,  292,   -1,   -1,   -1, 
           -1,   -1,  298,  299,   -1,  301,  302,  303,  304,   -1, 
          306,  307,  308,  309,  310,  311,   -1,  313,  314,  315, 
          316,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,  328,   -1,   -1,   59,   -1,   -1,   -1,  335, 
          336,  337,   -1,   -1,  340,  341,   -1,  343,  344,   -1, 
          346,   -1,  348,   -1,   -1,   -1,  352,   -1,   -1,   -1, 
           -1,  357,   -1,   -1,  360,   -1,  362,   91,   -1,  365, 
          366,  367,  368,  369,  370,   -1,   -1,   -1,  374,   -1, 
          376,  377,   -1,  379,  380,  381,  382,   -1,  384,  385, 
          386,   -1,   -1,   -1,  256,  257,  258,  259,  260,  261, 
          262,  263,  264,  265,  266,   -1,   -1,  269,  270,   -1, 
          272,  273,  274,  275,  276,  277,  278,   -1,  280,   -1, 
           10,  283,  284,  285,  286,  287,  288,  289,   -1,   -1, 
          292,   -1,   -1,   -1,   -1,   -1,  298,  299,   -1,  301, 
          302,  303,  304,   -1,  306,  307,  308,  309,  310,  311, 
           -1,  313,  314,  315,  316,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,  328,   -1,   -1,   59, 
           -1,   -1,   -1,  335,  336,  337,   -1,   -1,  340,  341, 
           -1,  343,  344,   -1,  346,   -1,  348,   -1,   -1,   -1, 
          352,   -1,   -1,   -1,   -1,  357,   -1,   -1,  360,   -1, 
          362,   91,   -1,  365,  366,  367,  368,  369,  370,   -1, 
           -1,   -1,  374,   -1,  376,  377,   -1,  379,  380,  381, 
          382,   -1,  384,  385,  386,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,  256,  257,  258,  259,  260,  261,  262,  263, 
          264,  265,  266,   -1,   -1,  269,  270,   -1,  272,  273, 
          274,  275,  276,  277,  278,   -1,  280,   -1,   10,  283, 
          284,  285,  286,  287,  288,  289,   -1,   -1,  292,   -1, 
           -1,   -1,   -1,   -1,  298,  299,   -1,  301,  302,  303, 
          304,   -1,  306,  307,  308,  309,  310,  311,   -1,  313, 
          314,  315,  316,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,  328,   -1,   -1,   59,   -1,   -1, 
           -1,  335,  336,  337,   -1,   -1,  340,  341,   -1,  343, 
          344,   -1,  346,   -1,  348,   -1,   -1,   -1,  352,   -1, 
           -1,   -1,   -1,  357,   -1,   -1,  360,   -1,  362,   91, 
           -1,  365,  366,  367,  368,  369,  370,   -1,   -1,   -1, 
          374,   -1,  376,  377,   -1,  379,  380,  381,  382,   -1, 
          384,  385,  386,   -1,   -1,   -1,  256,  257,  258,  259, 
          260,  261,  262,  263,  264,  265,  266,   -1,   -1,  269, 
          270,   -1,  272,  273,  274,  275,  276,  277,  278,   -1, 
           10,   -1,   -1,  283,  284,  285,  286,  287,  288,  289, 
           -1,   -1,  292,   -1,   -1,   -1,   -1,   -1,  298,  299, 
           -1,  301,  302,  303,  304,   -1,  306,  307,  308,  309, 
          310,  311,   -1,  313,  314,  315,  316,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  328,   59, 
           -1,   -1,   -1,   -1,   -1,  335,  336,  337,   -1,   -1, 
          340,  341,   -1,  343,  344,   -1,  346,   -1,  348,   -1, 
           -1,   -1,  352,   -1,   -1,   -1,   -1,  357,   -1,   -1, 
          360,   -1,   -1,   -1,   -1,  365,  366,  367,  368,  369, 
          370,   -1,   -1,   -1,  374,   -1,  376,  377,   -1,  379, 
          380,  381,  382,   -1,  384,  385,  386,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,  256,  257,  258,  259,  260,  261, 
          262,  263,  264,  265,  266,   -1,   -1,  269,  270,   -1, 
          272,  273,  274,  275,  276,  277,  278,   -1,   10,   -1, 
           -1,  283,  284,  285,  286,  287,  288,  289,   -1,   -1, 
          292,   -1,   -1,   -1,   -1,   -1,  298,  299,   -1,  301, 
          302,  303,  304,   -1,  306,  307,  308,  309,  310,  311, 
           -1,  313,  314,  315,  316,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,  328,   59,   -1,   -1, 
           -1,   -1,   -1,  335,  336,  337,   -1,   -1,  340,   -1, 
           -1,  343,  344,   -1,  346,   -1,  348,   -1,   -1,   -1, 
          352,   -1,   -1,   -1,   -1,  357,   -1,   -1,  360,   -1, 
           -1,   -1,   -1,  365,  366,  367,  368,  369,  370,   -1, 
           -1,   -1,  374,   -1,  376,  377,   -1,  379,  380,  381, 
          382,   -1,  384,  385,  386,   -1,  256,  257,  258,  259, 
          260,  261,  262,  263,  264,  265,  266,   -1,  268,  269, 
          270,  271,  272,  273,  274,  275,  276,  277,  278,   10, 
           -1,   -1,   -1,  283,  284,  285,  286,  287,  288,  289, 
           -1,   -1,  292,   -1,   -1,   -1,   -1,   -1,  298,  299, 
           -1,  301,  302,  303,  304,   -1,  306,  307,  308,  309, 
          310,  311,   -1,  313,  314,  315,  316,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   59,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,  337,   -1,   -1, 
          340,   -1,   -1,  343,  344,   -1,  346,   -1,  348,   -1, 
           -1,   -1,  352,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
          360,   -1,   -1,   -1,   -1,  365,  366,  367,  368,  369, 
          370,   -1,   -1,   -1,  374,   -1,  376,  377,   -1,  379, 
          380,  381,  382,   -1,  384,  385,  386,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,  256,  257,  258,  259,  260,  261, 
          262,  263,  264,  265,  266,   -1,   -1,  269,  270,   -1, 
          272,  273,  274,  275,  276,  277,  278,   10,   -1,   -1, 
           -1,  283,  284,  285,  286,  287,  288,  289,   -1,   -1, 
          292,   -1,   -1,   -1,   -1,   -1,  298,  299,   -1,  301, 
          302,  303,  304,   -1,  306,  307,  308,  309,  310,  311, 
           -1,  313,  314,  315,  316,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   59,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,  337,   -1,   -1,  340,   -1, 
           -1,  343,  344,   -1,  346,   -1,  348,   -1,   -1,   -1, 
          352,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  360,   -1, 
          362,   -1,   -1,  365,  366,  367,  368,  369,  370,   -1, 
           -1,   -1,  374,   -1,  376,  377,   -1,  379,  380,  381, 
          382,   -1,  384,  385,  386,  256,  257,  258,  259,  260, 
          261,  262,  263,  264,  265,  266,   -1,   -1,  269,  270, 
           -1,  272,  273,  274,  275,  276,  277,  278,   10,   -1, 
           -1,   -1,  283,  284,  285,  286,  287,  288,  289,   -1, 
           -1,  292,   -1,   -1,   -1,   -1,   -1,  298,  299,   -1, 
          301,  302,  303,  304,   -1,  306,  307,  308,  309,  310, 
          311,   -1,  313,  314,  315,  316,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   59,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,  337,   -1,   -1,  340, 
           -1,   -1,  343,  344,   -1,  346,   -1,  348,   -1,   -1, 
           -1,  352,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  360, 
           -1,   -1,   -1,   -1,  365,  366,  367,  368,  369,  370, 
           -1,   -1,   -1,  374,   -1,  376,  377,   -1,  379,  380, 
          381,  382,   -1,  384,  385,  386,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,  256,  257,  258,  259,  260,  261,   -1, 
           -1,  264,  265,  266,   -1,   -1,   -1,  270,   10,  272, 
          273,  274,  275,  276,  277,  278,   -1,   -1,   -1,   -1, 
          283,  284,  285,  286,  287,  288,  289,   -1,   -1,  292, 
           -1,   -1,   -1,   -1,   -1,  298,  299,   -1,  301,  302, 
          303,  304,   -1,  306,  307,  308,  309,  310,  311,   -1, 
          313,  314,  315,  316,   -1,   -1,   -1,   59,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,  337,   -1,   -1,  340,   -1,   -1, 
          343,  344,   -1,  346,   -1,  348,   -1,   -1,   -1,  352, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,  360,   -1,   -1, 
           -1,   -1,  365,  366,  367,  368,  369,  370,   -1,   -1, 
           -1,  374,   -1,  376,  377,   -1,  379,  380,  381,  382, 
           -1,  384,  385,  386,  256,  257,  258,  259,  260,  261, 
           -1,   -1,   -1,  265,  266,   -1,   -1,   -1,  270,   10, 
          272,  273,  274,  275,  276,  277,  278,   -1,   -1,   -1, 
           -1,  283,  284,  285,  286,  287,  288,  289,   -1,   -1, 
          292,   -1,   -1,   -1,   -1,   -1,  298,  299,   -1,  301, 
          302,  303,  304,   -1,  306,  307,  308,  309,  310,  311, 
           -1,  313,  314,  315,  316,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,  337,   -1,   -1,  340,   -1, 
           -1,  343,  344,   -1,  346,   -1,  348,   -1,   -1,   -1, 
          352,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  360,   -1, 
           -1,  363,   -1,  365,  366,  367,  368,  369,  370,   -1, 
           -1,   -1,  374,   -1,  376,  377,   -1,  379,  380,  381, 
          382,   -1,  384,  385,  386,  257,  258,  259,   -1,  261, 
           -1,   -1,   -1,  265,  266,   -1,   -1,   -1,  270,   10, 
          272,  273,  274,  275,  276,  277,  278,   -1,   -1,   -1, 
           -1,  283,  284,  285,  286,  287,  288,  289,   -1,   -1, 
          292,   -1,   -1,   -1,   -1,   -1,   -1,  299,   -1,   -1, 
          302,  303,  304,   -1,  306,  307,  308,  309,  310,  311, 
           -1,  313,  314,  315,  316,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,  337,   -1,   -1,  340,   -1, 
           -1,  343,  344,   -1,  346,   -1,   -1,   -1,   -1,   -1, 
          352,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  360,   -1, 
           -1,   -1,   -1,  365,  366,  367,  368,  369,  370,   -1, 
           -1,   -1,  374,   -1,  376,  377,   -1,  379,  380,  381, 
          382,   -1,   -1,  385,  386,   -1,  257,  258,  259,   -1, 
          261,   -1,   -1,   -1,  265,  266,   -1,   -1,   -1,  270, 
           -1,  272,  273,  274,  275,  276,  277,  278,   -1,   -1, 
           -1,   -1,  283,  284,  285,  286,  287,  288,  289,   -1, 
           -1,  292,   -1,   -1,   -1,   -1,   -1,   -1,  299,   -1, 
           -1,  302,  303,  304,   44,  306,  307,  308,  309,  310, 
          311,   -1,  313,  314,  315,  316,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,  337,   -1,   -1,  340, 
           -1,  342,  343,  344,   -1,  346,   -1,   -1,   -1,   -1, 
           -1,  352,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  360, 
           -1,   -1,   -1,   -1,  365,  366,  367,  368,  369,  370, 
           -1,   -1,   -1,  374,   -1,  376,  377,   -1,  379,  380, 
          381,  382,   -1,   -1,  385,  386,  257,  258,  259,   -1, 
          261,   -1,   -1,   -1,  265,  266,   -1,   -1,   -1,  270, 
           -1,  272,  273,  274,  275,  276,  277,  278,   -1,   -1, 
           -1,   -1,  283,  284,  285,  286, 
      };
   }

   private static final short[] yyCheck4() {
      return new short[] {

          287,  288,  289,   -1,   -1,  292,   -1,   -1,   -1,   -1, 
           -1,   -1,  299,   -1,   -1,  302,  303,  304,   44,  306, 
          307,  308,  309,  310,  311,   -1,  313,  314,  315,  316, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
          337,   -1,   -1,  340,   -1,   -1,  343,  344,   -1,  346, 
           -1,   -1,   -1,   -1,   -1,  352,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,  360,   -1,   -1,   -1,   -1,  365,  366, 
          367,  368,  369,  370,   -1,   -1,   -1,  374,   -1,  376, 
          377,   -1,  379,  380,  381,  382,   -1,   -1,  385,  386, 
           -1,  257,  258,  259,   -1,  261,   -1,   -1,   -1,  265, 
          266,   -1,   -1,   -1,  270,   -1,  272,  273,  274,  275, 
          276,  277,  278,   -1,   -1,   -1,   -1,  283,  284,  285, 
          286,  287,  288,  289,   -1,   -1,  292,   -1,   -1,   -1, 
           -1,   -1,   -1,  299,   -1,   -1,  302,  303,  304,   -1, 
          306,  307,  308,  309,  310,  311,   -1,  313,   -1,   -1, 
          316,   -1,   -1,   59,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,  337,   -1,   -1,  340,   -1,   -1,  343,  344,   -1, 
          346,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  365, 
          366,  367,  368,  369,  370,   -1,   -1,   -1,  374,   -1, 
          376,  377,   -1,  379,  380,  381,  382,   -1,   -1,  385, 
          386,  257,  258,  259,   -1,  261,   -1,   -1,   -1,  265, 
          266,   -1,   -1,   -1,  270,   -1,  272,  273,  274,  275, 
          276,  277,  278,   -1,   -1,   -1,   -1,  283,  284,  285, 
          286,  287,  288,  289,   -1,   -1,  292,   -1,   -1,   -1, 
           -1,   -1,   -1,  299,   -1,   -1,  302,  303,  304,   -1, 
          306,  307,  308,  309,  310,  311,   -1,  313,   -1,   -1, 
          316,   -1,   -1,   59,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,  337,   -1,   -1,  340,   -1,   -1,  343,  344,   -1, 
          346,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  365, 
          366,  367,  368,  369,  370,   -1,   -1,   -1,  374,   -1, 
          376,  377,   -1,  379,  380,  381,  382,   -1,   -1,  385, 
          386,  257,  258,  259,  260,  261,   -1,   -1,   -1,  265, 
          266,   -1,   -1,   -1,  270,   -1,  272,  273,  274,  275, 
          276,  277,  278,   -1,   -1,   -1,   -1,  283,  284,  285, 
          286,  287,  288,  289,   -1,   -1,  292,   -1,   -1,   -1, 
           -1,   -1,  298,  299,  300,  301,  302,  303,  304,   -1, 
          306,  307,  308,  309,  310,  311,   -1,  313,  314,  315, 
          316,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,  337,   -1,   -1,  340,   -1,   -1,  343,  344,   -1, 
          346,   -1,  348,   -1,   -1,   -1,  352,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,  360,   -1,   -1,   -1,   -1,  365, 
          366,  367,  368,  369,  370,   -1,   -1,   -1,  374,   -1, 
          376,  377,   -1,  379,  380,  381,  382,   -1,  384,  385, 
          386,  257,  258,  259,  260,  261,   -1,   -1,   -1,  265, 
          266,   -1,   -1,   -1,  270,   -1,  272,  273,  274,  275, 
          276,  277,  278,   -1,   -1,   -1,   -1,  283,  284,  285, 
          286,  287,  288,  289,   -1,   -1,  292,   -1,   -1,   -1, 
           -1,   -1,  298,  299,   -1,  301,  302,  303,  304,   -1, 
          306,  307,  308,  309,  310,  311,   -1,  313,  314,  315, 
          316,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,  337,   -1,   -1,  340,   -1,   -1,  343,  344,   -1, 
          346,   -1,  348,   -1,   -1,   -1,  352,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,  360,   -1,   -1,   -1,   -1,  365, 
          366,  367,  368,  369,  370,   -1,   -1,   -1,  374,   -1, 
          376,  377,   -1,  379,  380,  381,  382,   -1,  384,  385, 
          386,  257,  258,  259,  260,  261,  262,  263,  264,  265, 
          266,  267,  268,  269,  270,  271,  272,  273,  274,  275, 
          276,  277,  278,  279,  280,   -1,   -1,  283,  284,  285, 
          286,  287,  288,  289,  290,  291,  292,   -1,   -1,   -1, 
           -1,  297,  298,  299,  300,  301,  302,  303,  304,   -1, 
          306,  307,  308,  309,  310,  311,   -1,   -1,  314,  315, 
           -1,  317,  318,  319,  320,  321,  322,  323,   -1,   -1, 
          326,  327,   -1,   -1,   -1,  331,  332,  333,  334,   -1, 
           -1,   -1,   -1,   -1,   -1,  341,   -1,   -1,   -1,   -1, 
           -1,   -1,  348,  349,   -1,  351,  352,  353,  354,  355, 
          356,  357,  358,  359,  360,  361,   -1,   -1,  364,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,  387,  257,  258,  259,  260,  261,  262,  263,  264, 
          265,  266,  267,  268,  269,  270,  271,  272,  273,  274, 
          275,  276,  277,  278,  279,  280,   -1,   -1,  283,  284, 
          285,  286,  287,  288,  289,  290,  291,  292,   -1,   -1, 
           -1,   -1,  297,  298,  299,  300,  301,  302,  303,  304, 
           -1,  306,  307,  308,  309,  310,  311,   -1,   -1,  314, 
          315,   -1,  317,  318,  319,  320,  321,  322,  323,   -1, 
           -1,  326,  327,   -1,   -1,   -1,  331,  332,  333,  334, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,  348,  349,   -1,  351,  352,  353,  354, 
          355,  356,  357,  358,  359,  360,  361,   -1,   -1,  364, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,  387,  257,  258,  259,  260,  261,  262,  263, 
          264,  265,  266,  267,  268,  269,  270,  271,  272,  273, 
          274,  275,  276,  277,  278,  279,  280,   -1,   -1,  283, 
          284,  285,  286,  287,  288,  289,  290,  291,  292,   -1, 
           -1,   -1,   -1,  297,  298,  299,  300,  301,  302,  303, 
          304,   -1,  306,  307,  308,   -1,  310,   -1,   -1,   -1, 
          314,  315,   -1,  317,  318,  319,  320,  321,  322,  323, 
           -1,   -1,  326,  327,   -1,   -1,   -1,  331,  332,  333, 
          334,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,  348,  349,   -1,  351,  352,  353, 
          354,  355,  356,  357,  358,  359,  360,  361,   -1,   -1, 
          364,  365,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,  387,  257,  258,  259,  260,  261,  262, 
          263,  264,  265,  266,  267,  268,  269,  270,  271,  272, 
          273,  274,  275,  276,  277,  278,  279,  280,   -1,   -1, 
          283,  284,  285,  286,  287,  288,  289,  290,  291,  292, 
           -1,   -1,   -1,   -1,  297,  298,  299,  300,  301,  302, 
          303,  304,   -1,  306,  307,   -1,   -1,  310,   -1,   -1, 
           -1,  314,  315,   -1,  317,  318,  319,  320,  321,  322, 
          323,   -1,   -1,  326,  327,   -1,   -1,   -1,  331,  332, 
          333,  334,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,  348,  349,   -1,  351,  352, 
          353,  354,  355,  356,  357,  358,  359,  360,  361,   -1, 
           -1,  364,  365,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,  387,  257,  258,  259,  260,  261, 
          262,  263,  264,  265,  266,  267,  268,  269,  270,  271, 
          272,  273,  274,  275,  276,  277,  278,  279,  280,   -1, 
           -1,  283,  284,  285,  286,  287,  288,  289,  290,  291, 
          292,   -1,   -1,   -1,   -1,  297,  298,  299,  300,  301, 
          302,  303,  304,   -1,  306,  307,   -1,   -1,  310,   -1, 
           -1,   -1,  314,  315,   -1,  317,  318,  319,  320,  321, 
          322,  323,   -1,   -1,  326,  327,   -1,   -1,   -1,  331, 
          332,  333,  334,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,  348,  349,   -1,  351, 
          352,  353,  354,  355,  356,  357,  358,  359,  360,  361, 
           -1,   -1,  364,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
          256,  257,  258,  259,  260,  261,   -1,   -1,   -1,  265, 
          266,   -1,   -1,   -1,  270,  387,  272,  273,  274,  275, 
          276,  277,  278,   -1,   -1,   -1,   -1,  283,  284,  285, 
          286,  287,  288,  289,   -1,   -1,  292,   -1,   -1,   -1, 
           -1,   -1,  298,  299,  300,  301,  302,  303,  304,   -1, 
          306,  307,  308,  309,  310,  311,   -1,  313,  314,  315, 
          316,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,  337,   -1,   -1,  340,   -1,   -1,  343,  344,   -1, 
          346,   -1,  348,   -1,   -1,   -1,  352,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,  360,   -1,   -1,   -1,   -1,  365, 
          366,  367,  368,  369,  370,   -1,   -1,   -1,  374,   -1, 
          376,  377,   -1,  379,  380,  381,  382,   -1,  384,  385, 
          386,  256,  257,  258,  259,  260,  261,   -1,   -1,   -1, 
          265,  266,   -1,   -1,   -1,  270,   -1,  272,  273,  274, 
          275,  276,  277,  278,   -1,   -1,   -1,   -1,  283,  284, 
          285,  286,  287,  288,  289,   -1,   -1,  292,   -1,   -1, 
           -1,   -1,   -1,  298,  299,   -1,  301,  302,  303,  304, 
           -1,  306,  307,  308,  309,  310,  311,   -1,  313,  314, 
          315,  316,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,  337,   -1,   -1,  340,   -1,   -1,  343,  344, 
           -1,  346,   -1,  348,   -1,   -1,   -1,  352,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,  360,   -1,   -1,   -1,   -1, 
          365,  366,  367,  368,  369,  370,   -1,   -1,   -1,  374, 
           -1,  376,  377,   -1,  379,  380,  381,  382,   -1,  384, 
          385,  386,  256,  257,  258,  259,  260,  261,   -1,   -1, 
           -1,  265,  266,   -1,   -1,   -1,  270,   -1,  272,  273, 
          274,  275,  276,  277,  278,   -1,   -1,   -1,   -1,  283, 
          284,  285,  286,  287,  288,  289,   -1,   -1,  292,   -1, 
           -1,   -1,   -1,   -1,  298,  299,   -1,  301,  302,  303, 
          304,   -1,  306,  307,  308,  309,  310,  311,   -1,  313, 
          314,  315,  316,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,  337,   -1,   -1,  340,   -1,   -1,  343, 
          344,   -1,  346,   -1,  348,   -1,   -1,   -1,  352,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,  360,   -1,   -1,   -1, 
           -1,  365,  366,  367,  368,  369,  370,   -1,   -1,   -1, 
          374,   -1,  376,  377,   -1,  379,  380,  381,  382,   -1, 
          384,  385,  386,  257,  258,  259,   -1,  261,   -1,   -1, 
           -1,  265,  266,   -1,   -1,   -1,  270,   -1,  272,  273, 
          274,  275,  276,  277,  278,   -1,   -1,   -1,   -1,  283, 
          284,  285,  286,  287,  288,  289,   -1,   -1,  292,   -1, 
           -1,   -1,   -1,   -1,   -1,  299,   -1,   -1,  302,  303, 
          304,   -1,  306,  307,  308,  309,  310,  311,  312,  313, 
          314,  315,  316,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,  337,   -1,   -1,  340,   -1,   -1,  343, 
          344,   -1,  346,   -1,  348,   -1,  350,   -1,  352,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,  360,   -1,   -1,   -1, 
           -1,  365,  366,  367,  368,  369,  370,   -1,   -1,   -1, 
          374,   -1,  376,  377,   -1,  379,  380,  381,  382,   -1, 
           -1,  385,  386,  387,  257,  258,  259,   -1,  261,   -1, 
           -1,   -1,  265,  266,   -1,   -1,   -1,  270,   -1,  272, 
          273,  274,  275,  276,  277,  278,   -1,   -1,   -1,   -1, 
          283,  284,  285,  286,  287,  288,  289,   -1,   -1,  292, 
           -1,   -1,   -1,   -1,   -1,   -1,  299,   -1,   -1,  302, 
          303,  304,   -1,  306,  307,  308,  309,  310,  311,  312, 
          313,  314,  315,  316,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,  337,   -1,   -1,  340,   -1,   -1, 
          343,  344,   -1,  346,   -1,  348,   -1,  350,   -1,  352, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,  360,   -1,   -1, 
           -1,   -1,  365,  366,  367,  368,  369,  370,   -1,   -1, 
           -1,  374,   -1,  376,  377,   -1,  379,  380,  381,  382, 
           -1,   -1,  385,  386,  387,  257,  258,  259,   -1,  261, 
           -1,   -1,   -1,  265,  266,   -1,   -1,   -1,  270,   -1, 
          272,  273,  274,  275,  276,  277,  278,   -1,   -1,   -1, 
           -1,  283,  284,  285,  286,  287,  288,  289,   -1,   -1, 
          292,   -1,   -1,   -1,   -1,   -1,   -1,  299,   -1,   -1, 
          302,  303,  304,   -1,  306,  307,  308,  309,  310,  311, 
          312,  313,  314,  315,  316,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,  337,   -1,   -1,  340,   -1, 
           -1,  343,  344,   -1,  346,   -1,  348,   -1,  350,   -1, 
          352,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  360,   -1, 
           -1,   -1,   -1,  365,  366,  367,  368,  369,  370,   -1, 
           -1,   -1,  374,   -1,  376,  377,   -1,  379,  380,  381, 
          382,   -1,   -1,  385,  386,  387,  257,  258,  259,   -1, 
          261,   -1,   -1,   -1,  265,  266,   -1,   -1,   -1,  270, 
           -1,  272,  273,  274,  275,  276,  277,  278,   -1,   -1, 
           -1,   -1,  283,  284,  285,  286,  287,  288,  289,   -1, 
           -1,  292,   -1,   -1,   -1,   -1,   -1,   -1,  299,   -1, 
           -1,  302,  303,  304,   -1,  306,  307,  308,  309,  310, 
          311,  312,  313,  314,  315,  316,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,  337,   -1,   -1,  340, 
           -1,   -1,  343,  344,   -1,  346,   -1,  348,   -1,   -1, 
           -1,  352,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  360, 
           -1,   -1,   -1,   -1,  365,  366,  367,  368,  369,  370, 
           -1,   -1,   -1,  374,   -1,  376,  377,   -1,  379,  380, 
          381,  382,   -1,   -1,  385,  386,  387,  257,  258,  259, 
           -1,  261,   -1,   -1,   -1,  265,  266,   -1,   -1,   -1, 
          270,   -1,  272,  273,  274,  275,  276,  277,  278,   -1, 
           -1,   -1,   -1,  283,  284,  285,  286,  287,  288,  289, 
           -1,   -1,  292,   -1,   -1,   -1,   -1,   -1,   -1,  299, 
           -1,   -1,  302,  303,  304,   -1,  306,  307,  308,  309, 
          310,  311,  312,  313,  314,  315,  316,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,  337,   -1,   -1, 
          340,   -1,   -1,  343,  344,   -1,  346,   -1,   -1,   -1, 
          350,   -1,  352,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
          360,   -1,   -1,   -1,   -1,  365,  366,  367,  368,  369, 
          370,   -1,   -1,   -1,  374,   -1,  376,  377,   -1,  379, 
          380,  381,  382,   -1,   -1,  385,  386,  387,  257,  258, 
          259,   -1,  261,   -1,   -1,   -1,  265,  266,   -1,   -1, 
           -1,  270,   -1,  272,  273,  274,  275,  276,  277,  278, 
           -1,   -1,   -1,   -1,  283,  284,  285,  286,  287,  288, 
          289,   -1,   -1,  292,   -1,   -1,   -1,   -1,   -1,   -1, 
          299,   -1,   -1,  302,  303,  304,   -1,  306,  307,  308, 
          309,  310,  311,  312,  313,  314,  315,  316,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  337,   -1, 
           -1,  340,   -1,   -1,  343,  344,   -1,  346,   -1,  348, 
           -1,   -1,   -1,  352,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,  360,   -1,   -1,   -1,   -1,  365,  366,  367,  368, 
          369,  370,   -1,   -1,   -1,  374,   -1,  376,  377,   -1, 
          379,  380,  381,  382,   -1,   -1,  385,  386,  387,  257, 
          258,  259,   -1,  261,   -1,   -1,   -1,  265,  266,   -1, 
           -1,   -1,  270,   -1,  272,  273,  274,  275,  276,  277, 
          278,   -1,   -1,   -1,   -1,  283,  284,  285,  286,  287, 
          288,  289,   -1,   -1,  292,   -1,   -1,   -1,   -1,   -1, 
           -1,  299,   -1,   -1,  302,  303,  304,   -1,  306,  307, 
          308,  309,  310,  311,  312,  313,  314,  315,  316,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  337, 
           -1,   -1,  340,   -1,   -1,  343,  344,   -1,  346,   -1, 
           -1,   -1,   -1,   -1,  352,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,  360,   -1,   -1,   -1,   -1,  365,  366,  367, 
          368,  369,  370,   -1,   -1,   -1,  374,   -1,  376,  377, 
           -1,  379,  380,  381,  382,   -1,   -1,  385,  386,  387, 
          257,  258,  259,  260,  261,   -1,   -1,   -1,  265,  266, 
           -1,   -1,   -1,  270,   -1,  272,  273,  274,  275,  276, 
          277,  278,   -1,   -1,   -1,   -1,  283,  284,  285,  286, 
          287,  288,  289,   -1,   -1,  292,   -1,   -1,   -1,   -1, 
           -1,  298,  299,  300,  301,  302,  303,  304,   -1,  306, 
          307,  308,  309,  310,  311,   -1,  313,  314,  315,  316, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
          337,   -1,   -1,  340,   -1,   -1,  343,  344,   -1,  346, 
           -1,  348,   -1,   -1,   -1,  352,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,  360,   -1,   -1,   -1,   -1,  365,  366, 
          367,  368,  369,  370,   -1,   -1,   -1,  374,   -1,  376, 
          377,   -1,  379,  380,  381,  382,   -1,  384,  385,  386, 
          257,  258,  259,  260,  261,   -1,   -1,   -1,  265,  266, 
           -1,   -1,   -1,  270,   -1,  272,  273,  274,  275,  276, 
          277,  278,   -1,   -1,   -1,   -1,  283,  284,  285,  286, 
          287,  288,  289,   -1,   -1,  292,   -1,   -1,   -1,   -1, 
           -1,  298,  299,   -1,  301,  302,  303,  304,   -1,  306, 
          307,  308,  309,  310,  311,   -1,  313,  314,  315,  316, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
          337,   -1,   -1,  340,   -1,   -1,  343,  344,   -1,  346, 
           -1,  348,   -1,   -1,   -1,  352,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,  360,   -1,   -1,   -1,   -1,  365,  366, 
          367,  368,  369,  370,   -1,   -1,   -1,  374,   -1,  376, 
          377,   -1,  379,  380,  381,  382,   -1,  384,  385,  386, 
          257,  258,  259,   -1,  261,   -1,   -1,   -1,  265,  266, 
           -1,   -1,   -1,  270,   -1,  272,  273,  274,  275,  276, 
          277,  278,   -1,   -1,   -1,   -1,  283,  284,  285,  286, 
          287,  288,  289,   -1,   -1,  292,   -1,   -1,   -1,   -1, 
           -1,   -1,  299,   -1,   -1,  302,  303,  304,   -1,  306, 
          307,  308,  309,  310,  311,   -1,  313,  314,  315,  316, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
          337,   -1,   -1,  340,  341,   -1,  343,  344,   -1,  346, 
           -1,   -1,   -1,   -1,   -1,  352,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,  360,   -1,   -1,   -1,   -1,  365,  366, 
          367,  368,  369,  370,   -1,   -1,   -1,  374,   -1,  376, 
          377,   -1,  379,  380,  381,  382,   -1,   -1,  385,  386, 
          257,  258,  259,   -1,  261,   -1,   -1,   -1,  265,  266, 
           -1,   -1,   -1,  270,   -1,  272,  273,  274,  275,  276, 
          277,  278,   -1,   -1,   -1,   -1,  283,  284,  285,  286, 
          287,  288,  289,   -1,   -1,  292,   -1,   -1,   -1,   -1, 
           -1,   -1,  299,   -1,   -1,  302,  303,  304,   -1,  306, 
          307,  308,  309,  310,  311,   -1,  313,  314,  315,  316, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
          337,   -1,   -1,  340,   -1,   -1,  343,  344,   -1,  346, 
           -1,  348,   -1,   -1,   -1,  352,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,  360,   -1,   -1,   -1,   -1,  365,  366, 
          367,  368,  369,  370,   -1,   -1,   -1,  374,   -1,  376, 
          377,   -1,  379,  380,  381,  382,   -1,   -1,  385,  386, 
          257,  258,  259,   -1,  261,   -1,   -1,   -1,  265,  266, 
           -1,   -1,   -1,  270,   -1,  272,  273,  274,  275,  276, 
          277,  278,   -1,   -1,   -1,   -1,  283,  284,  285,  286, 
          287,  288,  289,   -1,   -1,  292,   -1,   -1,   -1,   -1, 
           -1,   -1,  299,   -1,   -1,  302,  303,  304,   -1,  306, 
          307,  308,  309,  310,  311,   -1,  313,  314,  315,  316, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
          337,   -1,   -1,  340,   -1,   -1,  343,  344,   -1,  346, 
           -1,  348,   -1,   -1,   -1,  352,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,  360,   -1,   -1,   -1,   -1,  365,  366, 
          367,  368,  369,  370,   -1,   -1,   -1,  374,   -1,  376, 
          377,   -1,  379,  380,  381,  382,   -1,   -1,  385,  386, 
          257,  258,  259,   -1,  261,   -1,   -1,   -1,  265,  266, 
           -1,   -1,   -1,  270,   -1,  272,  273,  274,  275,  276, 
          277,  278,   -1,   -1,   -1,   -1,  283,  284,  285,  286, 
          287,  288,  289,   -1,   -1,  292,   -1,   -1,   -1,   -1, 
           -1,   -1,  299,   -1,   -1,  302,  303,  304,   -1,  306, 
          307,  308,  309,  310,  311,   -1,  313,  314,  315,  316, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
          337,   -1,   -1,  340,   -1,   -1,  343,  344,   -1,  346, 
           -1,  348,   -1,   -1,   -1,  352,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,  360,   -1,   -1,   -1,   -1,  365,  366, 
          367,  368,  369,  370,   -1,   -1,   -1,  374,   -1,  376, 
          377,   -1,  379,  380,  381,  382,   -1,   -1,  385,  386, 
          257,  258,  259,   -1,  261,   -1,   -1,   -1,  265,  266, 
           -1,   -1,   -1,  270,   -1,  272,  273,  274,  275,  276, 
          277,  278,   -1,   -1,   -1,   -1,  283,  284,  285,  286, 
          287,  288,  289,   -1,   -1,  292,   -1,   -1,   -1,   -1, 
           -1,   -1,  299,   -1,   -1,  302,  303,  304,   -1,  306, 
          307,  308,  309,  310,  311,   -1,  313,  314,  315,  316, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
          337,   -1,   -1,  340,   -1,   -1,  343,  344,   -1,  346, 
           -1,  348,   -1,   -1,   -1,  352,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,  360,   -1,   -1,   -1,   -1,  365,  366, 
          367,  368,  369,  370,   -1,   -1,   -1,  374,   -1,  376, 
          377,   -1,  379,  380,  381,  382,   -1,   -1,  385,  386, 
          257,  258,  259,   -1,  261,   -1,   -1,   -1,  265,  266, 
           -1,   -1,   -1,  270,   -1,  272,  273,  274,  275,  276, 
          277,  278,   -1,   -1,   -1,   -1,  283,  284,  285,  286, 
          287,  288,  289,   -1,   -1,  292,   -1,   -1,   -1,   -1, 
           -1,   -1,  299,   -1,   -1,  302,  303,  304,   -1,  306, 
          307,  308,  309,  310,  311,   -1,  313,  314,  315,  316, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
          337,   -1,   -1,  340,   -1,   -1,  343,  344,   -1,  346, 
           -1,  348,   -1,   -1,   -1,  352,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,  360,   -1,   -1,   -1,   -1,  365,  366, 
          367,  368,  369,  370,   -1,   -1,   -1,  374,   -1,  376, 
          377,   -1,  379,  380,  381,  382,   -1,   -1,  385,  386, 
          257,  258,  259,   -1,  261,   -1,   -1,   -1,  265,  266, 
           -1,   -1,   -1,  270,   -1,  272,  273,  274,  275,  276, 
          277,  278,   -1,   -1,   -1,   -1,  283,  284,  285,  286, 
          287,  288,  289,   -1,   -1,  292,   -1,   -1,   -1,   -1, 
           -1,   -1,  299,   -1,   -1,  302,  303,  304,   -1,  306, 
          307,  308,  309,  310,  311,   -1,  313,  314,  315,  316, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
          337,   -1,   -1,  340,  341,   -1,  343,  344,   -1,  346, 
           -1,   -1,   -1,   -1,   -1,  352,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,  360,   -1,   -1,   -1,   -1,  365,  366, 
          367,  368,  369,  370,   -1,   -1,   -1,  374,   -1,  376, 
          377,   -1,  379,  380,  381,  382,   -1,   -1,  385,  386, 
          257,  258,  259,   -1,  261,   -1,   -1,   -1,  265,  266, 
           -1,   -1,   -1,  270,   -1,  272,  273,  274,  275,  276, 
          277,  278,   -1,   -1,   -1,   -1,  283,  284,  285,  286, 
          287,  288,  289,   -1,   -1,  292,   -1,   -1,   -1,   -1, 
           -1,   -1,  299,   -1,   -1,  302,  303,  304,   -1,  306, 
          307,  308,  309,  310,  311,   -1,  313,  314,  315,  316, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
          337,   -1,   -1,  340,   -1,   -1,  343,  344,   -1,  346, 
           -1,   -1,   -1,   -1,   -1,  352,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,  360,   -1,   -1,   -1,   -1,  365,  366, 
          367,  368,  369,  370,   -1,   -1,   -1,  374,   -1,  376, 
          377,   -1,  379,  380,  381,  382,   -1,   -1,  385,  386, 
          257,  258,  259,   -1,  261,   -1,   -1,   -1,  265,  266, 
           -1,   -1,   -1,  270,   -1,  272,  273,  274,  275,  276, 
          277,  278,   -1,   -1,   -1,   -1,  283,  284,  285,  286, 
          287,  288,  289,   -1,   -1,  292,   -1,   -1,   -1,   -1, 
           -1,   -1,  299,   -1,   -1,  302,  303,  304,   -1,  306, 
          307,  308,  309,  310,  311,   -1,  313,  314,  315,  316, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
          337,   -1,   -1,  340,   -1,   -1,  343,  344,   -1,  346, 
           -1,   -1,   -1,   -1,   -1,  352,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,  360,   -1,   -1,   -1,   -1,  365,  366, 
          367,  368,  369,  370,   -1,   -1,   -1,  374,   -1,  376, 
          377,   -1,  379,  380,  381,  382,   -1,   -1,  385,  386, 
          257,  258,  259,   -1,  261,   -1,   -1,   -1,  265,  266, 
           -1,   -1,   -1,  270,   -1,  272,  273,  274,  275,  276, 
          277,  278,   -1,   -1,   -1,   -1,  283,  284,  285,  286, 
          287,  288,  289,   -1,   -1,  292,   -1,   -1,   -1,   -1, 
           -1,   -1,  299,   -1,   -1,  302,  303,  304,   -1,  306, 
          307,  308,  309,  310,  311,   -1,  313,  314,  315,  316, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
          337,   -1,   -1,  340,   -1,   -1,  343,  344,   -1,  346, 
           -1,   -1,   -1,   -1,   -1,  352,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,  360,   -1,   -1,   -1,   -1,  365,  366, 
          367,  368,  369,  370,   -1,   -1,   -1,  374,   -1,  376, 
          377,   -1,  379,  380,  381,  382,   -1,   -1,  385,  386, 
          257,  258,  259,   -1,  261,   -1,   -1,   -1,  265,  266, 
           -1,   -1,   -1,  270,   -1,  272,  273,  274,  275,  276, 
          277,  278,   -1,   -1,   -1,   -1,  283,  284,  285,  286, 
          287,  288,  289,   -1,   -1,  292,   -1,   -1,   -1,   -1, 
           -1,   -1,  299,   -1,   -1,  302,  303,  304,   -1,  306, 
          307,  308,  309,  310,  311,   -1,  313,  314,  315,  316, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
          337,   -1,   -1,  340,   -1,   -1,  343,  344,   -1,  346, 
           -1,   -1,   -1,   -1,   -1,  352,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,  360,   -1,   -1,   -1,   -1,  365,  366, 
          367,  368,  369,  370,   -1,   -1,   -1,  374,   -1,  376, 
          377,   -1,  379,  380,  381,  382,   -1,   -1,  385,  386, 
          257,  258,  259,   -1,  261,   -1,   -1,   -1,  265,  266, 
           -1,   -1,   -1,  270,   -1,  272,  273,  274,  275,  276, 
          277,  278,   -1,   -1,   -1,   -1,  283,  284,  285,  286, 
          287,  288,  289,   -1,   -1,  292,   -1,   -1,   -1,   -1, 
           -1,   -1,  299,   -1,   -1,  302,  303,  304,   -1,  306, 
          307,  308,  309,  310,  311,   -1,  313,   -1,   -1,  316, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,  333,   -1,   -1,   -1, 
          337,   -1,   -1,  340,   -1,   -1,  343,  344,   -1,  346, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  365,  366, 
          367,  368,  369,  370,   -1,   -1,   -1,  374,   -1,  376, 
          377,   -1,  379,  380,  381,  382,   -1,   -1,  385,  386, 
          257,  258,  259,   -1,  261,   -1,   -1,   -1,  265,  266, 
           -1,   -1,   -1,  270,   -1,  272,  273,  274,  275,  276, 
          277,  278,   -1,   -1,   -1,   -1,  283,  284,  285,  286, 
          287,  288,  289,   -1,   -1,  292,   -1,   -1,   -1,   -1, 
           -1,   -1,  299,   -1,   -1,  302,  303,  304,   -1,  306, 
          307,  308,  309,  310,  311,   -1,  313,   -1,   -1,  316, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
          337,   -1,   -1,  340,   -1,   -1,  343,  344,   -1,  346, 
           -1,  348,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  365,  366, 
          367,  368,  369,  370,   -1,   -1,   -1,  374,   -1,  376, 
          377,   -1,  379,  380,  381,  382,   -1,   -1,  385,  386, 
          257,  258,  259,   -1,  261,   -1,   -1,   -1,  265,  266, 
           -1,   -1,   -1,  270,   -1,  272,  273,  274,  275,  276, 
          277,  278,   -1,   -1,   -1,   -1,  283,  284,  285,  286, 
          287,  288,  289,   -1,   -1,  292,   -1,   -1,   -1,   -1, 
           -1,   -1,  299,   -1,   -1,  302,  303,  304,   -1,  306, 
          307,  308,  309,  310,  311,   -1,  313,   -1,   -1,  316, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
          337,   -1,   -1,  340,   -1,   -1,  343,  344,   -1,  346, 
           -1,  348,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  365,  366, 
          367,  368,  369,  370,   -1,   -1,   -1,  374,   -1,  376, 
          377,   -1,  379,  380,  381,  382,   -1,   -1,  385,  386, 
          257,  258,  259,   -1,  261,   -1,   -1,   -1,  265,  266, 
           -1,   -1,   -1,  270,   -1,  272,  273,  274,  275,  276, 
          277,  278,   -1,   -1,   -1,   -1,  283,  284,  285,  286, 
          287,  288,  289,   -1,   -1,  292,   -1,   -1,   -1,   -1, 
           -1,   -1,  299,   -1,   -1,  302,  303,  304,   -1,  306, 
          307,  308,  309,  310,  311,   -1,  313,   -1,   -1,  316, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
          337,   -1,   -1,  340,   -1,   -1,  343,  344,   -1,  346, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  365,  366, 
          367,  368,  369,  370,   -1,   -1,   -1,  374,   -1,  376, 
          377,   -1,  379,  380,  381,  382,   -1,   -1,  385,  386, 
          257,  258,  259,   -1,  261,   -1,   -1,   -1,  265,  266, 
           -1,   -1,   -1,  270,   -1,  272,  273,  274,  275,  276, 
          277,  278,   -1,   -1,   -1,   -1,  283,  284,  285,  286, 
          287,  288,  289,   -1,   -1,  292,   -1,   -1,   -1,   -1, 
           -1,   -1,  299,   -1,   -1,  302,  303,  304,   -1,  306, 
          307,  308,  309,  310,  311,   -1,  313,   -1,   -1,  316, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
          337,   -1,   -1,  340,   -1,   -1,  343,  344,   -1,  346, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  365,  366, 
          367,  368,  369,  370,   -1,   -1,   -1,  374,   -1,  376, 
          377,   -1,  379,  380,  381,  382,   -1,   -1,  385,  386, 
          257,  258,  259,   -1,  261,   -1,   -1,   -1,  265,  266, 
           -1,   -1,   -1,  270,   -1,  272,  273,  274,  275,  276, 
          277,  278,   -1,   -1,   -1,   -1,  283,  284,  285,  286, 
          287,  288,  289,   -1,   -1,  292,   -1,   -1,   -1,   -1, 
           -1,   -1,  299,   -1,   -1,  302,  303,  304,   -1,  306, 
          307,  308,  309,  310,  311,   -1,  313,   -1,   -1,  316, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
          337,   -1,   -1,  340,   -1,   -1,  343,  344,   -1,  346, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  365,  366, 
          367,  368,  369,  370,   -1,   -1,   -1,  374,   -1,  376, 
          377,   -1,  379,  380,  381,  382,   -1,   -1,  385,  386, 
          257,  258,  259,   -1,  261,   -1,   -1,   -1,  265,  266, 
           -1,   -1,   -1,  270,   -1,  272,  273,  274,  275,  276, 
          277,  278,   -1,   -1,   -1,   -1,  283,  284,  285,  286, 
          287,  288,  289,   -1,   -1,  292,   -1,   -1,   -1,   -1, 
           -1,   -1,  299,   -1,   -1,  302,  303,  304,   -1,  306, 
          307,  308,  309,  310,  311,   -1,  313,   -1,   -1,  316, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
          337,   -1,   -1,  340,   -1,   -1,  343,  344,   -1,  346, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  365,  366, 
          367,  368,  369,  370,   -1,   -1,   -1,  374,   -1,  376, 
          377,   -1,  379,  380,  381,  382,  306,  307,  385,  386, 
          310,   -1,   -1,   -1,  314,  315,   -1,  317,  318,  319, 
          320,  321,  322,  323,   -1,   -1,  326,  327,   -1,   -1, 
           -1,  331,  332,  333,  334,   -1,   -1,   -1,   -1,   -1, 
           -1,  341,   -1,   -1,   -1,   -1,   -1,   -1,  348,  349, 
           -1,  351,  352,  353,  354,  355,  356,  357,  358,  359, 
          360,  361,   -1,   -1,  364,  306,  307,   -1,   -1,  310, 
           -1,   -1,   -1,  314,  315,   -1,  317,  318,  319,  320, 
          321,  322,  323,   -1,   -1,  326,  327,  387,   -1,   -1, 
          331,  332,  333,  334,   -1,   -1,   -1,   -1,   -1,   -1, 
          341,   -1,   -1,   -1,   -1,   -1,   -1,  348,  349,   -1, 
          351,  352,  353,  354,  355,  356,  357,  358,  359,  360, 
          361,   -1,   -1,  364,  306,  307,   -1,   -1,  310,   -1, 
           -1,   -1,  314,  315,   -1,  317,  318,  319,  320,  321, 
          322,  323,   -1,   -1,  326,  327,  387,   -1,   -1,  331, 
          332,  333,  334,   -1,   -1,   -1,   -1,   -1,   -1,  341, 
           -1,   -1,   -1,   -1,   -1,   -1,  348,  349,   -1,  351, 
          352,  353,  354,  355,  356,  357,  358,  359,  360,  361, 
           -1,   -1,  364,  306,  307,   -1,   -1,  310,   -1,   -1, 
           -1,  314,  315,   -1,  317,  318,  319,  320,  321,  322, 
          323,   -1,   -1,  326,  327,  387,   -1,   -1,  331,  332, 
          333,  334,   -1,   -1,   -1,   -1,   -1,   -1,  341,   -1, 
           -1,   -1,   -1,   -1,   -1,  348,  349,   -1,  351,  352, 
          353,  354,  355,  356,  357,  358,  359,  360,  361,   -1, 
           -1,  364,  306,  307,   -1,   -1,  310,   -1,   -1,   -1, 
          314,  315,   -1,  317,  318,  319,  320,  321,  322,  323, 
           -1,   -1,  326,  327,  387,   -1,   -1,  331,  332,  333, 
          334,   -1,   -1,   -1,   -1,   -1,   -1,  341,   -1,   -1, 
           -1,   -1,   -1,   -1,  348,  349,   -1,  351,  352,  353, 
          354,  355,  356,  357,  358,  359,  360,  361,   -1,   -1, 
          364,  306,  307,   -1,   -1,  310,   -1,   -1,   -1,  314, 
          315,   -1,  317,  318,  319,  320,  321,  322,  323,   -1, 
           -1,  326,  327,  387,   -1,   -1,  331,  332,  333,  334, 
           -1,   -1,   -1,   -1,   -1,   -1,  341,   -1,   -1,   -1, 
           -1,   -1,   -1,  348,  349,   -1,  351,  352,  353,  354, 
          355,  356,  357,  358,  359,  360,  361,   -1,   -1,  364, 
          306,  307,   -1,   -1,  310,   -1,   -1,   -1,  314,  315, 
           -1,  317,  318,  319,  320,  321,  322,  323,   -1,   -1, 
          326,  327,  387,   -1,   -1,  331,  332,  333,  334,   -1, 
           -1,   -1,   -1,   -1,   -1,  341,   -1,   -1,   -1,   -1, 
           -1,   -1,  348,  349,   -1,  351,  352,  353,  354,  355, 
          356,  357,  358,  359,  360,  361,   -1,   -1,  364,  306, 
          307,   -1,   -1,  310,   -1,   -1,   -1,  314,  315,   -1, 
          317,  318,  319,  320,  321,  322,  323,   -1,   -1,  326, 
          327,  387,   -1,   -1,  331,  332,  333,  334,   -1,   -1, 
           -1,   -1,   -1,   -1,  341,   -1,   -1,   -1,   -1,   -1, 
           -1,  348,  349,   -1,  351,  352,  353,  354,  355,  356, 
          357,  358,  359,  360,  361,   -1,   -1,  364,  306,  307, 
           -1,   -1,  310,   -1,   -1,   -1,  314,  315,   -1,  317, 
          318,  319,  320,  321,  322,  323,   -1,   -1,  326,  327, 
          387,   -1,   -1,  331,  332,  333,  334,   -1,   -1,   -1, 
           -1,   -1,   -1,  341,   -1,   -1,   -1,   -1,   -1,   -1, 
          348,  349,   -1,  351,  352,  353,  354,  355,  356,  357, 
          358,  359,  360,  361,   -1,   -1,  364,  306,  307,   -1, 
           -1,  310,   -1,   -1,   -1,  314,  315,   -1,  317,  318, 
          319,  320,  321,  322,  323,   -1,   -1,  326,  327,  387, 
           -1,   -1,  331,  332,  333,  334,   -1,   -1,   -1,   -1, 
           -1,   -1,  341,   -1,   -1,   -1,   -1,   -1,   -1,  348, 
          349,   -1,  351,  352,  353,  354,  355,  356,  357,  358, 
          359,  360,  361,   -1,   -1,  364,  306,  307,   -1,   -1, 
          310,   -1,   -1,   -1,  314,  315,   -1,  317,  318,  319, 
          320,  321,  322,  323,   -1,   -1,  326,  327,  387,   -1, 
           -1,  331,  332,  333,  334,   -1,   -1,   -1,   -1,   -1, 
           -1,  341,   -1,   -1,   -1,   -1,   -1,   -1,  348,  349, 
           -1,  351,  352,  353,  354,  355,  356,  357,  358,  359, 
          360,  361,   -1,   -1,  364,  306,  307,   -1,   -1,  310, 
           -1,   -1,   -1,  314,  315,   -1,  317,  318,  319,  320, 
          321,  322,  323,   -1,   -1,  326,  327,  387,   -1,   -1, 
          331,  332,  333,  334,   -1,   -1,   -1,   -1,   -1,   -1, 
          341,   -1,   -1,   -1,   -1,   -1,   -1,  348,  349,   -1, 
          351,  352,  353,  354,  355,  356,  357,  358,  359,  360, 
          361,   -1,   -1,  364,  306,  307,   -1,   -1,  310,   -1, 
           -1,   -1,  314,  315,   -1,  317,  318,  319,  320,  321, 
          322,  323,   -1,   -1,  326,  327,  387,   -1,   -1,  331, 
          332,  333,  334,   -1,   -1,   -1,   -1,   -1,   -1,  341, 
           -1,   -1,   -1,   -1,   -1,   -1,  348,  349,   -1,  351, 
          352,  353,  354,  355,  356,  357,  358,  359,  360,  361, 
           -1,   -1,  364,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,  387, 
      };
   }

}
